/*******************************************************************************
**
** The source file for the Jahshaka Core motionpath module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "coreobject.h"

motionPath::motionPath(void) {

  layerStatus  = 0;    keyStatus   = 0;
  foreground = 0;
  selected   = FALSE;
  drawKeyframes = FALSE;
  //add these to the save/load routines
  blend      = 1;        	
  blendmode1 = GL_DST_COLOR;
  blendmode2 = GL_ZERO;
  texturetype= GL_RGBA;
  	
  qtFlag     = TRUE;
  depth      = 1;
  cull       = 0;       lighting     = 0;
  width      =          height       = 0;
  layerres   = 1024;

  //Set default color
  Red = Green = Blue = 1.0f;

  //true means its textured, false is wireframe
  ObjectDisplayMode = TRUE;

  //layer mesh for effects
  MeshFirsttime = FALSE;

  foreground    = FALSE;   keyStatus    = FALSE;
  videotex      = FALSE;   invertKey    = FALSE;
  videotexkey   = FALSE;
  blend         = FALSE;   depth        = FALSE;

  //we can remove this later on
  videoext,videoextkey ="";

  //set this to blank so it will always be false the first time
  lastframeused = "";
  lastkeyframeused = "";

  //3d obj object data defaults
  model_list      = 0;	             // display list for object
  smoothing_angle = 90.0;	             // smoothing angle
  model_file      = NULL;              // name of the obect file
  material_mode   = 0;
  facet_normal    = GL_FALSE;

  //set up particle defaults
  ParticleDraw  = PARLINE;
  ParticleColors = FALSE;

  //set up the nodes for keyframes
  //nodes = new motionNode[10];

  numnodes   = 0;
  node            = new motionNode;    // node is always equal to the current node

  clear();                  // change to clear(node) ?;

  //addNodes( 0, node);       // create first 2 keyframes so arrays dont crash system
  //addNodes( 0, node);       // this needs to be fixed? check out others

  //this should be a global computed in the render object
  //and updated when the reolution or screensize change
  widthOffset  = (GLfloat)X_RESOLUTION/2;
  heightOffset = (GLfloat)Y_RESOLUTION/2;


}

motionPath::~motionPath(void) {

    for (int i=0; i<numnodes; i++) {
      delete nodes[i];
    }

    delete node;

}

void motionPath::printInfo(void) {

	//routine to print out all info on object
	if (JAHDEBUG) {

		QString data;

		debug("Printing out the layer data");
		debug("---------------------------");
		debug( "LayerType   :" + data.setNum(layertype   ) );
		debug( "ObjectType  :" + data.setNum(objtype     ) );
		debug( "Status      :" + data.setNum(layerStatus ) );
		debug( "Selected    :" + data.setNum(selected    ) );
		debug( " " );
		debug( "VideoTex    :" + data.setNum(videotex    ) );
		debug( "Key         :" + data.setNum(keyStatus   ) );
		debug( "DrawKey     :" + data.setNum(drawtheKey  ) );
		debug( "InvertKey   :" + data.setNum(invertKey   ) );
		debug( "VideoTexKey :" + data.setNum(videotexkey ) );
		debug( "StartFrame  :" + data.setNum(startframe  ) );
		debug( "EndFrame    :" + data.setNum(endframe    ) );
		debug( " " );
		debug( "Blending    :" + data.setNum(blend       ) );
		debug( "Depth       :" + data.setNum(depth       ) );
		debug( "Lighting    :" + data.setNum(lighting    ) );
		debug("---------------------------");
	
	}

}

void motionPath::addTextObj() {

     text = new textObj;
     text->fontshere = false;
     text->fontstyle = FPOLYGON;

}



void motionPath::changeKeyStatus() {

     invertKey = !(invertKey);
      textureKey.objectImage.invertPixels();

	  //LoadLayerImage(KEY);

	  //use this code instead of LoadLayerImage to save on the file io

	  textureKey =  buildGlTexture ( textureKey.objectImage );
      InitializeTexture(textureKey);  //486; }

      makeTexture();

}

void motionPath::getColor() {
    QColor newcolor;

    QColor c = QColorDialog::getColor( newcolor, 0, 0 );

    QRgb thecolor=c.rgb();

    int r=qRed(thecolor);   if (r==0)  Red = 0.0;    else {  Red   = (r/255.0); }
    int g=qGreen(thecolor); if (g==0)  Green = 0.0;  else {  Green = (g/255.0);  }
    int b=qBlue(thecolor);  if (b==0)  Blue  = 0.0;  else {  Blue  = (b/255.0);  }

}







