/*******************************************************************************
**
** The header file for the Jahshaka Core module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef COREOBJECT_H
#define COREOBJECT_H

#include <coreheaders.h>
#include <corestructures.h>
#include <videodata.h>
#include <p_vector.h>

extern QString JahBasePath;
extern bool JAHDEBUG;

////////////////////////////////////////////////////////////
//windows specific

#ifdef WIN32
#pragma warning (disable:4305) /* disable bogus conversion warnings */
#define drand48() (((float) rand())/((float) RAND_MAX))
#define lrand48() ((rand() << 16) ^ rand())
#define srand48(x) srand(x)
#endif

/*
#ifdef WIN32
#include <vfw.h>
#endif
*/
////////////////////////////////////////////////////////////
//opengl specific

#ifndef GL_ASSERT
#define GL_ASSERT() {GLenum sci_err; while ((sci_err = glGetError()) != GL_NO_ERROR) \
 << "OpenGL error: " << (char *)gluErrorString(sci_err) << " at " << __FILE__ <<":" << __LINE__ << endl;}
#endif


////////////////////////////////////////////////////////////
// the main layer object class

class motionPath {

public:
  motionPath(void);
  ~motionPath(void);

  //node routines
  void printInfo(void);
  void addNodes(int currentFrame, motionNode * newNode);
  void sortNodes(void);
  void swap(int &x, int &y);
  void updateNodes(int currentFrame);  //gets interpolation
  void getFirstLastFrame( int *firstFrame, int *lastFrame);
  int  getNode(int frameNumber, motionNode *mn);
  void clear(void);

  //jahdraw calls here
  void positionObject();
  void drawLayer(void);

  //jahobject object creation here
  void makeCylinder( void );
  void makeSphere( void );
  void makeLayer( void );

  void initializeMeshLayer( void);
  void setMeshCoord( int i, int j, GLfloat x, GLfloat y);
  void makeMeshLayer( void );

  void makeKeyLayer( void );

  void makeLayerRotate( void);
  void makeLayerTranslate( void);
  void makeLayerScale( void);


  void makeBackground( void );
  void makeBlackBackground( void );
  void makeBackdropOutline( void );
  void makeCube( void );

  void makeKeyFrames( void );
  void addTextObj();
  void drawText();
  void draw3dTextOutline();
  void makeText(void );
  bool loadFontText(void);

  void makeGrid( void );
  void makeParticle();
  void makeBezier(void);
  void makeTitleGrid( void );

  void getColor(void);       //gets object color

  //jahmodel i/o routines
  bool loadOBJ(const QString filename);
  void loadObjObject(void);
  bool loadMaxOBJ(const QString filename);
  void loadMaxObject(void);

  void makeObj(void);
  void makeMaxObj(void);


  //particle creation here
  void addParticleObj();
  void ComputeParticles();

  void Atom(bool first_time);
  void JetSpray(bool first_time);
  void Waterfall(bool first_time);
  void Balloons(bool first_time);
  void Rocket(bool first_time);
  void Shape(bool first_time);
  void FireFlies(bool first_time) ;
  void Shower(bool first_time);
  void Swirl(bool first_time);
  void Explosion(bool first_time);
  void Fountain(bool first_time);

  void DrawGroupQuadSplat(const pVector &, const pVector &,
  				float size_scale, bool draw_tex,
  				bool const_size, bool const_color);

  void DrawGroupTriSplat(const pVector &view, const pVector &up,
  				float size_scale, bool draw_tex,
  				bool const_size, bool const_color);

  //jahtexture texture calls here
  void makeTexture( void );
  void updateTexture( void );
  void makeLayerTexture( void );
  void makeKeyTexture( void );
  void changeKeyStatus(void );
  void rebuildTextureData (  QImage & buf  );
  void rebuildTextureKey (  QImage & buf  );

  void updateVidTex( int theframe);

  QString getsequencefilename( int frame );
  QString getkeysequencefilename( int frame );

  void updateLayerTexture( void );
  void setLayerTexture(void);
  void setKeyTexture(void);

  void updateKeyTexture( void );

  void reset( void );
  inline float Gaussian2(float x, float y, float sigma);
  void MakeGaussianSpotTexture();

  //jahfile i/o routines
  bool loadintoimage(QString filename, QImage & image);

  bool LoadGlImage( const QString filename, VideoCategory imagetype = CLIP );
  bool LoadLayerImage( VideoCategory imagetype = CLIP );
  bool LoadImage( const QString filename );
  bool LoadKey( const QString filename );
  void loadKey(void );
  void loadClip(void);
  TextureImage buildGlTexture (  QImage & buf  );      //was & buf
  void InitializeTexture(TextureImage & textured);


  ////////////////////////////////////////////////////////
  // variable declarations here

  TextureImage textureData;
  TextureImage textureKey;

  // do we draw as wireframe or as textured?
  bool ObjectDisplayMode;

  //can we use pointers here to free up memory?
  GLfloat MeshXCoord [74][50]; // should be 72,48 for 720 486
  GLfloat MeshYCoord [74][50];
  bool MeshFirsttime;

  QString     layername;
  int         layernumber;
  ObjectCategory    objtype;   //change tis to objectcategory
  LayerCategory    layertype;
  bool        layer,layerStatus,selected,qtFlag,keyStatus,foreground,invertKey;

  bool        drawtheLayer, drawtheKey;
  bool        drawKeyframes;
  GLint       object,key;
  QImage      keyimage;
  GLuint      texture,keytexture,texturetype; //=GL_RGBA;
  bool        videotex, videotexkey;
  QString		  videoext, videoextkey; //used for sequences
  GLubyte*    imageData;
  GLfloat     texwidth1,texheight1; //image
  GLfloat     texwidth2,texheight2; //key
  GLfloat     srcwidth,srcheight,width,height,keywidth,keyheight;
  GLfloat     layerres, keyres;
  bool        blend,depth,cull,lighting;
  GLenum      blendmode1, blendmode2;

  //color and transparency
  GLfloat     Red, Green, Blue;

  //for video layers
  QString	   	lastframeused, lastkeyframeused;
  int         startframe, endframe, currentframe;//, total_frames;
  int			    inframe, outframe, keyinframe, keyoutframe;
  int		    	numframes;
  int         keystartframe, keyendframe, keycurrentframe;//, keytotal_frames;
  int		    	numkeyframes;
  QString     filename,keyfilename,extension,keyextension;
  QString     pathname,keypathname;
  QString     objfilename,objextension;

  int         numnodes;
  motionNode *node;
  motionNode *nodes[99];
  //motionNode *nodes;
  textObj    *text;

  //for objects
  GLuint     model_list;		        /* display list for object */
  GLMmodel*  model;			        /* glm model data structure */
  GLfloat    scale;			        /* original scale factor */
  GLfloat    smoothing_angle;      	/* smoothing angle */
  char*      model_file;		        /* name of the obect file */
  GLuint     material_mode;
  GLboolean  facet_normal;

  //for maxobjects
  L3DS*      scene;

  //particles objects
  int particle_handle, action_handle, maxParticles;
  int demoNum,         numSteps,      prim;
  int listID, SpotTexID;
  float BlurRate;
  int   SteerShape;

  ParticleCategory   ParticleStyle;
  ParticleDrawstyle  ParticleDraw;
  bool               ParticleFirsttime;
  bool ParticleColors;

  //resolution based variables
  //should be in a subroutine later to be called whenever this data changes
  GLfloat widthOffset;
  GLfloat heightOffset;

};

#endif











































