/*******************************************************************************
**
** The Jahshaka global interface headers file
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

// define all qt libraries used in interface creation

#ifndef CORESTRUCTURES_H
#define CORESTRUCTURES_H

extern int X_RESOLUTION;
extern int Y_RESOLUTION;

//FTGL headers
#include <FTGLOutlineFont.h>
#include <FTGLPolygonFont.h>
#include <FTGLBitmapFont.h>
#include <FTGLTextureFont.h>
#include <FTGLPixmapFont.h>
#include "FTGLExtrdFont.h"


////////////////////////////////////////////////////////
//enum variables here

enum LayerCategory     { LAYER, PARTICLE, OBJECTS, FRAME, TEXT, GRID, AXIS, LIGHT };

enum ObjectCategory    { JAHLAYER, MESHLAYER, MAINBACK, BACKDROP,
                         BEZIERS, OBJOBJECTS, MAXOBJECTS, CUBE, SPHERE, CYLINDER,
                         JAHTEXT, TITLEGRID, JAHPARTICLE, JAHAXIS, JAHLIGHT };
/*was Category*/

enum ParticleCategory  { ATOMIC, JETSPRAY, WATERFALL, ROCKET, EXPLOSION, SWIRL,
                         BALLOONS, SHAPE, FIREFLIES, SHOWER, FOUNTAIN };

enum ParticleDrawstyle { PARLINE, PARPOINT, PARGAUSS, PARTEXTURE };

enum FontDrawstyle     { FBITMAP, FPIXMAP, FOUTLINE, FPOLYGON, FEXTRUDE, FTEXTURE };

enum FxCategory        { TRANSLATE, NOFX, NEGATIVE, GREY, EDGE, FISHEYE, SWIRLRT, CONTRAST, POLAR,
                         FLARES, BLUR, SHARPEN, COLORMATRIX, MIRROR, RADIALINES, SWAPRGB };

/////////////////////////////////////////////////////////
//core data structures

struct TextureImage {

    QImage       objectImage;                            // Image Data (Up To 32 Bits)
    GLuint       bpp;                                    // Image Color Depth In Bits Per Pixel.
    GLfloat      wresolution;                            // resolution for texture map
    GLfloat      hresolution;                            // resolution for texture map
    GLfloat      width;                                 // Image Width
    GLfloat      height;                                // Image Height
    GLfloat      x;                                     // 3dspace x cord
    GLfloat      y;                                     // 3dspace y cord

    QImage      texture;                                 // the opengl texture
    GLfloat      texwidth;                               // texture Width for map
    GLfloat      texheight;                              // texture Height for map
    GLuint      texID;                                   // Texture ID Used To Select A Texture

    GLfloat      mapwidth;                                 // Image mapping width
    GLfloat      mapheight;                                // Image mapping height


};

struct textObj {

    QString     text;
    QString     font;
    double      extrude;
    int         size;
    QString     getText() { return text; };
    FontDrawstyle fontstyle;

	//for freetype fonts

	FTFont* fonts[6];
  bool fontshere;
	int width;
	int height;

};

struct motionNode {

    int         nodenum;                                   // this is the keyframe
    GLfloat     tx, ty, tz;
    GLfloat     rx, ry, rz;
    GLfloat     sx, sy, sz, sc;
    GLfloat     Alpha;
    //int read(char *);
    void copy(motionNode*);
    void clear(void);
};



////////////////////////////////////////////////////////////
//the effect layer class

class effectLayer {

public:
    effectLayer(void) {};
    ~effectLayer(void) {};

    int         effectnumber;
    QString     effectname;
    FxCategory  objtype;
    bool        selected;
    bool        layerStatus;
    bool        isOn;
    GLfloat     x,y,z;
    GLfloat     r;

    //bool        active;
};


#endif





