/*******************************************************************************
**
** The source file for the Jahshaka Core jahtexture module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "coreobject.h"

////////////////////////////////////////////////////////////////////////////////
// we need to break these functions out into the libraries along with jahobject
// we can add support for any source file here, avi, quicktime, etc
// texturing needs to be fast!!!
// are we passing all this data back in the object?
// also

// heard this was a faster way to do things, plus the textures dont have to be a power of 2
// and so they can be computed directly from the image! this is great!!!
// glTexSubImage2D (GL_TEXTURE_2D, 0, 0, 0, textureData.texture.width(), textureData.texture.height(),
//	                            GL_RGBA,  GL_UNSIGNED_BYTE, textureData.texture.bits());


////////////////////////////////////////////////////////////
// build, initialize and rebuild textures

TextureImage motionPath::buildGlTexture (  QImage & buf  ) {

    TextureImage textured; // builds a texture object from the qimage buf

    //we should check if a images is 24 or 32 bit here for the texture module
    int mapwidth =  buf.width();
    int mapheight = buf.height();
    //int mapval, mapsize;
    int mapvalw, mapvalh;

    //here we determine the resolution of the opengl texture
    //since opengl textures must be an ordxer of 2^8 and symetrical

    //do they need to be symmetrical? can we do 128x256?
    if (mapwidth<=128)  { mapvalw = 128; } else
      if (mapwidth<=256)  { mapvalw = 256; } else
        if (mapwidth<=512)  { mapvalw = 512; } else
         if (mapwidth<=1024) { mapvalw = 1024;}

    //do they need to be symmetrical? can we do 128x256?
    if (mapheight<=128)  { mapvalh = 128; } else
      if (mapheight<=256)  { mapvalh = 256; } else
        if (mapheight<=512)  { mapvalh = 512; } else
         if (mapheight<=1024) { mapvalh = 1024;}

      textured.objectImage   = buf;
      textured.bpp           = 32;         // dafault to 32bit but we need to test for this for efficiency
      textured.wresolution    = mapvalw;    // was layerres
      textured.hresolution    = mapvalh;    // was layerres
      textured.width         = mapwidth;
      textured.height        = mapheight;

      textured.texture       = QImage(textured.wresolution, textured.hresolution, textured.bpp);

      textured.texwidth      = ( textured.width / textured.wresolution);
      textured.texheight     =  1- ( textured.height / textured.hresolution );

      //  calculation is           640/2=320 /10 = 32
      textured.x = (mapwidth/2)/10;
      textured.y = ((mapheight/2)/10) * 1.24; //aspect ratio corrections here

    //mapping ration to product video aspect 4:3
	  textured.mapwidth = (GLfloat)mapwidth / 2;
	  textured.mapheight= (GLfloat)mapheight / 2;

  return textured;
}

void motionPath::InitializeTexture(TextureImage & textured) {

	textured.texture       = textured.objectImage.copy(0,0,textured.wresolution,textured.hresolution,0);
	textured.texture       = QGLWidget::convertToGLFormat( textured.texture );  // flipped 32bit RGBA

}

void motionPath::setLayerTexture(void) {

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,  textureData.texID );
}

void motionPath::setKeyTexture(void) {

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,  textureKey.texID );
}

//who calls this and why?
void motionPath::rebuildTextureData (  QImage & buf  ) {

    textureData.objectImage = buf;

    InitializeTexture(textureData);
    updateTexture();
}

void motionPath::rebuildTextureKey (  QImage & buf  ) {

    textureKey.objectImage = buf;

    InitializeTexture(textureKey);
    updateTexture();
}


////////////////////////////////////////////////////////////
// make the texture itself

void motionPath::makeTexture( void ) {

    //erase texture if it already exists
	//theres a memory leak in makeLayerTexture maybe the deleate isnt working?

    if (textureData.texID)     {   glDeleteTextures(1,& textureData.texID); }

    makeLayerTexture();

    if (  keyStatus == TRUE) {
      if (textureKey.texID)     {   glDeleteTextures(1,& textureKey.texID);  }

          makeKeyTexture();
    }

}

void motionPath::makeLayerTexture( void ) {   //procedure to map textureimage to texture

	//this routine is causing a major memory leak for some reason
	//hiting get desktop for a layer repeatedly will show this

    glGenTextures(1, & textureData.texID);
    glBindTexture(GL_TEXTURE_2D,  textureData.texID);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    // Sgi systems (o2's at least) wont allow a 32bit texture and alpha blending to work simultaneously
    // the work around is to disable the alpha channel on 32 bit image files
    // maybe this will be fixed on ocatnes and or tga files

   //change this to a global ifdef later...
	
   #ifdef _OS_IRIX_
    glTexImage2D( GL_TEXTURE_2D, 0, 3,        textureData.texture.width(), textureData.texture.height(), 0,      //last value is the border
                                    GL_RGBA, GL_UNSIGNED_BYTE, textureData.texture.bits() );
   #else
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA,  textureData.texture.width(), textureData.texture.height(), 0,      //last value is the border
                                    GL_RGBA, GL_UNSIGNED_BYTE, textureData.texture.bits() );
   #endif

}

void motionPath::makeKeyTexture( void ) {  //procedure to map textureimage to keytexture

    glGenTextures(1, & textureKey.texID);
    glBindTexture(GL_TEXTURE_2D,  textureKey.texID);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture

    glTexImage2D( GL_TEXTURE_2D, 0, 3, textureKey.texture.width(), textureKey.texture.height(), 0,      //last value is the border
                                       GL_RGBA, GL_UNSIGNED_BYTE, textureKey.texture.bits() );

}

////////////////////////////////////////////////////////////
// video and image-sequncing routines

void motionPath::updateVidTex(int theframe) {

	//figure out the current frame with the offset applied

	//this is the frame we are at in the timeline
	//not sure why this is a global...
    currentframe=theframe;

	int currentkeyframe=theframe; //if theres a key...

	//this is where we compute the offset
	int offset;

	if (startframe>0) { offset = startframe-1; }
	else if (startframe<=0) { offset = startframe; }

	//apply the offset to curentframe
	currentframe -= offset;


	QString pathfilename;
	bool flag = false;
	
	pathfilename = getsequencefilename(currentframe);
	if (lastframeused!=pathfilename) {
		lastframeused = pathfilename; flag = true;
		LoadImage (pathfilename);
	}

	if ( keyStatus ) {

		if ( videotexkey ) {
			//this is where we compute the offset
			int keyoffset;

			if (keystartframe>0) { keyoffset = keystartframe-1; }
			else if (keystartframe<=0) { keyoffset = keystartframe; }

			//apply the offset to curentframe
			currentkeyframe -= keyoffset;

			pathfilename = getkeysequencefilename(currentkeyframe);

					
		} else {
			
			pathfilename = keypathname+keyfilename+keyextension;
		
		}

		if (lastkeyframeused!=pathfilename) {
			lastkeyframeused = pathfilename; flag = true;
			LoadKey (pathfilename);
		}

	}

	//now update the texture with the new image
	if (flag) updateTexture();
	
}


void motionPath::updateTexture( void ) {

    //used for video-clips we delete the old textures and create new ones
    //can we can SPEEEED this up by looking into
    //glCopyTexImage2D() and  copy the new data into the texture

    //its safe to assume the texture was already built once
    //glDeleteTextures(1,& textureData.texID); //should be taken care of in image loader?

//	makeLayerTexture();  	
   updateLayerTexture();

    if (  keyStatus ) {
           //glDeleteTextures(1,& textureKey.texID);
           updateKeyTexture();
    }

}

void motionPath::updateLayerTexture( void ) {   //procedure to update texture in a sequence

    glBindTexture(GL_TEXTURE_2D,  textureData.texID);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture
	
    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, textureData.texture.width(), textureData.texture.height(),
                                    GL_RGBA, GL_UNSIGNED_BYTE, textureData.texture.bits());

}

void motionPath::updateKeyTexture( void ) {   //procedure to update texture in a sequence

    glBindTexture(GL_TEXTURE_2D,  textureKey.texID);   // 2d texture (x and y size)

    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // scale linearly when image bigger than texture
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // scale linearly when image smalled than texture
	
    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, textureKey.texture.width(), textureKey.texture.height(),
                                    GL_RGBA, GL_UNSIGNED_BYTE, textureKey.texture.bits());

}

///////////////////////////////////////////////////////////////////////////////
// for the particle system

inline float motionPath::Gaussian2(float x, float y, float sigma)
{
// The sqrt of 2 pi.
#define SQRT2PI 2.506628274631000502415765284811045253006

//exp is the exponential function
	return exp(-0.5 * (x*x + y*y) / (sigma*sigma)) / (SQRT2PI * sigma);
}


void motionPath::MakeGaussianSpotTexture()
{
#define DIM 32
#define DIM2 (DIM>>1)
#define TEX_SCALE 7.0

	glGenTextures(1, (GLuint *)&texture);
	glBindTexture(GL_TEXTURE_2D, texture);

	float *img = new float[DIM*DIM];

	for(int y=0; y<DIM; y++)
	{
		for(int x=0; x<DIM; x++)
		{
			if(x==0 || x==DIM-1 || y==0 || y==DIM-1)
				img[y*DIM+x] = 0;
			else
			{
				img[y*DIM+x] = TEX_SCALE * Gaussian2(x-DIM2, y-DIM2, (DIM*0.15));
			}
		}
	}

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	float col[4] = {1.f, 1.f, 1.f, 1.f};
	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, col);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	//GL_ASSERT();

	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_ALPHA16, DIM, DIM, GL_ALPHA, GL_FLOAT, img);
}























