/*******************************************************************************
**
** The header file for the Jahshaka render module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/


#include "jahrender.h"

JahRender::JahRender(void) { zoomVal=1; }

JahRender::~JahRender(void) { }

void JahRender::setToolMode(MouseMode mmode) { theToolMode = mmode; }

void JahRender::setZoom(GLfloat zoom) {

  zoomVal = zoom;

  resizeOpenGL( renderwidth, renderheight, fAspectval );

}

void JahRender::positionObject( motionNode * node ) {

        glTranslatef( node->tx, node->ty, node->tz );
        glRotatef( node->rx, 1.0, 0.0, 0.0 );
        glRotatef( node->ry, 0.0, 1.0, 0.0 );
        glRotatef( node->rz, 0.0, 0.0, 1.0 );
        glScalef ( node->sx, node->sy, node->sz );

}

void JahRender::drawLayer(motionPath & layer) {
// we need ways to speed this up as its the core routine

	if ( layer.layerStatus )  {

     glPushMatrix();

     //position the object acording to the keyframe set in node
	 positionObject(layer.node);

	 //now we set up drawing variables and draw the object

		  // used for object selection - background is excluded
          if (layer.objtype!=BACKDROP || layer.objtype!=MAINBACK) {  glLoadName(layer.layernumber); }

		  // set object color and transparency
          glColor4f( layer.Red,  layer.Green,  layer.Blue, ( layer.node->Alpha/100) );

          if ( layer.blend )     {  glEnable(GL_BLEND);
                                    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA); }

          if ( layer.depth)      { glEnable(GL_DEPTH_TEST);       }
          if ( layer.lighting )  { glEnable(GL_LIGHTING);         }

			  //Object drawing routines

			  //when culling is involved we draw the cull first
			  if ( layer.cull )      {  glEnable(GL_CULL_FACE); glCullFace(GL_FRONT);
									    makeObject(layer);
									    glCullFace(GL_BACK); }

			  //we make the key first
			  if ( layer.drawtheKey ) { glBlendFunc(layer.blendmode1,layer.blendmode2);
									    makeKey(layer);
									    glBlendFunc(GL_ONE, GL_ONE); }

			  //draw the main object here
			  if ( layer.layerStatus ) makeObject(layer);

          // clean up environment
          //if ( layer.keyStatus ) { glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA); }
          if ( layer.blend )     { glDisable(GL_BLEND);           }
          if ( layer.lighting )  { glDisable(GL_LIGHTING);        }
          if ( layer.cull)       { glDisable(GL_CULL_FACE);       }

		  //if object is selected we draw the selection profile here
          if ( layer.selected )  { makeLayerOutline(layer);		  }

          if ( layer.depth )     { glDisable(GL_DEPTH_TEST);      }
		
     glPopMatrix();

	}
 	//if (JAHDEBUG) debug("Finished Drawing the layer");

}

void JahRender::drawNodes(motionPath & layer) {
// we need ways to speed this up as its the core routine

	if ( layer.layerStatus )  {

     glPushMatrix();

     //position the object acording to the keyframe set in node
	   //positionObject(layer.node);
     layer.makeKeyFrames();
	   //now we set up drawing variables and draw the object

		
		
     glPopMatrix();

	}
 	//if (JAHDEBUG) debug("Finished Drawing the layer");

}



void JahRender::initializeOpenGL() {       //Set up the OpenGL rendering state, and define display list

    //set up environment and turn on depth testing
    glDepthFunc(GL_LESS); glEnable( GL_DEPTH_TEST ); glShadeModel(GL_SMOOTH);

    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glEnable( GL_TEXTURE_2D );

    GLfloat light_ambient[] = {0.5, 0.5, 0.5, 1.0};     //    GLfloat light_ambient[] = {0.3, 0.3, 1.0, 1.0};
    GLfloat light_diffuse[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat light_specular[] = {1.0, 1.0, 1.0, 1.0};
    GLfloat light_position[] = {0.0f, 0.0f, 1.0f, 0.0f};

    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);

    glEnable(GL_LIGHT0);


    // Set up various other stuff
    glClearColor( 0.3, 0.3, 0.3, 0.0 ); // Let OpenGL clear to grey
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );

    glEnable( GL_NORMALIZE );     //glEnable(GL_LINE_SMOOTH);



}

void JahRender::resizeOpenGL( int w, int h, GLfloat &fAspect ) {

  glViewport(0, 0, w, h);

  //store these for access later on
  renderwidth = w;  renderheight = h;

  //QString thenum;
  //debug("Resize OpenGL Width :"+thenum.setNum(w));  //1210
  //debug("Resize OpenGL Height:"+thenum.setNum(h));  //700

	fAspect = ((GLfloat)1210 ) / ((GLfloat)700 ) ;   // Calculate aspect ratio of the window
  fAspectval = fAspect;

	//fAspect = (GLfloat)X_RESOLUTION/(GLfloat)Y_RESOLUTION;   // Calculate aspect ratio of the window
	//fAspect = (GLfloat)w/(GLfloat)h;   // Calculate aspect ratio of the window
	glMatrixMode(GL_PROJECTION);    // Set the perspective coordinate system
	glLoadIdentity();

	gluPerspective(45.0f, fAspect, 1.0, 10000.0);   	

	glMatrixMode(GL_MODELVIEW);    // Modelview matrix reset
	glLoadIdentity();

  glTranslatef( 0.0, 0.0, ((GLfloat)-846.0 * zoomVal) );

  //now scale the model viewport to keep the correct perspective!
  glScalef ( (GLfloat)1210/(GLfloat)w, (GLfloat)700/(GLfloat)h, 1.0 );

}


