/*******************************************************************************
**
** The source file for the Jahshaka RenderObject file
** The Jahshaka Project
** Copyright (C) 2000-2003 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/


#include <jahrender.h>

////////////////////////////////////////////////////
// these routines call the object which 
// then draws itself

////////////////////////////////////////////////////
// this routine draws layers and objects

void JahRender::makeObject( motionPath & layer ) {

	//debug("going to draw the object now");
  
  layer.setLayerTexture(); //first we select the texture map for the layer this should be in the loops so its not called 2 much

  switch ( layer.layertype ) {

  case LAYER     :  {  if (layer.drawtheLayer) {

						  switch ( layer.objtype ) {

							//background layers
							case BACKDROP     :     {    layer.makeBackground();          break; }
							case MAINBACK     :     {    layer.makeBlackBackground();     break; }

							case JAHLAYER     :     {    layer.makeLayer();               break; }
							case MESHLAYER    :     {    layer.makeMeshLayer();           break; }						
							case CUBE         :     {    layer.makeCube();                break; }
							case SPHERE       :     {    layer.makeSphere();              break; }
							case CYLINDER     :     {    layer.makeCylinder();            break; }
							case BEZIERS      :     {    layer.makeBezier();              break; }
							case TITLEGRID    :     {    layer.makeTitleGrid();           break; }
							default           :     {                                     break; }
								  }

						  break; }
					}

    case PARTICLE  :  {  layer.makeParticle();            break; }

    case OBJECTS   :  {    

						  switch ( layer.objtype ) {

							case OBJOBJECTS   :     {    layer.makeObj();                 break; }
							case MAXOBJECTS   :     {    layer.makeMaxObj();              break; }
							default           :     {                                     break; }
								  }

		                 break; }

		//frame objecttype is obsolete was replaced by titlegrid
    case FRAME     :  {  layer.makeTitleGrid();           break; }

    case TEXT      :  {  layer.drawText();                break; }

    case GRID      :  {  layer.makeGrid();                break; }
	default        :  {                                   break; }
  }

	//layer should clean up after itself
	glDisable(GL_TEXTURE_2D);

}

////////////////////////////////////////////////////
// this routine draws the key since it is actually
// a separate layer

void JahRender::makeKey( motionPath & layer ) {

	layer.setKeyTexture(); //first we select the texture map for the key

	switch ( layer.objtype ) {

		case JAHLAYER    :     {    layer.makeKeyLayer();                 break; }
		default          :     {                                          break; }
	}

	//layer should clean up after itself
	glDisable(GL_TEXTURE_2D);
}


////////////////////////////////////////////////////
// this routine draws the outline or object control / GUI widgets
// these subroutines shouldnt be in the layer code...
// should be here or in the world as its a gui element
// but currently they are attached to the layers themselves

// we could query the layer and get its position and draw the elements here...

void JahRender::makeLayerOutline( motionPath & layer) {

	switch ( layer.layertype ) { //add .layertype for more options


    case LAYER     :  {  
						if (layer.objtype==MAINBACK) layer.makeBackdropOutline(); else 
							if (layer.objtype==BACKDROP) layer.makeBackdropOutline(); else {

							switch(theToolMode)
							{
							case MTRANSLATE : { layer.makeLayerTranslate();   break; } //makeLayerTranslate( void);
							case MSCALE     : { layer.makeLayerScale();   break; } //makeLayerScale( void);
							case MROTATE    : { layer.makeLayerRotate();    break; } //makeLayerRotate( void);
							case MOFF       : { break; }
							}
						}
					  }

    case TEXT      :  {    
						if (layer.objtype==JAHTEXT) {
						  switch ( layer.text->fontstyle ) {

						    case FPOLYGON     :     {                                     break; }
							case FEXTRUDE     :     {    layer.draw3dTextOutline();       break; }
						  }
						}
					  }

    default        :  {                               break; }

  }

}
