 /*******************************************************************************
**
** The source file for the Jahshaka glworld.cpp module
** The Jahshaka Project
** Copyright (C) 2000-2003 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "glworld.h"

GLWorld::GLWorld( QWidget* parent, const char* name, QHBox* controller, int* globalclipnumber)
  : QGLWidget( parent, name ) {  if (JAHDEBUG) JahDebug("Initializing Module",name);

  //initialize all global variables
  numberLayers = 0;
  minFrame=1; maxFrame=100;
  gridval = 0;
  renderpath = JahBasePath+"media/renders/";  mmode = MOFF;
  forceplay = false;
  ModuleName  = name;

  //set the module clipnumber to shared global
  clipnumber = globalclipnumber;

  //the main objects
  layers      = new motionPath[NUMBER_LAYERS];
  thegrid     = new motionPath;
  titlesafe   = new motionPath;
  textsafe    = new motionPath;
  camera      = new motionPath;

  //the renderspace
  renderSpace = new JahRender;
  worldorder  = new worldSort[NUMBER_LAYERS];

  //builds the controller
  buildController(controller);

  //initalize world objects
  initializeWorld();

  //set default render quality to uncompressed BMP
  renderquality = 2;

  //controls the fps for scene playback
  redrawWait=15;
  timer = new QTimer( this );
  connect( timer, SIGNAL(timeout()), SLOT(update()) );
  timer->start( redrawWait, TRUE );

};

GLWorld::~GLWorld(void) {

  delete [] layers;
  delete [] worldorder;

};


void GLWorld::paintGL() {

  if ( animation ) {  updateanimation(currentFrame); currentFrame++;
                      if ( currentFrame > maxFrame ) currentFrame=1;
  }

   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
   glPushMatrix();

  //turn off the backdrop for selection mode until we fix selection Z testing
  //draw black video safe area if no layer present
  if (!select) {
    if (layers[0].layerStatus) {  renderSpace->drawLayer(layers[0]);  }
                          else {  renderSpace->drawLayer(*titlesafe); }
  }

  //we position the camera here
  renderSpace->positionObject(camera->node);

  if (gridval) {  renderSpace->drawLayer(*thegrid); }

  //set up opengl layer selection
  glInitNames(); glPushName(0);                      // initialize the selection stack

  //draw all layers that are in the world space
  int dlayer;

  //if we have layers added draw them
  if (numberLayers>0) {

    //if there are more than 1 layers sort them
    if (numberLayers>1) { sortLayers(); }

    for (int i=1; i<=numberLayers; i++) {

      if (numberLayers>1) { dlayer = worldorder[i-1].laynum; }
      else                { dlayer = i; }

  	  if (layers[dlayer].layerStatus && layers[dlayer].foreground!=1 ) {
           renderSpace->drawLayer(layers[dlayer]);
           if(layers[dlayer].drawKeyframes) { renderSpace->drawNodes(layers[dlayer]); }
      }

    }
  }

   glPopMatrix();

  //draw the forground layers ie objects not affeceted by worldspace or sorting
  if (numberLayers>0) {
    for (int i=1; i<=numberLayers; i++) {
      if (layers[i].layerStatus && layers[i].foreground==1) {
        renderSpace->drawLayer(layers[i]);
        if(layers[i].drawKeyframes) { renderSpace->drawNodes(layers[i]); }
      }
    }
  }

  if (textsafe->layerStatus) {  renderSpace->drawLayer(*textsafe);  }

  // Wait this many msecs before redraw
  if ( animation ) {
    glFlush();  timer->start( redrawWait, TRUE );
  } else glFlush();

}



void GLWorld::initializeGL() {  renderSpace->initializeOpenGL(); }

void GLWorld::initializeWorld() {

  polySmooth=0;
  activeLayer = 0; currentFrame=1;
  lastLayer = 0; select = FALSE;  // lastLayer is used to unselect...


  animation = FALSE; animsliderselected=FALSE;  renderStatus=FALSE;
  lastx = lasty = 0;  currentFrame =0;

  // set up the world first   - is this necessary? what are the defaults
  camera->node->sx = camera->node->sy = camera->node->sz = camera->node->sc = 1.0;
  camera->layerStatus = TRUE;
  camera->drawtheLayer = TRUE;

  // set up the black title safe area
  titlesafe->layertype = LAYER; titlesafe->objtype = MAINBACK;
  titlesafe->blend = FALSE; titlesafe->depth = FALSE;
  titlesafe->Red=1.0; titlesafe->Green=0.0;titlesafe->Blue=0.0;
  titlesafe->layerStatus = TRUE;
  titlesafe->drawtheLayer = TRUE;

  // this is the green textsafe overlay
  textsafe->node->sx = 0.8; textsafe->node->sy = 0.8; //it is scaled to fit here...
  textsafe->layertype = FRAME; textsafe->objtype = TITLEGRID;
  textsafe->Red=0.0; textsafe->Green=1.0; textsafe->Blue=0.0;
  textsafe->blend = FALSE; textsafe->depth = FALSE;
  textsafe->foreground = TRUE; textsafe->layerStatus = FALSE;
  textsafe->drawtheLayer = TRUE;

  // this is the grid
  //thegrid->node->sx = 5;      thegrid->node->sy = 5;      thegrid->node->tx = -100;
  //thegrid->node->ty = -30;    thegrid->node->tz = -100;   thegrid->node->rx = 90;
  thegrid->layertype = GRID;
  thegrid->Red=0.75; thegrid->Green=0.75; thegrid->Blue=0.75;
  thegrid->blend = FALSE; thegrid->depth = TRUE;
  thegrid->layerStatus = TRUE;

}

void GLWorld::resizeGL( int w, int h ) {

  this->makeCurrent();
  renderSpace->resizeOpenGL( w,h, fAspect );
}

void GLWorld::changeZoom(int val) {

  //val is -10 to 10 we need to compute this
  val = val+10;           //this maps it to 0 to 20
  GLfloat value = val*10; //now its from 0 to 200

  if (value==0) value= 1; //trap divide by zero error

  GLfloat theval = value/100;

  this->makeCurrent();
  renderSpace->setZoom(theval);
  updateGL();

}

//void JahRender::setZoom(GLfloat zoom) { zoomVal = zoom; }


void GLWorld::initializeObjects() {   /* this is overridden */    }

//////////////////////////////////////////////
// Slider related

void GLWorld::updateSliders()  { /* this is overridden */   }

void GLWorld::updateSlider(int value) {

  controllerslider->setValue(value);
  currentFrame = value;
}

void GLWorld::sliderselected() { animsliderselected = TRUE;  }

void GLWorld::sliderreleased() {
          animsliderselected = FALSE; currentFrame = controllerslider->value();
          updateanimframe(currentFrame);
          updateSliders();
}

////////////////////////////////////////////////
// Player related

void GLWorld::setForcePlay()  { forceplay = !forceplay; }

void GLWorld::firstframeanimation()  {

  currentFrame=1;
  updateSlider(currentFrame);
  updateanimframe(currentFrame);
  updateSliders();

}

void GLWorld::previousframeanimation()  {

  if ( animation ) { toggleAnimation(FALSE); }

  currentFrame-=1;
  updateSlider(currentFrame);
  updateanimframe(currentFrame);
  updateSliders();

}

void GLWorld::nextframeanimation()  {

  if ( animation ) { toggleAnimation(FALSE); }

  currentFrame+=1;
  updateSlider(currentFrame);
  updateanimframe(currentFrame);
  updateSliders();

}

void GLWorld::lastframeanimation()  {

  if ( animation ) { toggleAnimation(FALSE); }

  currentFrame=maxFrame;
  updateSlider(currentFrame);
  updateanimframe(currentFrame);
  updateSliders();

}


////////////////////////////////////////
// Toggle and Set routines

void GLWorld::nameLayer(void) {  namepopup->popup(NameButton); }

void GLWorld::setlayerName(QString name) {

	//cant rename the world mon
	if (activeLayer == 0) return;
	//check also if -1 means nothing is selected

	QListViewItem * rasta;
	rasta = lv->currentItem();

	rasta->setText ( 0,name);
	lv->repaint(rasta);

	layers[activeLayer].layername = name;

}

void GLWorld::setGrid()   { if (gridval) { gridval = 0;  } else gridval = 1; updateGL(); }

void GLWorld::setSmooth() { if (polySmooth) { polySmooth = 0; } else polySmooth = 1; updateGL(); }

void GLWorld::setBlur() {   }    //for motionblur not implemented yet...

void GLWorld::setTextSafe() {  textsafe->layerStatus =  !textsafe->layerStatus; updateGL(); }

void GLWorld::toggleForegroundStatus() {  layers[activeLayer].foreground =  !layers[activeLayer].foreground; updateGL(); }

void GLWorld::toggleLight(void) { layers[activeLayer].lighting = !layers[activeLayer].lighting; updateGL(); }

//this func is obsolete
void GLWorld::toggleScene(void) { /* switches to the load save panel */

	int theid = controlpanel->id (controlpanel->visibleWidget() );
		
	if (theid==0)	controlpanel->raiseWidget(1);
	    else controlpanel->raiseWidget(0);

}

////////////////////////////////////////
// Font control routines

void GLWorld::setText(const QString& wassup) { layers[activeLayer].text->text = wassup;
											   layers[activeLayer].makeText(); updateGL();  }  //newTxt

void GLWorld::setFontColor() { layers[activeLayer].getColor();
							   layers[activeLayer].makeText(); updateGL(); } //  slotFontColor  SlotColor ??

void GLWorld::loadFont() {
    //need to use a loader here... give the user choice
    QString format = "ttf";

    QString filename = QFileDialog::getOpenFileName( "fonts",  QString( "*.%1" ).arg( format.lower() ), this );
    if ( !filename.isEmpty() ) {

      layers[activeLayer].text->font = filename;

	  layers[activeLayer].loadFontText();

	  layers[activeLayer].makeText();
    }
}

/////////////////////////////////////////
// 3D model control here

void GLWorld::loadObject(){ layers[activeLayer].loadObjObject(); }

void GLWorld::loadMaxObject(){ layers[activeLayer].loadMaxObject(); }


void GLWorld::changeParticleColors() {

  layers[activeLayer].ParticleColors = !layers[activeLayer].ParticleColors;

}

//////////////////////////////////////////
// Clip and Key control

void GLWorld::setKeyStatus()         {
	
	if (activeLayer>0) layers[activeLayer].drawtheKey =  !layers[activeLayer].drawtheKey;
	if (layers[activeLayer].drawtheKey) layers[activeLayer].depth = false;

	updateGL();
}

void GLWorld::setClipStatus()         {
	
	if (activeLayer>0) layers[activeLayer].drawtheLayer =  !layers[activeLayer].drawtheLayer;
	if (layers[activeLayer].drawtheKey) layers[activeLayer].depth = false;

	updateGL();
}

void GLWorld::InvertKeyData()  { layers[activeLayer].changeKeyStatus(); updateGL();}     //loads a still, need to fix this for animations/avi

void GLWorld::loadClip()  { layers[activeLayer].loadClip();   layers[activeLayer].videotex = FALSE;     updateGL();}     //loads a still, need to fix this for animations/avi

void GLWorld::loadKey()   { layers[activeLayer].loadKey();    layers[activeLayer].videotexkey = FALSE;  updateGL();}     //loads a still, need to fix this for animations/avi

void GLWorld::addClip(videoData clip) {

  layers[activeLayer].layername   = clip.clipname;
  layers[activeLayer].layernumber = clip.clipnumber;
  layers[activeLayer].filename    = clip.filename;
  layers[activeLayer].pathname    = clip.location;
  layers[activeLayer].extension   = clip.extension;

  layers[activeLayer].numframes   = clip.endframe;
  layers[activeLayer].startframe  = 1;
  layers[activeLayer].endframe    = clip.endframe-clip.startframe+1;
  layers[activeLayer].inframe	    = clip.startframe;
  layers[activeLayer].outframe    = clip.endframe;

  switch (clip.theCategory) {

        case IMAGE         : { layers[activeLayer].videotex = FALSE;  break; }
        case CLIP          : { layers[activeLayer].videotex = TRUE;
  			                       if (clip.cliptype==SEQUENCE) { layers[activeLayer].videoext = clip.videoext; }
  			                       break; }
  }

  layers[activeLayer].LoadLayerImage(IMAGE);
  layers[activeLayer].makeTexture();

  updateGL(); updateSliders();

}

void GLWorld::addClipKey(videoData clip) {

  layers[activeLayer].keyfilename    = clip.filename;
  layers[activeLayer].keypathname    = clip.location;
  layers[activeLayer].keyextension   = clip.extension;

  layers[activeLayer].numkeyframes   = clip.endframe;
  layers[activeLayer].keystartframe  = 1;
  layers[activeLayer].keyendframe    = clip.endframe-clip.startframe+1;
  layers[activeLayer].keyinframe     = clip.startframe;
  layers[activeLayer].keyoutframe    = clip.endframe;

   switch (clip.theCategory) {

          case IMAGE         : { layers[activeLayer].videotexkey = FALSE;  break; }
          case CLIP          : { layers[activeLayer].videotexkey = TRUE;
                                 if (clip.cliptype==SEQUENCE) { layers[activeLayer].videoextkey = clip.videoext; }
					                       break; }
   }

  layers[activeLayer].keyStatus = TRUE;
  layers[activeLayer].LoadLayerImage(KEY);
  layers[activeLayer].makeTexture();

  updateGL(); updateSliders();

}


/////////////////////////////////////////////////
// Reset and clear layers

void GLWorld::ResetLayer() {

  if (activeLayer==0) { camera->reset(); } else { layers[activeLayer].reset(); }
  animation = FALSE;
  updateGL();

}

void GLWorld::ResetAll() {

  camera->reset();

  for (int i=0; i<NUMBER_LAYERS; i++) { layers[i].reset(); }

  animation = FALSE;
  updateGL();
}

void GLWorld::ClearAll() {

    lv->clear();

    delete [] layers;  layers = new motionPath[NUMBER_LAYERS];
    delete camera;     camera = new motionPath;

    numberLayers = 0;

    initializeWorld();
    initializeObjects();
}

void GLWorld::loadClearAll() {

    lv->clear();

    delete [] layers;  layers = new motionPath[NUMBER_LAYERS];
    delete camera;     camera = new motionPath;

    numberLayers = 0;

    initializeWorld();
   // initializeObjects();
}




















