/*******************************************************************************
**
** The header file for the Jahshaka glworld module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef GLWORLD_H
#define GLWORLD_H

#include "interfaceheaders.h"
#include <coreobject.h>
#include <widget.h>
#include <videodata.h>
#include <ctype.h>
#include <qtextstream.h>
#include "InputLCD.h"
#include <dialogs.h>
#include <qdial.h>
#include <jahrender.h>
#include <glworldheaders.h>


extern int X_RESOLUTION;
extern int Y_RESOLUTION;
extern bool JAHDEBUG;

extern QString JahBasePath;

const int NUMBER_LAYERS = 30;


class GLWorld : public QGLWidget {
  Q_OBJECT

public:

    GLWorld( QWidget* parent, const char* name, QHBox* controller, int* globalclipnumber );
    ~GLWorld();

  //build the modules interface
  void buildController( QHBox* controller);

  //opengl routines
  void initializeGL();                                              // OpenGL  functions
  virtual void paintGL();
  void resizeGL( int w, int h );

  void initializeWorld();
  virtual void initializeObjects();

  //default module variables
  QString     ModuleName;
  motionPath  *layers;
  motionPath  *thegrid;
  motionPath  *titlesafe;
  motionPath  *textsafe;
  motionPath  *camera;

  //used to srot the layers before drawing the scene
  worldSort  *worldorder;

  //controls the rendering of objects in a scene
  JahRender* renderSpace;

  //the timer controls screen redraw
  QTimer* timer;

  QString renderpath;
  int numberLayers;
  int gridval;
  bool polySmooth, select;
  bool displaySplit, displayToggle;
  int  *clipnumber;
  int activeLayer, currentFrame, lastLayer;
  int minFrame, maxFrame;
  bool animation, animsliderselected;
  int  lastx,lasty,redrawWait;
  bool renderStatus;
  QString renderext; QString renderformat; int renderquality;
  bool forceplay;

  GLfloat fAspect;

  //main create layer function
  bool createLayer( int nl, QString layerName, LayerCategory layertype, ObjectCategory objtype );

  //virtual functions are meant to be overridden in calling modules
  virtual void updateVideoTexture(int, int);
  QImage  screenShot(int Frame,int xloffset,int xroffset,int ytoffset,int yboffset,
                     int xres, int yres);

  //////////////////////////////////////////////////////////////////
  //mouse actions - should these be virtual ?
  bool	mousePressed;
  void	mousePressEvent( QMouseEvent *e );
  void	mouseReleaseEvent( QMouseEvent *e );
  void	mouseMoveEvent( QMouseEvent *e );
  void  mouseMoveCamera( int x,int y );
  void  mouseMoveLayer ( int thelayer, int x, int y );
  void  processSelection(GLuint *pSelectBuff);
  MouseMode mmode;

  //interface builders
  void addButton( QPushButton * button, int x, int y, int w, int h, QString text );
  void addCheckButton( QCheckBox * button, int x, int y, int w, int h, QString text );
  void addToolButton( QToolButton * button, int x, int y, int w, int h, QString icon );
  void addToolButton( QToolButton * button, int x, int y, int w, int h  );
  void addIconButton( QPushButton * button, int x, int y, int w, int h , QString icon );
  void addIconButton( QPushButton * button, int x, int y, int w, int h  );

  void addSlider( QSlider * slider, int x, int y, int w, int h, int range);
  void addRangeSlider( QSlider * slider, int x, int y, int w, int h, int rangemin, int rangemax);

  void addSliderLcd( InputLCD * lcd, int x, int y, int w, int h, int range);
  void addLabel( QLabel * label, int x, int y, int w, int h, QString text, int size);

  void addLcdLabel      ( QFrame * frame,
                          InputLCD * lcd, int x, int y, int w, int h, int range,
                          QString labeltext, int size );

  void addLabelLcd      ( QFrame * frame,
                          InputLCD * lcd, int x, int y, int w, int h, int range,
                          QString labeltext, int size );

  void addSliderAndLcd( QSlider * slider, int x, int y, int w, int h, int range,
                        InputLCD * lcd );

  void addSliderAndLcd( QSlider * slider, int x, int y, int w, int h, int rangemin, int rangemax,
                        InputLCD * lcd );

  void addSliderLcdLabel( QFrame * frame,
                          QSlider  * slider, int x, int y, int w, int h, int range,
                          InputLCD * lcd,
                          QString labeltext, int size );

  void addSliderLcdLabel( QFrame * frame,
                          QSlider  * slider, int x, int y, int w, int h,
                          int rangemin, int rangemax,
                          InputLCD * lcd,
                          QString labeltext, int size );

  void addSliderLcdLabel( QFrame * frame,
                          QSlider  * slider, int x, int y, int w, int h,
                          int rangemin, int rangemax, int preset,
                          InputLCD * lcd,
                          QString labeltext, int size );

  void addDialLcdLabel  ( QFrame * frame,
                          QDial  * slider, int x, int y, int w, int h, int range,
                          InputLCD * lcd,
                          QString labeltext, int size );

  ///////////////////////////////////////////////////////////////////

  void swap(worldSort &x, worldSort &y);


  ///////////////////////////////////////////////////////////////////

  QFrame* maincontroller;
  QWidgetStack* MainController;
  QWidget* page;
  QFrame* forceplayframe;
  QCheckBox* forceplaycheckbox;
  QFrame* controllerframe;
  QPushButton* controllerpreviousbutton;
  QPushButton* controllerstopbutton;
  QPushButton* controllerplaybutton;
  QPushButton* controllerffworardbutton;
  QPushButton* controllernextbutton;
  QPushButton* controllerrewindbutton;
  QSlider* controllerslider;
  QFrame* timecodeframe;
  QPushButton* keyframebutton;
  QLCDNumber* timecodedisplay;
  QFrame* imagingframe;
  QPushButton* imagingphotobutton;
  QPushButton* imagingrenderbutton;

  //protected:
  QHBoxLayout* Form1Layout;
  QHBoxLayout* maincontrollerLayout;
  QHBoxLayout* pageLayout;

  //////////////////////////////////////////////////////////////////////
  //controller variables
  int    Astartframe;  int  Aendframe;  int  Aanimframe; int  MaxFrames;

  QListView*     lv;

  QHBox*       MiddleFrame;		QSlider *     Slider2;
  InputLCD*    animframe;			QLabel *      aframetext;  //was QLCDNumber
  QLabel*      sframetext;		InputLCD *    lcdstartframe;
  QLabel*      eframetext;		InputLCD *    lcdendframe;

  QCheckBox    * forcedplay;

  QPushButton  * scrubfirst, * scrubprevious, * scrubstop, * scrubplay, * scrubnext, * scrublast,
   * scrubkey, * scrubrender, * scrubrenderAll;

  //buttons that control the listbox
  QPushButton *  AddButton, *  DelButton, * SceneButton;
  QPushButton*  NameButton;
  FancyPopup* namepopup;

  //FancyPopup* namepopup;

  //scene control variables
  QWidgetStack* controlpanel;
  QFrame* scene_controls;
  QFrame* object_controls;

  //interface stacks
  QWidgetStack*  fxaxis;
  QWidgetStack*  fxobject;

public slots:

  // world controls
  void setWorldXR( int degrees );    void	setWorldYR( int degrees );    void	setWorldZR( int degrees );
  void setWorldXT( int degrees );    void	setWorldYT( int degrees );    void	setWorldZT( int degrees );

  // layer controls
  void setXRotation( int degrees );    void	setYRotation( int degrees );    void	setZRotation( int degrees );
  void setXScale   ( int degrees );    void	setYScale   ( int degrees );    void	setZScale   ( int degrees );
  void setXTrans   ( int degrees );    void	setYTrans   ( int degrees );    void	setZTrans   ( int degrees );
  void setAlpha    ( int degrees );    void	setZoom     ( int degrees );    void	setExtrude  ( int degrees );

  void setKeyframeDrawStatus(void);

  //for spaceball
  void spaceEvent  ( double x, double y, double z, double a, double b, double c );

  // for video clips
  void setinFrames      ( int frame );
  void setoutFrames     ( int frame );
  void setinKeyFrames   ( int frame );
  void setoutKeyFrames  ( int frame );

  //for editing
  void toolTranslate( void );    void toolScale( void );     void toolRotate( void );
  void changeZoom( int val );

  //for effects
  void toolToggleView( void);	   void toolSplitEffect(void);

  void updateToolDisplay(void);

  void  loadClip();    void  loadKey();
  void setKeyStatus(void);
  void setClipStatus(void);

  void  loadObject();   void  loadMaxObject();
  void  toggleForegroundStatus();

  void  setText(const QString& wassup);    void  setFontColor( void );    void  loadFont( void );
  void  setTextSafe( void );

  void  setGrid( void );
  void  setSmooth( void );
  void  setBlur( void );
  void  InvertKeyData(void);

  virtual void  updateSliders();//to be overridden...
  void  sliderselected();    void  sliderreleased();
  void  updateSlider(int value);

  void  updatesliderStartframe(int);
  void  updatesliderEndframe(int);

  void setForcePlay(void);

  void firstframeanimation(void);
  void previousframeanimation(void);
  void nextframeanimation(void);
  void lastframeanimation(void);

  void  startanimation();    void  stopanimation();
  void  toggleAnimation(bool value);

  void  updateanimframe(int Frame);

  void  updateanimation(int Frame);


  void  addKeyframe();

  //void  updateScene(void);

  QImage  Photo(videoData);
  QImage  RenderAll(videoData,int,int);

  void  renderGL(void);
  void  Render(void);
  void  RenderScene(void);
  void  checkRenderQuality(void);
  void  setRenderQuality(int quality);

  //added for siggraph demo
  void  SoftRender(void);
  QImage  SoftPhoto(videoData);

  void  addClip(videoData);
  void  addClipKey(videoData);

  void printScene(void);
  void toggleScene(void);

  void  LoadAll(QString loadfilename);      void SceneLoad(void);
  void  LoadAllData(QFile &, int laynum);

  void  LoadImageData( QFile & ,int);       void LoadImageKeyData(QFile & ,int);
  void  LoadNodes(QFile & ,int);            void LoadColor(QFile & ,int);
  void  LoadText(QFile & ,int);             void LoadObjectData(QFile & ,int);
  void  LoadParticleData(QFile & ,int);

  void SetLayerTexture(int i);
  void SetLayerKeyTexture(int i);

  void  SaveAll(QString savefilename);     void SceneSave(void);
  void  SaveAllData( QFile & );

  void  SaveImageData( QFile & ,int);       void SaveImageKeyData(QFile & ,int);
  void  SaveNodes(QFile & ,int);            void SaveColor(QFile & ,int);
  void  SaveText(QFile & ,int);             void SaveObjectData(QFile & ,int);
  void  SaveParticleData(QFile & ,int);

  void nameLayer(void);

  void  ResetAll(void);
  void  ResetLayer(void);
  void  ClearAll(void);
  void  loadClearAll(void);

  void sortLayers(void);

  int   ParseDataInt(QString data);
  QString ParseDataString(QString data);
  QString ParseWinDataString(QString data);
  QString ParseDataFilename(QString data);
  float ParseDataFloat(QString data);

  void  setlayerName(QString);
  void  toggleLight(void);
  void  changeParticleColors();

  void  addSelection(bool status = TRUE);
  void  changeSelection(int layer);

  void  activateObject(int obj);

  //void  setLayerVisible( int  );  //this can be overridden if necessary
  void  setLayerVisible( int, bool  );  //this can be overridden if necessary

  //change layer type
  void ChangeObjectLayer(void);
  void ChangeObjectCube(void);
  void ChangeObjectCylinder(void);
  void ChangeObjectSphere(void);
  void ChangeObjectBezier(void);
  void ChangeObjectMesh(void);

  void  objectMenu(int ); // this must be overridden in the calling object

  //void clicked( QListViewItem* items );

  void  delLayer(void);

  // debugging messages
  QString JahDebug(QString message);
  QString JahDebug(QString message, int value);
  QString JahDebug(QString message, float value);
  QString JahDebug(QString message, QString value);
  QString JahDebug(QString message, ObjectCategory cat );
  QString JahDebug(QString message, LayerCategory cat );

signals:
  void    updateDesktop( videoData );
//this was added for Siggraph dualhead demo
//since our video card couldnt do live opengl video refresh
  void    updatePlayer ( videoData );
  void    updateBrowser( QString debugmessage );

};

#endif // GLWORLD_H




















































