/*******************************************************************************
**
** The source file for the Jahshaka glworld dataio code
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/


#include "glworld.h"
#include "qfile.h"
#include <qfiledialog.h>

#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qaccel.h>

#include <qpainter.h>
#include <qpaintdevicemetrics.h>

///////////////////////////////////////////////////////////////
// file load routine

void GLWorld::SceneLoad()
{
    QString fn = QFileDialog::getOpenFileName( "scenes", "*.jsf", this );
    if ( !fn.isEmpty() ) {
	//filename = fn;
	LoadAll(fn);
    } else {
		QMessageBox::information(0, "Loading aborted", fn);
    }
}

void GLWorld::LoadAll(QString loadfilename) {

  QString text;

  QFile f( loadfilename );
  if ( !f.open( IO_ReadOnly ) ) {
  QMessageBox::information(0, "Could not read file  ", loadfilename);
  return;
  }

  QTextStream t( &f );

  //read the header and parse the file
  text = t.readLine(); JahDebug(text);                   //line
  text = t.readLine(); JahDebug(text);   //gets app version here

  text = ParseDataString( t.readLine() );   //scene name is here
  JahDebug("The scene name is:",text);

  QString modtext = ParseDataString( t.readLine() );   //module name is here
  JahDebug("The module is:",modtext);

  int frames = ParseDataInt( t.readLine() );    //read in top keyframe
  Aanimframe = frames;
  updatesliderEndframe(Aanimframe);                 //this sets maxframe for us

  //lcdendframe is defined in the module so we need to check if it was before stting it
  if (lcdendframe) { lcdendframe->setValue(Aanimframe); }

	if (modtext==ModuleName) {
    loadClearAll();
    int laynum = ParseDataInt( t.readLine() );    //numberLayers is here
    JahDebug("The numberLayers is:",laynum);    numberLayers = laynum;

    text = t.readLine(); JahDebug(text);   //line
    LoadAllData(f, laynum);  		
    f.close();
    updateGL();
    QMessageBox::information(0, "File loaded successfully ", loadfilename);

    //ok now we set activelayer to World and select world in the listbox

	} else {

		f.close();
		QMessageBox::information(0, "Error loading Scene file  ", "Scene file was written for the "+modtext+" module");

	}

}


///////////////////////////////////////////////////////////////
// file save routine

void GLWorld::SceneSave()
{
  QString fn = QFileDialog::getSaveFileName( "scenes", "*.jsf",  this );
  if ( !fn.isEmpty() ) { SaveAll(fn);
  } else { QMessageBox::information(0, "Saving aborted", fn);
  //	statusBar()->message( "Saving aborted", 2000 );
  }
}

void GLWorld::SaveAll(QString savefilename) {

  QString text;

  QFile f( savefilename );
  if ( !f.open( IO_WriteOnly ) ) {
  QMessageBox::information(0, "Could not write to file  ", savefilename);
  //statusBar()->message( QString("Could not write to %1").arg(filename),  2000 );
  return;
  }

  QTextStream t( &f );

  text = JahDebug("---------------------------------");  text += "\n"; t << text;
  text = JahDebug("Jahshaka 1.9 Animation Scene File");  text += "\n"; t << text;
  text = JahDebug("Scene File Name:"+ParseDataFilename(savefilename) );      text += "\n"; t << text;
  text = JahDebug("Module:",ModuleName);     text += "\n"; t << text;
  text = JahDebug("Number of keyframes:",maxFrame);     text += "\n"; t << text;
  text = JahDebug("Number Of Layers:",numberLayers);     text += "\n"; t << text;
  text = JahDebug("---------------------------------");  text += "\n"; t << text;

  SaveAllData(f);

  f.close();

  QMessageBox::information(0, "File saved successfully ", savefilename);


}

/* these are the conversions
short toShort  (bool*ok=0, intbase=10)const
ushorttoUShort (bool*ok=0, intbase=10)const
int   toInt    (bool*ok=0, intbase=10)const
uint  toUInt   (bool*ok=0, intbase=10)const
long  toLong   (bool*ok=0, intbase=10)const
ulong toULong  (bool*ok=0, intbase=10)const
float toFloat  (bool*ok=0)const
doubletoDouble (bool*ok=0)const
*/

int GLWorld::ParseDataInt(QString data) {

  int index,jitlen;

  index	= data.findRev(":");   index +=1;     //find the colon and skip over it
  jitlen = data.length();    data = data.right(jitlen - index) ;
  int integer = data.toInt();
  return integer;
}

float GLWorld::ParseDataFloat(QString data) {

  int index,jitlen;

  index	= data.findRev(":");   index +=1;     //find the colon and skip over it
  jitlen = data.length();    data = data.right(jitlen - index) ;
  float integer = data.toFloat();
  return integer;
}

QString GLWorld::ParseDataString(QString data) {

  int index,jitlen;

  index	= data.findRev(":");   index +=1;     //find the colon and skip over it
  jitlen = data.length();    data = data.right(jitlen - index) ;
  return data;
}

//used for windows paths where the drive letter messes up our search as it has a :
QString GLWorld::ParseWinDataString(QString data) {

  int index,jitlen;

  index	= data.find(":");   index +=1;     //find the colon and skip over it
  jitlen = data.length();    data = data.right(jitlen - index) ;
  return data;
}

QString GLWorld::ParseDataFilename(QString data) { //caution, on unix this may be a  \ backslash

  int index,jitlen;

  index	= data.findRev("/");   index +=1;     //find the slash and skip over it
  jitlen = data.length();    data = data.right(jitlen - index) ;
  jitlen = data.length();    data = data.left(jitlen - 4) ; //file extension is four characters including the dot .jsf
  return data;
}





