#ifndef ANIME_H
#define ANIME_H

/*******************************************************************************
**
** The header file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include <glworld.h>

#include "qcheckbox.h"
#include "qtabwidget.h"

class GLAnime :  public  GLWorld {
  Q_OBJECT

public:

GLAnime( QWidget* parent, const char* name,
         QHBox* controls, QHBox* controller, QHBox* leftcontroller, QHBox* rightcontroller,
         int* globalclipnumber  )
        :  GLWorld(parent,name,controller,globalclipnumber ) {


    buildInterface(controls);
    buildInterfaceMenus(leftcontroller,rightcontroller);
    initializeObjects();

};

GLAnime::~GLAnime() { }

    void buildInterface( QHBox* qtinterface);
    void buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller );

    void initializeObjects();

public slots:


    void addLayer(void);

    void clicked( QListViewItem* item );

    void changeParticle();
    void changeParticleDraw();
    void changeFontDraw();
    void changeObjectDraw();
    void keyCliplayer();
    void keyClipkey();

    void updateSliders();
	
    void grabDesktop(void) { emit GrabClip(); };
    void grabDesktopKey(void) { emit GrabClipKey(); };


signals:
    void    GrabClip(void);    //linked to main jahcontrol to talk to desktop
    void    GrabClipKey(void);    //linked to main jahcontrol to talk to desktop

protected:
  //  void	  paintGL();         //overrides the base glworld class for custom modules
  //  void objectMenu(int menu);



private:

  ////////////////////////////////////////////////////////////////////////////
  //interface variables

  //for the left and right menus
  QFrame         *glworldLeft;	
  QPushButton    *tooltranslate, *toolscale, *toolrotate;

  QFrame         *glworldRight;
  QPushButton    *toolzoomin, *toolzoomout;
  QSlider        *zoom;

  //aded for siggraph demo
  QPushButton    *toolphoto;



  QFrame*      LayersFrame;

  QWidgetStack*  EffectsMain;
  QFrame*        SceneFrame;
  QFrame*        EffectsFrame;
  bool           EffectsStatus;

  QPushButton*   ResetButton;             QPushButton*   SaveButton;    QPushButton*   LoadButton;    QPushButton*   ClearButton;
  QCheckBox*     SmoothSelect;            QCheckBox*     BlurSelect;    QCheckBox*     TextSelect;
  	
  QPushButton*  ResetLayerButton;
  QPushButton*  ResetWorldButton;

  QTabWidget*        tabframe;
  QWidget*        axiscontrols;
  QWidget*        objectcontrols;
  QWidget*        texturecontrols;
  QWidget*        scenecontrols;
  QWidget*        effectscontrols;

  //    QWidgetStack*  fxaxis;
  QHBox*         fxaxis_clipone;     QHBox*         fxaxis_cliptwo;
  QHBox*         fxaxis_object;      QHBox*         fxaxis_world;
  QHBox*         fxaxis_text;

  QFrame*        ControlFrame1;
  QFrame*        ControlFrame0;
  QFrame*        sceneframe;
  QLabel*        RotateText_1;
  QLabel*        TranslateText_1;
  QLabel*        ScaleText_1;

  QSlider        *RotX_1,    *RotY_1,    *RotZ_1;
  InputLCD       *RotXlcd_1, *RotYlcd_1, *RotZlcd_1;
  QLabel         *RotX_1H,   *RotY_1H,   *RotZ_1H;

  QSlider*       TranX_1;       InputLCD*  TranXlcd_1;
  QSlider*       TranY_1;       InputLCD*  TranYlcd_1;
  QSlider*       TranZ_1;       InputLCD*  TranZlcd_1;

  QSlider*       ScaleX_1;      InputLCD*  ScaleXlcd_1;
  QSlider*       ScaleY_1;      InputLCD*  ScaleYlcd_1;
  QSlider*       ScaleZ_1;      InputLCD*  ScaleZlcd_1;

  QCheckBox*     ShowKeySelect;

  //world control variables
  QFrame*        ControlFrame4;
  QLabel*        RotateText_4;
  QLabel*        TranslateText_4;
  QLabel*        ScaleText_4;

  QSlider        *RotX_4, *RotY_4, *RotZ_4;
  InputLCD       *RotXlcd_4, *RotYlcd_4, *RotZlcd_4;

  QSlider        *TranX_4, *TranY_4, *TranZ_4;
  InputLCD       *TranXlcd_4, *TranYlcd_4, *TranZlcd_4;

  QSlider*       ScaleX_4;      InputLCD*  ScaleXlcd_4;

  QCheckBox*     GridSelect;

  //widget selection sidebar variables
  QFrame*        ControlFrame;       //QWidgetStack*  fxobject;
  QHBox*         fxobject_clip;      QHBox*         fxobject_text;
  QHBox*         fxobject_world;     QHBox*         fxobject_key;
  QHBox*         fxobject_particle;  QHBox*         fxobject_object;
  QHBox*		   fxobject_maxobject;

  //texture options
  QFrame*        ControlFrame2;
  QCheckBox*     ClipSelect;

  //world option box
  QFrame*        ControlFrame3;

  //layer options
  QFrame*        ControlFrame5;
  QLabel*        ClipLabel;

  QPushButton*   loadimage_1;        QPushButton*   grabdesktopClip; QPushButton*   grabdesktopKey;
  QToolButton*   lighticon;          QLabel*        light1text;      QCheckBox*     light1;
  QLabel*        Transparency_1;     QSlider*       Trans_1;         InputLCD*   Translcd_1;

  //key options
  QFrame*        ControlFrame8;
  QPushButton*   loadimage_2;        QPushButton*  keyimage_2;       QPushButton*  loadkey; QPushButton*  keyimage_3;
  QCheckBox*     KeySelect;

  QSlider*      Trans_2;          QLCDNumber*   Translcd_2;
  QPushButton* keyStatusBox;

  //video labels
  QLabel*			VideoLabel;
  InputLCD*		FramesLcd;		InputLCD*		inFramesLcd;	InputLCD*		outFramesLcd;
  InputLCD*		KeyFramesLcd;	InputLCD*		inKeyFramesLcd;	InputLCD*		outKeyFramesLcd;
  QToolButton*    lockicon;
  //font options
  QFrame*        ControlFrame6;
  QLabel*        ScaleText_5;        QSlider*      ScaleX_5;         InputLCD*  ScaleXlcd_5;
  QLineEdit*     textEd;
  QPushButton*   textFont;           QPushButton*  textColor;        QPushButton*  textDraw;

  //particles
  QFrame*        ControlFrame7;      QPushButton * loadimage_3, * loadimage_4, *  particlestyle;

  //objects
  QFrame*        ControlFrame9;      QPushButton * loadobject_1, *  objectstyle;

  QFrame*        ControlFrame10;     QPushButton*  loadmaxobject_1;
  QFrame*        ControlFrame11;     QCheckBox*    ForegroundSelect;

  QPushButton*	ObjectLayer;
  QPushButton*	ObjectCube;
  QPushButton*	ObjectCylinder;
  QPushButton*	ObjectSphere;
  QPushButton*	ObjectBezier;
  QPushButton*	ObjectMesh;


};



#endif // ANIMATION_H





