/*******************************************************************************
**
** The source file for the Jahshaka animation interface module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "anime.h"

#include "Pixmaps/light.xpm"

void GLAnime::buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller) {

  //left menu here
  glworldLeft = new QFrame (leftcontroller, "gl left");
  //glworldLeft->setFixedSize( QSize( 30, 300 ) );

  tooltranslate = new QPushButton( ( QIconSet( QPixmap("Pixmaps/translate.xpm"), QPixmap("Pixmaps/translate.xpm") )) ,"", glworldLeft, "play" );
  tooltranslate->setGeometry( QRect( 0, 205, 30, 27 ) );
  tooltranslate->setFlat(true);

  toolrotate = new QPushButton( ( QIconSet( QPixmap("Pixmaps/rotate.xpm"), QPixmap("Pixmaps/rotate.xpm") )) ,"", glworldLeft, "rotate" );
  toolrotate->setGeometry( QRect( 0, 233, 30, 27 ) );
  toolrotate->setFlat(true);

  toolscale = new QPushButton( ( QIconSet( QPixmap("Pixmaps/scale.xpm"), QPixmap("Pixmaps/scale.xpm") )) ,"", glworldLeft, "scale" );
  toolscale->setGeometry( QRect( 0, 263, 30, 27 ) );
  toolscale->setFlat(true);

  //added for siggraph demo
  toolphoto = new QPushButton( ( QIconSet( QPixmap("Pixmaps/scale.xpm"), QPixmap("Pixmaps/scale.xpm") )) ,"", glworldLeft, "scale" );
  toolphoto->setGeometry( QRect( 0, 300, 30, 27 ) );
  toolphoto->setFlat(true);
  connect( toolphoto, SIGNAL(clicked()), SLOT( SoftRender()) );   //change this mon


  //rightmenu here
  glworldRight =  new QFrame (rightcontroller, "gl right" );
  // glworldRight->setFixedSize( QSize( 30, 300 ) );

  toolzoomin = new QPushButton( ( QIconSet( QPixmap("Pixmaps/zoomin.xpm"), QPixmap("Pixmaps/zoomin.xpm") )) ,"", glworldRight, "zoomin" );
  toolzoomin->setGeometry( QRect( 0, 115, 30, 27 ) );
  toolzoomin->setFlat(true);

  zoom = new QSlider( glworldRight, "scrubslider" );
  zoom->setStyle( new QPlatinumStyle );
  zoom->setGeometry( QRect( 0, 150, 30, 200 ) );
  zoom->setOrientation( QSlider::Vertical );
  zoom->setMinValue( -10 );    zoom->setMaxValue( 10 );    zoom->setValue   ( 0 );

  toolzoomout = new QPushButton( ( QIconSet( QPixmap("Pixmaps/zoomout.xpm"), QPixmap("Pixmaps/zoomout.xpm") )) ,"", glworldRight, "zoomout" );
  toolzoomout->setGeometry( QRect( 0, 355, 30, 27 ) );
  toolzoomout->setFlat(true);

  //connect tools to object and add object to display widget
  connect( tooltranslate,  SIGNAL(clicked()), this, SLOT(toolTranslate() )  );
  connect( toolscale,      SIGNAL(clicked()), this, SLOT(toolScale() )  );
  connect( toolrotate,     SIGNAL(clicked()), this, SLOT(toolRotate() )  );

  connect( zoom,  SIGNAL(sliderMoved(int)), this, SLOT(changeZoom(int) )  );

}

void GLAnime::buildInterface( QHBox* qtinterface ) {

  ///////////////////////////////////////////////////////////
  //The layers interface
  LayersFrame = new QFrame( qtinterface, "EffectsFrame" );
  LayersFrame->setGeometry( QRect( 0, 0, 60, 195 ) );
  LayersFrame->setMinimumSize( QSize( 60, 195 ) );  //was 140
  LayersFrame->setMaximumSize( QSize( 60, 195 ) );

  ///////////////////////////////////////////////////////////
  //The layer options
  AddButton = new QPushButton( LayersFrame, "AddButton" );
  addButton( AddButton,  0, 5, 55, 25,  "Add" );
  connect  ( AddButton,  SIGNAL(clicked()), this, SLOT( addLayer())  );

  NameButton = new QPushButton( LayersFrame, "NameButton" );
  addButton( NameButton, 0, 30, 55, 25, "Name" );
  connect  ( NameButton,  SIGNAL(clicked()), this, SLOT( nameLayer())  );

  namepopup = new FancyPopup( this, "Enter Layer Name",170,90 );  //send size and position as well
  connect ( namepopup,      SIGNAL( returnText(QString)), SLOT( setlayerName(QString)) );

  DelButton = new QPushButton( LayersFrame, "Delete Button" );
  addButton( DelButton, 0, 60, 55, 25 , "Del" );
  connect  ( DelButton,  SIGNAL(clicked()), this, SLOT( delLayer())  );

  //////////////////////////////////////////////////////////////
  // set up the scene options, or prefernces
  EffectsMain = new QWidgetStack(qtinterface);
  SceneFrame   = new QFrame(EffectsMain);
  EffectsFrame = new QFrame(EffectsMain);

  EffectsMain->addWidget(SceneFrame,0);
  EffectsMain->addWidget(EffectsFrame,1);
  EffectsMain->setGeometry( QRect( 0, 0, 855, 195) );

  EffectsStatus = FALSE; EffectsMain->raiseWidget(1);

  /////////////////////////////////////////////////////////
  //this is the list-view
  lv = new QListView( EffectsFrame );

  connect( lv, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( clicked( QListViewItem* ) ) );

  lv->setGeometry( QRect( 2, 2, 146, 180 ) );
  lv->addColumn( "LAYERS          ", -1 );
  lv->setSorting( -1,1 );   // disables the autosort


	//////////////////////////////////////////////////////////
	// set up options here

	controlpanel = new QWidgetStack(EffectsFrame);

  //this may need to be QHBox as it is below...
  scene_controls  = new QFrame(controlpanel);     controlpanel->addWidget(scene_controls,0);
  object_controls = new QFrame(controlpanel);     controlpanel->addWidget(object_controls,1);
  	
  controlpanel->setGeometry( QRect( 148, 1, 700, 180) );

  /////////////////////////////////////////////////////////
  //Set up scene interface

  sceneframe = new QFrame( scene_controls, "scenecontrols" );
  sceneframe->setGeometry( QRect( 1, 1, 700, 180 ) );
  sceneframe->setStyle( new QPlatinumStyle );

  //fxobject->setGeometry( QRect( 0, 0, 700, 120) );   // was 635
  /*
  LoadButton = new QPushButton( sceneframe, "LoadButton" );
  addButton( LoadButton, 10, 10, 86, 25 , "Load" );
  connect  ( LoadButton,  SIGNAL(clicked()), this, SLOT( SceneLoad())  );

  // clean up this mess here!
  */

  /////////////////////////////////////////////////////////
  //Set up tabbed interface

  // tabframe = new QTabWidget( EffectsFrame, "axiscontrols" );
  tabframe = new QTabWidget( object_controls, "axiscontrols" );
  tabframe->setGeometry( QRect( 1, 2, 700, 180 ) );
  tabframe->setTabShape( QTabWidget::Triangular );
  tabframe->setAutoMask( FALSE );
  tabframe->setStyle( new QPlatinumStyle );


  //this has become the settings panel need to update variable names
  scenecontrols = new QWidget( EffectsFrame, "scenesettings" );
  tabframe->insertTab( scenecontrols, tr( "  SCENE  " ) );

  objectcontrols = new QWidget( EffectsFrame, "objectcontrols" );
  tabframe->insertTab( objectcontrols, tr( "  OBJECT  " ) );

  axiscontrols = new QWidget( EffectsFrame, "axiscontrols" );
  tabframe->insertTab( axiscontrols, tr( "  AXIS  " ) );

  texturecontrols = new QWidget( EffectsFrame, "texturecontrols" );
  tabframe->insertTab( texturecontrols, tr( "    CLIP    " ) );

  effectscontrols = new QWidget( EffectsFrame, "effectscontrols" );
  tabframe->insertTab( effectscontrols, tr( "  EFFECTS  " ) );



  /////////////////////////////////////////////////////////////////////
  // the layout boxes for different types of layers
  // these should be changed and regroups to represent tabs above
  // these should be arrays

  // the axis control tab bar
  // this is just really world and layer now
  // we can get rid of the rest

    fxaxis = new QWidgetStack(axiscontrols);

     fxaxis_clipone = new QHBox(fxaxis);     fxaxis->addWidget(fxaxis_clipone,0);
     fxaxis_cliptwo = new QHBox(fxaxis);     fxaxis->addWidget(fxaxis_cliptwo,1);
     fxaxis_object  = new QHBox(fxaxis);     fxaxis->addWidget(fxaxis_object, 2);
     fxaxis_world   = new QHBox(fxaxis);     fxaxis->addWidget(fxaxis_world,  3);
     fxaxis_text    = new QHBox(fxaxis);     fxaxis->addWidget(fxaxis_text,   4);

    fxaxis->setGeometry( QRect( 0, 0, 700, 180));


    ///////////////////////////////////////////////////////////////////
    // the object control tab bar
	  // this is the object control panel

    fxobject = new QWidgetStack(objectcontrols);

     fxobject_clip     = new QHBox(fxobject);          fxobject->addWidget(fxobject_clip,    0);
     fxobject_text     = new QHBox(fxobject);          fxobject->addWidget(fxobject_text,    1);
     fxobject_world    = new QHBox(fxobject);          fxobject->addWidget(fxobject_world,   2);
     fxobject_key      = new QHBox(fxobject);          fxobject->addWidget(fxobject_key,     3);
     fxobject_particle = new QHBox(fxobject);          fxobject->addWidget(fxobject_particle,4);
     fxobject_object   = new QHBox(fxobject);          fxobject->addWidget(fxobject_object,  5);
	   fxobject_maxobject= new QHBox(fxobject);          fxobject->addWidget(fxobject_maxobject,  6);

    fxobject->setGeometry( QRect( 0, 0, 700, 180));   // was 635


    QPalette p( QColor( 0, 0, 0 ) );
    p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );

    ///////////////////////////////////////////////////////////////////
    // the scene control tab bar

    //mover this into the layers...

    ControlFrame0 = new QFrame( scenecontrols, "ControlFrame3" );
    ControlFrame0->setGeometry( QRect( 0, 0, 660, 120 ) );

    LoadButton = new QPushButton( ControlFrame0, "LoadButton" );
    addButton( LoadButton, 10, 10, 86, 25 , "Load" );
    connect  ( LoadButton,  SIGNAL(clicked()), this, SLOT( SceneLoad())  );

    SaveButton = new QPushButton( ControlFrame0, "SaveButton" );
    addButton( SaveButton, 10, 45, 86, 25, "Save" );
    connect  ( SaveButton,  SIGNAL(clicked()), this, SLOT( SceneSave())  );

    ResetButton = new QPushButton( ControlFrame0, "ResetButton" );
    addButton( ResetButton, 10, 80, 86, 25, "Reset All" );
    connect  ( ResetButton,  SIGNAL(clicked()), this, SLOT( ResetAll())  );

    ClearButton = new QPushButton( ControlFrame0, "ClearButton" );
    addButton( ClearButton, 110, 10, 86, 25, "Clear Scene" );
    connect( ClearButton,  SIGNAL(clicked()), this, SLOT( ClearAll())  );

  //need to move this into the module
    //this is mandatory for all modules...

    sframetext = new QLabel( ControlFrame0, "sframetext" );
    sframetext->setGeometry( QRect( 400, 4, 80, 27 ) );
    sframetext->setText( tr( "STARTFRAME" ) );
    sframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont sframetext_font(  sframetext->font() );
    sframetext_font.setPointSize( 9 );
    sframetext->setFont( sframetext_font );

    lcdstartframe = new InputLCD( ControlFrame0, "startframe",9 );
	  addSliderLcd(lcdstartframe, 500, 5, 90, 21, Astartframe);
	  lcdstartframe->setMinMaxInt( Astartframe, Aendframe-1 );
    lcdstartframe->setValue(Astartframe);
	  connect( lcdstartframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderStartframe(int)) );

    eframetext = new QLabel( ControlFrame0, "eframetext" );
    eframetext->setGeometry( QRect( 400, 34, 80, 27 ) );
    eframetext->setText( tr( "ENDFRAME"  ) );
    eframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont eframetext_font(  eframetext->font() );
    eframetext_font.setPointSize( 9 );
    eframetext->setFont( eframetext_font );

//addSliderLcd(animframe, 5, 0, 90, 31, 100);
    lcdendframe = new InputLCD( ControlFrame0, "endframe",9 );
	  addSliderLcd(lcdendframe, 500, 34, 90, 21, Aendframe);
	  lcdendframe->setMinMaxInt( Astartframe+1, MaxFrames );
    lcdendframe->setValue(Aendframe);
	  connect( lcdendframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderEndframe(int)) );

    ///////////////////////////////////////////////////////////////////////
    //set up world control set

    ControlFrame3 = new QFrame( fxobject_world, "ControlFrame3" );
    ControlFrame3->setGeometry( QRect( 0, 0, 660, 120 ) );


    TextSelect = new QCheckBox( ControlFrame3, "TitleSafe" );
    addCheckButton( TextSelect, 300, 80, 110, 21, "title safe" );
    connect( TextSelect,  SIGNAL(clicked()), this, SLOT(setTextSafe())  );

    GridSelect = new QCheckBox( ControlFrame3, "Grid" );
    addCheckButton( GridSelect, 300, 40, 110, 21, "grid" );
    connect( GridSelect,  SIGNAL(clicked()), this, SLOT(setGrid())  );

    SmoothSelect = new QCheckBox( ControlFrame3, "SmoothSelect" );
    addCheckButton( SmoothSelect, 300, 10, 110, 21, "aliasing" );
    connect( SmoothSelect,  SIGNAL(clicked()), this, SLOT(setSmooth())  );

    BlurSelect = new QCheckBox( ControlFrame3, "BlurSelect" );
    addCheckButton( BlurSelect, 10, 10, 110, 21, "motion blur" );
    connect( BlurSelect,  SIGNAL(clicked()), this, SLOT(setBlur())  );

    ResetWorldButton = new QPushButton( ControlFrame3, "ResetLayerButton" );
    addButton( ResetWorldButton, 410, 10, 86, 25, "Reset" );
    connect  ( ResetWorldButton,  SIGNAL(clicked()), this, SLOT( ResetLayer())  );


	


   /////////////////////////////////////////////////////////////////////////////
    //world control sliders
    ControlFrame4 = new QFrame( fxaxis_world, "ControlFrame4" );
    ControlFrame4->setGeometry( QRect( 0, 0, 660, 120 ) );

    // set up rotate sliders
    RotateText_4 = new QLabel( ControlFrame4, "RotateText_4" );
    addLabel( RotateText_4, 10, 10, 110, 21, "ROTATE", 9);

    RotX_4 = new QSlider( ControlFrame4 );   RotXlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, RotX_4, 10, 30, 10, 21,  500, RotXlcd_4,  "X", 9);
    connect( RotX_4, SIGNAL(valueChanged(int)),     this,       SLOT(setWorldXR(int)) );
    connect( RotX_4, SIGNAL(valueChanged(int)),     RotXlcd_4,  SLOT(display(int)) );
	connect(RotXlcd_4, SIGNAL(valueChanged(int)),   this,       SLOT(setWorldXR(int)) );

    RotY_4 = new QSlider( ControlFrame4 );   RotYlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, RotY_4, 10, 60, 10, 21,  500, RotYlcd_4,  "Y", 9);
    connect( RotY_4, SIGNAL(valueChanged(int)),     this,       SLOT(setWorldYR(int)) );
    connect( RotY_4, SIGNAL(valueChanged(int)),     RotYlcd_4,  SLOT(display(int)) );
	connect(RotYlcd_4, SIGNAL(valueChanged(int)),   this,       SLOT(setWorldYR(int)) );

    RotZ_4 = new QSlider( ControlFrame4 );   RotZlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, RotZ_4, 10, 90, 10, 21,  500, RotZlcd_4,  "Z", 9);
    connect( RotZ_4, SIGNAL(valueChanged(int)),     this,       SLOT(setWorldZR(int)) );
	connect( RotZ_4, SIGNAL(valueChanged(int)),     RotZlcd_4,  SLOT(display(int)) );
    connect( RotZlcd_4, SIGNAL(valueChanged(int)),  this,       SLOT(setWorldZR(int)) );


    // set up translate sliders
    TranslateText_4 = new QLabel( ControlFrame4, "TranslateText_4" );
    addLabel( TranslateText_4, 200, 10, 110, 21, "TRANSLATE", 9);

    TranX_4 = new QSlider( ControlFrame4 );   TranXlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, TranX_4, 200, 30, 10, 21,  500, TranXlcd_4,  "X", 9);
    connect( TranX_4, SIGNAL(valueChanged(int)),    this,       SLOT(setWorldXT(int)) );
    connect( TranX_4, SIGNAL(valueChanged(int)),    TranXlcd_4, SLOT(display(int)) );
    connect( TranXlcd_4, SIGNAL(valueChanged(int)), this,       SLOT(setWorldXT(int)) );

    TranY_4 = new QSlider( ControlFrame4 );   TranYlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, TranY_4, 200, 60, 10, 21,  500, TranYlcd_4,  "Y", 9);
    connect( TranY_4, SIGNAL(valueChanged(int)),    this,       SLOT(setWorldYT(int)) );
    connect( TranY_4, SIGNAL(valueChanged(int)),    TranYlcd_4, SLOT(display(int)) );
    connect( TranYlcd_4, SIGNAL(valueChanged(int)), this,       SLOT(setWorldYT(int)) );

    TranZ_4 = new QSlider( ControlFrame4 );   TranZlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, TranZ_4, 200, 90, 10, 21,  500, TranZlcd_4,  "Z", 9);
    connect( TranZ_4, SIGNAL(valueChanged(int)),    this,       SLOT(setWorldZT(int)) );
    connect( TranZ_4, SIGNAL(valueChanged(int)),    TranZlcd_4, SLOT(display(int)) );
    connect( TranZlcd_4, SIGNAL(valueChanged(int)), this,       SLOT(setWorldZT(int)) );

    // set up zoom slider
    //ScaleText_4 = new QLabel( ControlFrame4, "ScaleText_4" );
    //addLabel( ScaleText_4, 370, 10, 110, 21, "ZOOM", 9);

    //ScaleX_4 = new QSlider( ControlFrame4 );   ScaleXlcd_4 = new InputLCD( ControlFrame4 );
    //addSliderLcdLabel( ControlFrame4, ScaleX_4, 370, 30, 10, 21,  100, ScaleXlcd_4,  "Z", 9);
    //connect( ScaleX_4, SIGNAL(valueChanged(int)), ScaleXlcd_4, SLOT(display(int)) );


    ///////////////////////////////////////////////////////////////
    // object layer options
    ControlFrame5 = new QFrame( fxobject_clip, "ControlFrame5" );
    ControlFrame5->setGeometry( QRect( 0, 0, 660, 120 ) );

	//the different object types

    ObjectLayer = new QPushButton( ControlFrame5, "Layer" );
    addButton( ObjectLayer, 10, 10, 86, 25 , "Layer" );
    connect  ( ObjectLayer,  SIGNAL(clicked()), this, SLOT( ChangeObjectLayer())  );

    ObjectCube = new QPushButton( ControlFrame5, "Cube" );
    addButton( ObjectCube, 10, 45, 86, 25, "Cube" );
    connect  ( ObjectCube,  SIGNAL(clicked()), this, SLOT( ChangeObjectCube())  );

    ObjectCylinder = new QPushButton( ControlFrame5, "Cylinder" );
    addButton( ObjectCylinder, 10, 80, 86, 25, "Cylinder" );
    connect  ( ObjectCylinder,  SIGNAL(clicked()), this, SLOT( ChangeObjectCylinder())  );

    ObjectSphere = new QPushButton( ControlFrame5, "Sphere" );
    addButton( ObjectSphere, 110, 10, 86, 25, "Sphere" );
    connect( ObjectSphere,  SIGNAL(clicked()), this, SLOT( ChangeObjectSphere())  );

    ObjectBezier = new QPushButton( ControlFrame5, "Bezier" );
    addButton( ObjectBezier, 110, 45, 86, 25, "Bezier" );
    connect( ObjectBezier,  SIGNAL(clicked()), this, SLOT( ChangeObjectBezier())  );

    ObjectMesh = new QPushButton( ControlFrame5, "Mesh" );
    addButton( ObjectMesh, 110, 80, 86, 25, "Mesh" );
    connect( ObjectMesh,  SIGNAL(clicked()), this, SLOT( ChangeObjectMesh())  );

    ResetLayerButton = new QPushButton( ControlFrame5, "ResetLayerButton" );
    addButton( ResetLayerButton, 410, 10, 86, 25, "Reset" );
    connect  ( ResetLayerButton,  SIGNAL(clicked()), this, SLOT( ResetLayer())  );


    ///////////////////////////////////////////////////////////////////////
    //set up object control set and sliders

    ControlFrame1 = new QFrame( fxaxis_clipone, "ControlFrame1" );
    ControlFrame1->setGeometry( QRect( 0, 0, 760, 120 ) );    // was 510

    // set up rotate sliders
    RotateText_1 = new QLabel( ControlFrame1, "RotateText_1" );
    addLabel( RotateText_1, 10, 10, 110, 21, "ROTATE", 9);

    RotX_1 = new QSlider( ControlFrame1 );   RotXlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, RotX_1, 0, 30, 10, 21,  500, RotXlcd_1,  "X", 9);
    connect( RotX_1, SIGNAL(valueChanged(int)), this, SLOT(setXRotation(int)) );
    connect( RotX_1, SIGNAL(valueChanged(int)), RotXlcd_1, SLOT(display(int)) );
	  connect(RotXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXRotation(int)) );

    RotY_1 = new QSlider( ControlFrame1 );   RotYlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, RotY_1, 0, 60, 10, 21,  500, RotYlcd_1,  "Y", 9);
    connect( RotY_1, SIGNAL(valueChanged(int)), this, SLOT(setYRotation(int)) );
    connect( RotY_1, SIGNAL(valueChanged(int)), RotYlcd_1, SLOT(display(int)) );
	  connect(RotYlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setYRotation(int)) );

    RotZ_1 = new QSlider( ControlFrame1 );   RotZlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, RotZ_1, 0, 90, 10, 21,  500, RotZlcd_1,  "Z", 9);
    connect( RotZ_1, SIGNAL(valueChanged(int)), this, SLOT(setZRotation(int)) );
    connect( RotZ_1, SIGNAL(valueChanged(int)), RotZlcd_1, SLOT(display(int)) );
	  connect(RotZlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setZRotation(int)) );


    // set up translate sliders
    TranslateText_1 = new QLabel( ControlFrame1, "TranslateText_1" );
    addLabel( TranslateText_1, 180, 10, 110, 21, "TRANSLATE", 9);

    TranX_1 = new QSlider( ControlFrame1 );   TranXlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, TranX_1, 180, 30, 10, 21,  500, TranXlcd_1,  "X", 9);
    connect( TranX_1,   SIGNAL(valueChanged(int)), this, SLOT(setXTrans(int)) );
    connect( TranX_1,   SIGNAL(valueChanged(int)), TranXlcd_1, SLOT(display(int)) );
	  connect(TranXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXTrans(int)) );

    TranY_1 = new QSlider( ControlFrame1 );   TranYlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, TranY_1, 180, 60, 10, 21,  500, TranYlcd_1,  "Y", 9);
    connect( TranY_1, SIGNAL(valueChanged(int)), this, SLOT(setYTrans(int)) );
    connect( TranY_1, SIGNAL(valueChanged(int)), TranYlcd_1, SLOT(display(int)) );
	  connect(TranYlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setYTrans(int)) );

    TranZ_1 = new QSlider( ControlFrame1 );   TranZlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, TranZ_1, 180, 90, 10, 21,  500, TranZlcd_1,  "Z", 9);
    connect( TranZ_1,   SIGNAL(valueChanged(int)), this, SLOT(setZTrans(int)) );
    connect( TranZ_1,   SIGNAL(valueChanged(int)), TranZlcd_1, SLOT(display(int)) );
	  connect(TranZlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setZTrans(int)) );


    // set up scale sliders
    ScaleText_1 = new QLabel( ControlFrame1, "ScaleText_1" );
    addLabel( ScaleText_1, 350, 10, 110, 21, "SCALE", 9);

    ScaleX_1 = new QSlider( ControlFrame1 );   ScaleXlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, ScaleX_1, 350, 30, 10, 21,  100, ScaleXlcd_1,  "X", 9);
    connect( ScaleX_1, SIGNAL(valueChanged(int)), this, SLOT(setXScale(int)) );
    connect( ScaleX_1, SIGNAL(valueChanged(int)), ScaleXlcd_1, SLOT(display(int)) );
	  connect(ScaleXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXScale(int)) );

    ScaleY_1 = new QSlider( ControlFrame1 );   ScaleYlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, ScaleY_1, 350, 60, 10, 21,  100, ScaleYlcd_1,  "Y", 9);
    connect( ScaleY_1, SIGNAL(valueChanged(int)), this, SLOT(setYScale(int)) );
    connect( ScaleY_1, SIGNAL(valueChanged(int)), ScaleYlcd_1, SLOT(display(int)) );
	  connect(ScaleYlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setYScale(int)) );

    ScaleZ_1 = new QSlider( ControlFrame1 );   ScaleZlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, ScaleZ_1, 350, 90, 10, 21,  100, ScaleZlcd_1,  "Z", 9);
    connect( ScaleZ_1, SIGNAL(valueChanged(int)), this, SLOT(setZScale(int)) );
    connect( ScaleZ_1, SIGNAL(valueChanged(int)), ScaleZlcd_1, SLOT(display(int)) );
	  connect(ScaleZlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setZScale(int)) );


    ShowKeySelect = new QCheckBox( ControlFrame1, "KeySelect" );
    addCheckButton( ShowKeySelect, 510, 30, 145, 25, "Show Keyframes" );
    connect( ShowKeySelect,  SIGNAL(clicked()), this, SLOT(setKeyframeDrawStatus())  ); //for clip

    ///////////////////////////////////////////////////////////////
    // texture layer options
	// need to add video-texture options here

	//image options

    ControlFrame2 = new QFrame( texturecontrols, "ControlFrame5" );
    ControlFrame2->setGeometry( QRect( 0, 0, 660, 120 ) );

    ClipSelect = new QCheckBox( ControlFrame2, "ClipSelect" );
    addCheckButton( ClipSelect, 0, 30, 45, 25, "Clip" );
    connect( ClipSelect,  SIGNAL(clicked()), this, SLOT(setClipStatus())  ); //for clip

    grabdesktopClip = new QPushButton( ControlFrame2, "grabdesktop" );
    addButton( grabdesktopClip, 45, 30, 110, 25, "Grab Desktop" );
    connect( grabdesktopClip,  SIGNAL(clicked()), this, SLOT(grabDesktop())  );

    keyimage_2 = new QPushButton( ControlFrame2, "keyOne_2" );
    addButton( keyimage_2, 165, 30, 35, 25, "IM" );
    connect( keyimage_2,  SIGNAL(clicked()), this, SLOT(keyCliplayer())  );


    KeySelect = new QCheckBox( ControlFrame2, "KeySelect" );
    addCheckButton( KeySelect, 0, 75, 45, 25, "Key" );
    connect( KeySelect,  SIGNAL(clicked()), this, SLOT(setKeyStatus())  );

    grabdesktopKey = new QPushButton( ControlFrame2, "grabdesktop" );
    addButton( grabdesktopKey, 45, 75, 110, 25, "Grab Desktop" );
    connect( grabdesktopKey,  SIGNAL(clicked()), this, SLOT(grabDesktopKey())  ); //for key...

    keyimage_3 = new QPushButton( ControlFrame2, "keyOne_3" );
    addButton( keyimage_3, 165, 75, 35, 25, "KM" );
    connect( keyimage_3,  SIGNAL(clicked()), this, SLOT(keyClipkey())  );

    keyStatusBox = new QPushButton( ControlFrame2, "Select1" );
    addButton( keyStatusBox, 220, 75, 35, 25, "Invert" );
    connect( keyStatusBox,  SIGNAL(clicked()), this, SLOT(InvertKeyData())  );

	// video options
	// need to hook this up now...

  VideoLabel = new QLabel( ControlFrame2, "TranslateText_1" );
  addLabel( VideoLabel, 380, 10, 110, 21, "VIDEO OPTIONS", 9);


  FramesLcd = new InputLCD( ControlFrame2 );
  addLabelLcd( ControlFrame2, FramesLcd,       380, 30, 60, 21,  0,  "Frames", 8);

  inFramesLcd = new InputLCD( ControlFrame2 );
  addLcdLabel( ControlFrame2, inFramesLcd,     460, 30, 30, 21,  0,  "In", 8);
  connect(inFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setinFrames(int)) );

  outFramesLcd = new InputLCD( ControlFrame2 );
  addLcdLabel( ControlFrame2, outFramesLcd,    530, 30, 30, 21,  0,  "Out", 8);
  connect(outFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setoutFrames(int)) );

  KeyFramesLcd = new InputLCD( ControlFrame2 );
  addLabelLcd( ControlFrame2, KeyFramesLcd,    380, 75, 60, 21,  0,  "Frames", 8);

  inKeyFramesLcd = new InputLCD( ControlFrame2 );
  addLcdLabel( ControlFrame2, inKeyFramesLcd,  460, 75, 30, 21,  0,  "In", 8);
  connect(inKeyFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setinKeyFrames(int)) );

  outKeyFramesLcd = new InputLCD( ControlFrame2 );
  addLcdLabel( ControlFrame2, outKeyFramesLcd, 530, 75, 30, 21,  0,  "Out", 8);
  connect(outKeyFramesLcd, SIGNAL(valueChanged(int)), this, SLOT(setoutKeyFrames(int)) );

    //need to make the icon a lock... 3x current height as well for links
  	lockicon	= new QToolButton(ControlFrame2,"lockicon");
    lockicon->setGeometry( QRect( 600, 55, 18, 26 ) );
  	lockicon->setFixedSize(18, 26);  	//lighticon->setUsesBigPixmap(true);
  	lockicon->setAutoRaise(true);

  	QIconSet	*icoSet2;
    	icoSet2 = new QIconSet(QPixmap((const char**)light_xpm) );
    	icoSet2->setPixmap(QPixmap((const char**)light_xpm), QIconSet::Large, QIconSet::Active);
    	icoSet2->setPixmap(QPixmap((const char**)light_xpm), QIconSet::Large, QIconSet::Disabled);
    	lockicon->setIconSet(*icoSet2);
    delete icoSet2;

    /////////////////font options
    ControlFrame6 = new QFrame( fxobject_text, "ControlFrame4" );
    ControlFrame6->setGeometry( QRect( 0, 0, 40, 120 ) );

    ScaleText_5 = new QLabel( ControlFrame6, "ScaleText_4" );
    addLabel( ScaleText_5, 0, 10, 110, 21, "EXTRUDE", 9);

    ScaleX_5 = new QSlider( ControlFrame6 ); ScaleXlcd_5 = new InputLCD( ControlFrame6 );
    addSliderLcdLabel( ControlFrame6, ScaleX_5,   300, 30, 10, 21,  1, 200,  30,  ScaleXlcd_5,  "E", 9);

    connect( ScaleX_5,    SIGNAL(valueChanged(int)), ScaleXlcd_5, SLOT(display(int)) );
    connect( ScaleX_5,    SIGNAL(valueChanged(int)), this, SLOT(setExtrude(int)) );
    connect( ScaleXlcd_5, SIGNAL(valueChanged(int)), this, SLOT(setExtrude(int)) );

    textDraw = new QPushButton( ControlFrame6, "draw" );
    addButton( textDraw, 115, 30, 36, 21, "Draw" );
    connect( textDraw,  SIGNAL(clicked()), this, SLOT(changeFontDraw())  );

    textEd	= new QLineEdit( ControlFrame6, "text" );
    textEd->setGeometry( QRect( 0, 60, 150, 21 ) );
    textEd->setFocus();    textEd->setText( "jahshaka" );
    connect( textEd, SIGNAL(textChanged(const QString&)), this, SLOT(setText(const QString&)) );

    textFont = new QPushButton( ControlFrame6, "font" );
    addButton( textFont, 0, 90, 55, 21, "Font" );
    connect( textFont,  SIGNAL(clicked()), this, SLOT(loadFont())  );

    textColor = new QPushButton( ControlFrame6, "color" );
    addButton( textColor, 60, 90, 55, 21, "Color" );
    connect( textColor,  SIGNAL(clicked()), this, SLOT(setFontColor())  );

    ///////////////////////////////////////////////////////
    //particle options

    ControlFrame7 = new QFrame( fxobject_particle, "ControlFrame4" );
    ControlFrame7->setGeometry( QRect( 0, 0, 40, 120 ) );

    loadimage_3 = new QPushButton( ControlFrame7, "emitter" );
    addButton( loadimage_3, 0, 20, 111, 20,  "Emitter" );
    connect( loadimage_3,  SIGNAL(clicked()), this, SLOT(changeParticle())  );

    loadimage_4 = new QPushButton( ControlFrame7, "style" );
    addButton( loadimage_4, 0, 60, 111, 20,  "Style" );
    connect( loadimage_4,  SIGNAL(clicked()), this, SLOT(changeParticleDraw())  );

    particlestyle = new QPushButton( ControlFrame7, "tex" );
    addButton( particlestyle, 115, 20, 35, 20, "Color" );
    connect( particlestyle,  SIGNAL(clicked()), this, SLOT(changeParticleColors())  );

    ////////////////////////////////////////////////////////
    //object options

    ControlFrame9 = new QFrame( fxobject_object, "ControlFrame4" );
    ControlFrame9->setGeometry( QRect( 0, 0, 40, 120 ) );


    loadobject_1 = new QPushButton( ControlFrame9, "load" );
    addButton( loadobject_1, 0, 20, 111, 30, "Load Object" );
    connect( loadobject_1,  SIGNAL(clicked()), this, SLOT(loadObject())  );

    objectstyle = new QPushButton( ControlFrame9, "draw" );
    addButton( objectstyle, 115, 20, 35, 30, "fill" );
    connect( objectstyle,  SIGNAL(clicked()), this, SLOT(changeObjectDraw())  );


    ////////////////////////////////////////////////////////
    //max object options

    ControlFrame10 = new QFrame( fxobject_maxobject, "ControlFrame4" );
    ControlFrame10->setGeometry( QRect( 0, 0, 40, 120 ) );

    loadmaxobject_1 = new QPushButton( ControlFrame10, "load" );
    addButton( loadmaxobject_1, 0, 20, 111, 30, "Load MObject" );
    connect( loadmaxobject_1,  SIGNAL(clicked()), this, SLOT(loadMaxObject())  );


    ////////////////////////////////////////////////////////
    //effects options

    ControlFrame8 = new QFrame( effectscontrols, "ControlFrame5" );
    ControlFrame8->setGeometry( QRect( 0, 0, 660, 120 ) );

	//toggle foreground background status
    ForegroundSelect = new QCheckBox( ControlFrame8, "foreground" );
    addCheckButton( ForegroundSelect, 10, 10, 110, 21, "foreground" );
    connect( ForegroundSelect,  SIGNAL(clicked()), this, SLOT(toggleForegroundStatus())  );

    //add a subroutine for creating icons
  	lighticon	= new QToolButton(ControlFrame8,"lighticon");
    lighticon->setGeometry( QRect( 600, 10, 18, 26 ) );
  	lighticon->setFixedSize(18, 26);  	//lighticon->setUsesBigPixmap(true);
  	lighticon->setAutoRaise(true);

  	QIconSet	*icoSet;
    	icoSet = new QIconSet(QPixmap((const char**)light_xpm) );
    	icoSet->setPixmap(QPixmap((const char**)light_xpm), QIconSet::Large, QIconSet::Active);
    	icoSet->setPixmap(QPixmap((const char**)light_xpm), QIconSet::Large, QIconSet::Disabled);
    	lighticon->setIconSet(*icoSet);
    delete icoSet;

    light1 = new QCheckBox( ControlFrame8, "light1" );
    light1->setGeometry( QRect( 600, 36, 20, 21 ) );
    connect( light1,  SIGNAL(clicked()), this, SLOT(toggleLight())  );

    Transparency_1 = new QLabel( ControlFrame8, "ScaleText_2" );
    addLabel( Transparency_1, 400, 5, 110, 21, "TRANSPARENCY", 9);
    Trans_1 = new QSlider( ControlFrame8 );  Translcd_1 = new InputLCD( ControlFrame8 );
    addSliderLcdLabel( ControlFrame8, Trans_1,   400, 25, 10, 21,  1, 100,    Translcd_1,  "T", 9);
    connect( Trans_1,    SIGNAL(valueChanged(int)), Translcd_1, SLOT(display(int)) );
    connect( Trans_1,    SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );
    connect( Translcd_1, SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );
    ////////////////////////////////////////////////////////
    // raise the control bar and raise backgound as default

    fxaxis->raiseWidget(3);
    fxobject->raiseWidget(0);
	controlpanel->raiseWidget(1);

}













