/*******************************************************************************
**
** The source file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/


#include "colorize.h"


void GLColorize::initializeObjects() {   if (JAHDEBUG) JahDebug("In the GLColorize initialize objects");

  effects = new effectLayer[NUMBER_LAYERS];

  //make the current opengl context current so the textures can be set up properly
  this->makeCurrent();

  /// first layer is the backgroundimage
  layers[0].blend = FALSE;                 layers[0].depth = FALSE;
  layers[0].filename = "Pixmaps/jahlayer"; layers[0].extension = ".bmp";
  createLayer(0,"World", LAYER, BACKDROP );

  //dont know why we do this here but i think its a workaround
  //addSelection(TRUE);   // adds it as 'off' uses default numberLayers = 1
  //layers[0].updateTexture();   //add selection screws up somehow... this fixes it

  addSelection(FALSE);   // adds it as 'off' uses default numberLayers = 1
  objectMenu(numberLayers); updateSliders();

  rebuildStatus = true;
  //effectLayerNode = 0;

  displaySplit = displayToggle = FALSE;
  	
  //add the default color matrix effect
  numberLayers+=1;
  effects[numberLayers].effectnumber = numberLayers; layers[numberLayers].layername = "1 ColorOps";
  effects[numberLayers].objtype = COLORMATRIX;
  layers[numberLayers].node->tx = 50;
  layers[numberLayers].node->ty = 50;
  layers[numberLayers].node->tz = 50;
  addSelection();

  updateGL();


}


void GLColorize::buildEffect() {

	if (numberLayers>0) {

		//grab the source image
		QImage sourceimage = layers[0].textureData.objectImage.copy();

		//grab the source image into buffer ti apply effects to
		QImage renderimage = layers[0].textureData.objectImage.copy();

		//we reset the mesh every time if itrs in use - used for when we turn of geometry effects
		//need to fix this later on we'll start with effects being mesh-only
		if (layers[0].objtype == MESHLAYER) layers[0].initializeMeshLayer();

		//now we build the effects stacked on top of each other using renderimage
		for (int i=1; i<=numberLayers; i++) {
			if (layers[i].layerStatus ) // && effects(i).recalculate to be implemented later
				{
					callEffect(effects[i].objtype,i,renderimage);
				}
		}

		//draw the updated visual using the effects generated image
		//the memory leak is here!!!!!
		layers[0].rebuildTextureData(renderimage);

		//restore the source image
		layers[0].textureData.objectImage = sourceimage;


	}
	
}

void GLColorize::paintGL() {

	if (JAHDEBUG) JahDebug("we are in colorize module paintGL");

    if ( animation ) {  updateanimation(currentFrame); currentFrame++;
         if ( currentFrame > maxFrame ) currentFrame=1;
    }

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glPushMatrix();

    //turn off the backdrop for selection mode to work  until we figure out how to do selection Z testing
	//draw black video safe area if no layer present
    if (!select) {

        if (layers[0].layerStatus) {  	if (rebuildStatus) buildEffect();
										renderSpace->drawLayer(layers[0]);  }

        else {  renderSpace->drawLayer(*titlesafe); }

    }

    glPopMatrix();

	//this would be titlesafe...	
	  if (textsafe->layerStatus) {  renderSpace->drawLayer(*textsafe);  }

    if ( animation ) {
        glFlush();  timer->start( redrawWait, TRUE );                // Wait this many msecs before redraw
    } else glFlush();

}

void GLColorize::updateSliders() {  if (JAHDEBUG) debug("GLColorize we are updating the sliders");

   if ( activeLayer>1 ) {     //its the layers


    TranX_1->setValue( (int)layers[activeLayer].node->tx );
    TranY_1->setValue( (int)layers[activeLayer].node->ty );
    TranZ_1->setValue( (int)layers[activeLayer].node->tz );


    }
}




void GLColorize::clicked( QListViewItem* items )
{
 if (JAHDEBUG) JahDebug("we are in colorize clicked");

 if (!items) return;

 int height = items->height();
 int pos    = items->itemPos();
 QString laynum =  items->text( 1 );

 if (JAHDEBUG) {
	 JahDebug("height is  ", height );
	 JahDebug("pos is     ", pos );
	 JahDebug("layernum   ", laynum    );
 }
 //int toInt ( bool * ok = 0, int base = 10 ) const
  int pp = laynum.toInt();


  int itemstatus;
  if (      ( ( QCheckListItem* )items )->isOn()==1    )     {   JahDebug("posis on");   itemstatus=1; }
  if (      ( ( QCheckListItem* )items )->isOn()==0    )     {   JahDebug("posis off");  itemstatus=0; }

	//brings up the menu for the selected layer according to layer number
  objectMenu(pp);

	if ( ( itemstatus==1 ) && (layers[pp].layerStatus==0)) { setLayerVisible(pp, TRUE);    }
		else if ( ( itemstatus==0 ) && (layers[pp].layerStatus==1) ){ setLayerVisible(pp, FALSE);  }
		  else updateGL();

  updateSliders();


}



void GLColorize::objectMenu(int menu) {           //having trouble overriding this locally

    if (JAHDEBUG) JahDebug("In colorize object menu");
	/*
    // raise the main control box
    switch ( layers[menu].objtype ) {
      case BACKDROP     : { fxoptionals->raiseWidget(3); break; }  //was three
      case LAYER        : { fxoptionals->raiseWidget(0); break; }
      default           : { fxoptionals->raiseWidget(0); break; }
    }
*/
    // raise the right menu box
    if (layers[menu].objtype==BACKDROP)  { fxselect->raiseWidget(0); }     // was 2
      else if ( layers[menu].objtype==TEXT )   { fxselect->raiseWidget(1); }
         else if ( layers[menu].objtype==PARTICLE )   { fxselect->raiseWidget(4); }
             else if ( layers[menu].objtype==OBJECTS )   { fxselect->raiseWidget(5); }
                 else if (layers[menu].keyStatus==TRUE)     {  fxselect->raiseWidget(3); }
                     else {  fxselect->raiseWidget(0); }

    //select new object and update scene
    changeSelection(menu);
}


void GLColorize::sceneSettings(void)
{
    //name layer
    EffectsStatus = !EffectsStatus;
    if (EffectsStatus) { EffectsMain->raiseWidget(0); }
        else  { EffectsMain->raiseWidget(1); }

}

void GLColorize::addClip(videoData clip) {

debug("overriden in gleffect");

  layers[0].layername   = clip.clipname;
  layers[0].layernumber = clip.clipnumber;
  layers[0].filename    = clip.filename;
  layers[0].pathname    = clip.location;
  layers[0].extension   = clip.extension;

  layers[0].numframes   = clip.endframe;
  layers[0].startframe  = 1;
  layers[0].endframe    = clip.endframe-clip.startframe+1;
  layers[0].inframe	  = clip.startframe;
  layers[0].outframe    = clip.endframe;

  switch (clip.theCategory) {

        case IMAGE         : { layers[0].videotex = FALSE;  break; }
        case CLIP          : { layers[0].videotex = TRUE;
  			                       if (clip.cliptype==SEQUENCE) { layers[0].videoext = clip.videoext; }
  			                       break; }
  }

  layers[0].LoadLayerImage(IMAGE);
  layers[0].makeTexture();

  updateGL(); updateSliders();

}






















