#ifndef COLORIZE_H
#define COLORIZE_H

/*******************************************************************************
**
** The header file for the Jahshaka Colorize module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include <glworld.h>

#include "qcheckbox.h"
#include "qtabwidget.h"

#include <jahrender.h>

class GLColorize :  public  GLWorld {
  Q_OBJECT

public:


GLColorize( QWidget* parent, const char* name,
            QHBox* controls, QHBox* controller, QHBox* leftcontroller, QHBox* rightcontroller,
            int* globalclipnumber  )
        :  GLWorld(parent,name,controller,globalclipnumber ) {

    //clipnumber = globalclipnumber;

    buildInterface(controls);
    buildInterfaceMenus(leftcontroller,rightcontroller);
    initializeObjects();

};

GLColorize::~GLColorize() {  }

    void buildInterface( QHBox* qtinterface);
    void buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller );

    void initializeObjects();

    effectLayer  *effects;
public slots:
    void addClip(videoData clip);

    void addLayer(void);
	
    void clicked( QListViewItem* item );

    void updateSliders();
    void sceneSettings(void);

    void grabDesktop(void) { emit GrabClip(); };

	//these routines can also be shared
	void  callEffect( FxCategory, int, QImage& );

    void doColorMatrix(int layer);



signals:
    void    GrabClip(void);    //linked to main jahcontrol to talk to desktop

protected:

    void objectMenu(int menu);
    void paintGL();
	void buildEffect();

private:

	bool rebuildStatus;
	int effectobj;
	
	////////////////////////////////////////////////////////////////////////////
	// effect based variables

	QImage sourceimage,savedimage;

    ////////////////////////////////////////////////////////////////////////////
    //interface variables
    QFrame*      LayersFrame;

	QWidgetStack*  EffectsMain;
    QFrame*        SceneFrame;
    QFrame*        EffectsFrame;
    bool           EffectsStatus;

    QPushButton*   ResetButton;             QPushButton*   SaveButton;
	QPushButton*   LoadButton;				QPushButton*   ClearButton;
	QCheckBox*     SmoothSelect;            QCheckBox*     BlurSelect;    QCheckBox*     TextSelect;

    QTabWidget*        tabframe;
    QWidget*        axiscontrols;
    QWidget*        objectcontrols;
    QWidget*        texturecontrols;
    QWidget*        colorcontrols;
    QWidget*        effectscontrols;

    QWidgetStack*  fxoptionals;
    QHBox*         fxoptionals_clipone;     QHBox*         fxoptionals_cliptwo;
    QHBox*         fxoptionals_object;      QHBox*         fxoptionals_world;
    QHBox*         fxoptionals_text;

    QFrame*        ControlFrame1;
    QLabel*        RotateText_1;
    QLabel*        TranslateText_1;
    QLabel*        ScaleText_1;

    QSlider*       RotX_1;        InputLCD*  RotXlcd_1;   QLabel* RotX_1H;
    QSlider*       RotY_1;        InputLCD*  RotYlcd_1;   QLabel* RotY_1H;
    QSlider*       RotZ_1;        InputLCD*  RotZlcd_1;   QLabel* RotZ_1H;

    QDial*         TranX_1;       InputLCD*  TranXlcd_1;
    QDial*         TranY_1;       InputLCD*  TranYlcd_1;
    QDial*         TranZ_1;       InputLCD*  TranZlcd_1;

    QSlider*       ScaleX_1;      InputLCD*  ScaleXlcd_1;
    QSlider*       ScaleY_1;      InputLCD*  ScaleYlcd_1;
    QSlider*       ScaleZ_1;      InputLCD*  ScaleZlcd_1;

    //world control variables
    QFrame*        ControlFrame4;
    QLabel*        RotateText_4;
    QLabel*        TranslateText_4;
    QLabel*        ScaleText_4;
    QSlider*       RotX_4;        InputLCD*  RotXlcd_4;
    QSlider*       RotY_4;        InputLCD*  RotYlcd_4;
    QSlider*       RotZ_4;        InputLCD*  RotZlcd_4;

    QSlider*       TranX_4;       InputLCD*  TranXlcd_4;
    QSlider*       TranY_4;       InputLCD*  TranYlcd_4;
    QSlider*       TranZ_4;       InputLCD*  TranZlcd_4;

	  QSlider*       ScaleX_4;      InputLCD*  ScaleXlcd_4;

	  QCheckBox*     GridSelect;

	//widget selection sidebar variables
    QFrame*        ControlFrame;       QWidgetStack*  fxselect;
    QHBox*         fxselect_clip;      QHBox*         fxselect_text;
    QHBox*         fxselect_world;     QHBox*         fxselect_key;
    QHBox*         fxselect_particle;  QHBox*         fxselect_object;
	QHBox*		   fxselect_maxobject;


	//layer options
    QFrame*        ControlFrame5;
    QPushButton*   loadimage_1;        QPushButton*   grabdesktop;
    QToolButton*   lighticon;          QLabel*        light1text;      QCheckBox*     light1;
    QLabel*        Transparency_1;     QSlider*       Trans_1;         InputLCD*   Translcd_1;

	//key options
    QFrame*        ControlFrame8;
    QPushButton*   loadimage_2;        QPushButton*  keyimage_2;       QPushButton*  loadkey; QPushButton*  keyimage_3;
    QLabel*        Transparency_2;     QSlider*      Trans_2;          QLCDNumber*   Translcd_2;
    QPushButton* keyStatusBox;
	//font options
    QFrame*        ControlFrame6;
    QLabel*        ScaleText_5;        QSlider*      ScaleX_5;         InputLCD*  ScaleXlcd_5;
    QLineEdit*     textEd;
    QPushButton*   textFont;           QPushButton*  textColor;        QPushButton*  textDraw;

  //particles
    QFrame*        ControlFrame7;      QPushButton*  loadimage_3;      QPushButton*  particlestyle;

  //objects
    QFrame*        ControlFrame9;      QPushButton*  loadobject_1;     QPushButton*  objectstyle;

	QFrame*        ControlFrame10;     QPushButton*  loadmaxobject_1;


};



#endif // COLORIZE_H





