/*******************************************************************************
**
** The source file for the Jahshaka animation interface module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "colorize.h"

#include "Pixmaps/light.xpm"

void GLColorize::buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller) {

   //////////////////////////////////////////////////////////////
   // This is to be overridden...

}

void GLColorize::buildInterface( QHBox* qtinterface ) {

  ///////////////////////////////////////////////////////////
  //The layers interface
  LayersFrame = new QFrame( qtinterface, "EffectsFrame" );
  LayersFrame->setGeometry( QRect( 0, 0, 60, 195 ) );
  LayersFrame->setMinimumSize( QSize( 60, 195 ) );  //was 960
  LayersFrame->setMaximumSize( QSize( 60, 195 ) );

  ///////////////////////////////////////////////////////////
  //The layer options
  AddButton = new QPushButton( LayersFrame, "AddButton" );
  addButton( AddButton,  0, 5, 55, 25,  "Add" );
  connect  ( AddButton,  SIGNAL(clicked()), this, SLOT( addLayer())  );

  NameButton = new QPushButton( LayersFrame, "NameButton" );
  addButton( NameButton, 0, 30, 55, 25, "Name" );
  connect  ( NameButton,  SIGNAL(clicked()), this, SLOT( nameLayer())  );

      namepopup = new FancyPopup( this, "Enter Layer Name",200,100 );  //send size and position as well
      connect ( namepopup,      SIGNAL( returnText(QString)), SLOT( setlayerName(QString)) );

    DelButton = new QPushButton( LayersFrame, "Delete Button" );
    addButton( DelButton, 0, 60, 55, 25 , "Del" );
    connect  ( DelButton,  SIGNAL(clicked()), this, SLOT( delLayer())  );

    //this is now printbutton
    SceneButton = new QPushButton( LayersFrame, "NameButton" );
    addButton( SceneButton, 0, 100, 55, 25, "Data" );
    connect  ( SceneButton,  SIGNAL(clicked()), this, SLOT( printScene())  );       // sceneSettings() now obsolete


    //////////////////////////////////////////////////////////////
    // set up the scene options, or prefernces
    EffectsMain = new QWidgetStack(qtinterface);
      SceneFrame   = new QFrame(EffectsMain);
      EffectsFrame = new QFrame(EffectsMain);

    EffectsMain->addWidget(SceneFrame,0);
    EffectsMain->addWidget(EffectsFrame,1);
    EffectsMain->setGeometry( QRect( 0, 0, 855, 195) );

    EffectsStatus = FALSE; EffectsMain->raiseWidget(1);


    /////////////////////////////////////////////////////////
    //this is the list-view
    lv = new QListView( EffectsFrame );

      connect( lv, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( clicked( QListViewItem* ) ) );

      lv->setGeometry( QRect( 2, 2, 146, 180 ) );
      lv->addColumn( "PRESETS         ", -1 );
      lv->setSorting( -1,1 );   // disables the autosort

    /////////////////////////////////////////////////////////
    //Set up tabbed interface

    tabframe = new QTabWidget( EffectsFrame, "axiscontrols" );
    tabframe->setGeometry( QRect( 148, 1, 700, 180 ) );
    tabframe->setTabShape( QTabWidget::Triangular );
    tabframe->setAutoMask( FALSE );
    tabframe->setStyle( new QPlatinumStyle );

    axiscontrols = new QWidget( EffectsFrame, "axiscontrols" );
    tabframe->insertTab( axiscontrols, tr( "  CONTROLS  " ) );

    objectcontrols = new QWidget( EffectsFrame, "objectcontrols" );
    tabframe->insertTab( objectcontrols, tr( "  EFFECTS  " ) );

    colorcontrols = new QWidget( EffectsFrame, "scenesettings" );
    tabframe->insertTab( colorcontrols, tr( "  SCENE  " ) );

    /////////////////////////////////////////////////////////////////////
    // the layout boxes for different types of layers
    // these should be changed and regroups to represent tabs above

    // the axis control tab bar
    fxoptionals = new QWidgetStack(axiscontrols);

     fxoptionals_clipone = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_clipone,0);
     fxoptionals_cliptwo = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_cliptwo,1);
     fxoptionals_object  = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_object, 2);
     fxoptionals_world   = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_world,  3);
     fxoptionals_text    = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_text,   4);

    fxoptionals->setGeometry( QRect( 0, 0, 700, 180) );


    ///////////////////////////////////////////////////////////////////
    // the object control tab bar

    fxselect = new QWidgetStack(objectcontrols);

     fxselect_clip     = new QHBox(fxselect);          fxselect->addWidget(fxselect_clip,    0);
     fxselect_text     = new QHBox(fxselect);          fxselect->addWidget(fxselect_text,    1);
     fxselect_world    = new QHBox(fxselect);          fxselect->addWidget(fxselect_world,   2);
     fxselect_key      = new QHBox(fxselect);          fxselect->addWidget(fxselect_key,     3);
     fxselect_particle = new QHBox(fxselect);          fxselect->addWidget(fxselect_particle,4);
     fxselect_object   = new QHBox(fxselect);          fxselect->addWidget(fxselect_object,  5);
	 fxselect_maxobject= new QHBox(fxselect);          fxselect->addWidget(fxselect_maxobject,  6);

    fxselect->setGeometry( QRect( 0, 0, 700, 180) );   // was 635

    ///////////////////////////////////////////////////////////////////
    // the scene control tab bar

    //fxselect->setGeometry( QRect( 0, 0, 700, 120) );   // was 635
    LoadButton = new QPushButton( colorcontrols, "LoadButton" );
    addButton( LoadButton, 10, 10, 86, 25 , "Load" );
    connect  ( LoadButton,  SIGNAL(clicked()), this, SLOT( SceneLoad())  );

    SaveButton = new QPushButton( colorcontrols, "SaveButton" );
    addButton( SaveButton, 10, 45, 86, 25, "Save" );
    connect  ( SaveButton,  SIGNAL(clicked()), this, SLOT( SceneSave())  );

    ResetButton = new QPushButton( colorcontrols, "ResetButton" );
    addButton( ResetButton, 10, 80, 86, 25, "Reset" );
    connect  ( ResetButton,  SIGNAL(clicked()), this, SLOT( ResetAll())  );

    ClearButton = new QPushButton( colorcontrols, "ResetButton" );
    addButton( ClearButton, 10, 115, 86, 25, "Clear" );
    connect( ClearButton,  SIGNAL(clicked()), this, SLOT( ClearAll())  );

    //these are checkboxes...
    SmoothSelect = new QCheckBox( colorcontrols, "SmoothSelect" );
    addCheckButton( SmoothSelect, 130, 10, 110, 21, "aliasing" );
    connect( SmoothSelect,  SIGNAL(clicked()), this, SLOT(setSmooth())  );

    BlurSelect = new QCheckBox( colorcontrols, "BlurSelect" );
    addCheckButton( BlurSelect, 130, 45, 110, 21, "motion blur" );
    connect( BlurSelect,  SIGNAL(clicked()), this, SLOT(setBlur())  );

    TextSelect = new QCheckBox( colorcontrols, "TitleSafe" );
    addCheckButton( TextSelect, 130, 80, 110, 21, "title safe" );
    connect( TextSelect,  SIGNAL(clicked()), this, SLOT(setTextSafe())  );

  //need to move this into the module
    //this is mandatory for all modules...

    sframetext = new QLabel( colorcontrols, "sframetext" );
    sframetext->setGeometry( QRect( 400, 4, 80, 27 ) );
    sframetext->setText( tr( "STARTFRAME" ) );
    sframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont sframetext_font(  sframetext->font() );
    sframetext_font.setPointSize( 9 );
    sframetext->setFont( sframetext_font );

    lcdstartframe = new InputLCD( colorcontrols, "startframe",9 );
	  addSliderLcd(lcdstartframe, 500, 5, 90, 21, Astartframe);
	  lcdstartframe->setMinMaxInt( Astartframe, Aendframe-1 );
    lcdstartframe->setValue(Astartframe);
	  connect( lcdstartframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderStartframe(int)) );

    eframetext = new QLabel( colorcontrols, "eframetext" );
    eframetext->setGeometry( QRect( 400, 34, 80, 27 ) );
    eframetext->setText( tr( "ENDFRAME"  ) );
    eframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont eframetext_font(  eframetext->font() );
    eframetext_font.setPointSize( 9 );
    eframetext->setFont( eframetext_font );

//addSliderLcd(animframe, 5, 0, 90, 31, 100);
    lcdendframe = new InputLCD( colorcontrols, "endframe",9 );
	  addSliderLcd(lcdendframe, 500, 34, 90, 21, Aendframe);
	  lcdendframe->setMinMaxInt( Astartframe+1, MaxFrames );
    lcdendframe->setValue(Aendframe);
	  connect( lcdendframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderEndframe(int)) );


    QPalette p( QColor( 0, 0, 0 ) );
    p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );

    ///////////////////////////////////////////////////////////////////////
    //set up object control set and sliders

    ControlFrame1 = new QFrame( fxoptionals_clipone, "ControlFrame1" );
    ControlFrame1->setGeometry( QRect( 0, 0, 760, 120 ) );    // was 510

    // set up color translation sliders
    TranslateText_1 = new QLabel( ControlFrame1, "TranslateText_1" );
    addLabel( TranslateText_1, 180, 10, 110, 21, "COLORIZE RGB", 9);

    TranX_1 = new QDial( ControlFrame1 );   TranXlcd_1 = new InputLCD( ControlFrame1 );
    addDialLcdLabel( ControlFrame1, TranX_1, 10, 30, 30, 80,  100, TranXlcd_1,  "R", 9);

    connect( TranX_1,   SIGNAL(valueChanged(int)), this, SLOT(setXTrans(int)) );
    connect( TranX_1,   SIGNAL(valueChanged(int)), TranXlcd_1, SLOT(display(int)) );
	connect(TranXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXTrans(int)) );

    TranY_1 = new QDial( ControlFrame1 );   TranYlcd_1 = new InputLCD( ControlFrame1 );
    addDialLcdLabel( ControlFrame1, TranY_1, 220, 30, 30, 80,  100, TranYlcd_1,  "G", 9);

    connect( TranY_1, SIGNAL(valueChanged(int)), this, SLOT(setYTrans(int)) );
    connect( TranY_1, SIGNAL(valueChanged(int)), TranYlcd_1, SLOT(display(int)) );
	connect(TranYlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setYTrans(int)) );

    TranZ_1 = new QDial( ControlFrame1 );   TranZlcd_1 = new InputLCD( ControlFrame1 );
    addDialLcdLabel( ControlFrame1, TranZ_1, 420, 30, 30, 80,  100, TranZlcd_1,  "B", 9);

    connect( TranZ_1,   SIGNAL(valueChanged(int)), this, SLOT(setZTrans(int)) );
    connect( TranZ_1,   SIGNAL(valueChanged(int)), TranZlcd_1, SLOT(display(int)) );
	connect(TranZlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setZTrans(int)) );


    ///////////////////////////////////////////////////////////////
    // object layer options
    ControlFrame5 = new QFrame( fxselect_clip, "ControlFrame5" );
    ControlFrame5->setGeometry( QRect( 0, 0, 660, 120 ) );

    loadimage_1 = new QPushButton( ControlFrame5, "loadimage_1" );
    addButton( loadimage_1, 0, 10, 110, 25, "Load Image" );
    connect( loadimage_1,  SIGNAL(clicked()), this, SLOT(loadClip())  );

    grabdesktop = new QPushButton( ControlFrame5, "grabdesktop" );
    addButton( grabdesktop, 140, 10, 110, 25, "Grab Desktop" );
    connect( grabdesktop,  SIGNAL(clicked()), this, SLOT(grabDesktop())  );

    //add a subroutine for creating icons
  	lighticon	= new QToolButton(ControlFrame5,"lighticon");
    lighticon->setGeometry( QRect( 320, 10, 18, 26 ) );
  	lighticon->setFixedSize(18, 26);  	//lighticon->setUsesBigPixmap(true);
  	lighticon->setAutoRaise(true);

  	QIconSet	*icoSet;
    	icoSet = new QIconSet(QPixmap((const char**)light_xpm) );
    	icoSet->setPixmap(QPixmap((const char**)light_xpm), QIconSet::Large, QIconSet::Active);
    	icoSet->setPixmap(QPixmap((const char**)light_xpm), QIconSet::Large, QIconSet::Disabled);
    	lighticon->setIconSet(*icoSet);
    delete icoSet;

    light1 = new QCheckBox( ControlFrame5, "light1" );
    light1->setGeometry( QRect( 320, 36, 20, 21 ) );
    connect( light1,  SIGNAL(clicked()), this, SLOT(toggleLight())  );


    Transparency_1 = new QLabel( ControlFrame5, "ScaleText_2" );
    addLabel( Transparency_1, 0, 70, 110, 21, "TRANSPARENCY", 9);

    //problems in the layout here mon qframe qhbox issues mon

    Trans_1 = new QSlider( ControlFrame5 );  Translcd_1 = new InputLCD( ControlFrame5 );
    addSliderLcdLabel( ControlFrame5, Trans_1,   0, 89, 110, 21,  1, 100,    Translcd_1,  "T", 9);
    connect( Trans_1,    SIGNAL(valueChanged(int)), Translcd_1, SLOT(display(int)) );
    connect( Trans_1,    SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );
    connect( Translcd_1, SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );


    //////////////////////////////////////////////////////////////////
    // key options

    ControlFrame8 = new QFrame( fxselect_key, "ControlFrame4" );
    ControlFrame8->setGeometry( QRect( 0, 0, 40, 120 ) );

    loadimage_2 = new QPushButton( ControlFrame8, "ClipOne_2" );
    addButton( loadimage_2, 0, 20, 111, 30, "Load Image" );
    connect( loadimage_2,  SIGNAL(clicked()), this, SLOT(loadClip())  );

    //keyimage_2 = new QPushButton( ControlFrame8, "keyOne_2" );
    //addButton( keyimage_2, 115, 20, 35, 30, "IM" );
    //connect( keyimage_2,  SIGNAL(clicked()), this, SLOT(keyCliplayer())  );

    loadkey = new QPushButton( ControlFrame8, "ClipOne_2" );
    addButton( loadkey, 0, 55, 111, 30, "Load Key" );
    connect( loadkey,  SIGNAL(clicked()), this, SLOT(loadKey())  );

   // keyimage_3 = new QPushButton( ControlFrame8, "keyOne_3" );
   // addButton( keyimage_3, 115, 55, 35, 30, "KM" );
   // connect( keyimage_3,  SIGNAL(clicked()), this, SLOT(keyClipkey())  );

    Transparency_2 = new QLabel( ControlFrame8, "ScaleText_2" );           //use label function here
    addLabel( Transparency_2, 0, 90, 110, 21, "Invert Key", 9);

    keyStatusBox = new QPushButton( ControlFrame8, "Select1" );
    addButton( keyStatusBox, 0, 90, 111, 30, "Invert Key" );
    connect( keyStatusBox,  SIGNAL(clicked()), this, SLOT(InvertKeyData())  );



    ////////////////////////////////////////////////////////
    // raise the control bar and raise backgound as default

    fxoptionals->raiseWidget(0);
    fxselect->raiseWidget(0);

}























