#ifndef EDIT_H
#define EDIT_H

/*******************************************************************************
**
** The header file for the Jahshaka editing module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include <glworld.h>


static const int imageRTTI2 = 984376;
const bool no_writing2 = FALSE;

class EditItem: public QCanvasRectangle {
public:
    //EditItem( QImage full, QImage img, QCanvas *canvas, QString& clipname );
    EditItem( videoData full, QImage img, QCanvas *canvas, QString& clipname );
    int rtti () const { return imageRTTI2; }
    bool hit( const QPoint&) const;

    QImage getImage() { return fullres; }

    videoData getClip() { return itemdata; }

    QString getText() { return name; }

    void setText(QString& newname) { name=newname;}
protected:
    void drawShape( QPainter & );
private:
    QImage image;
    QImage fullres;
    QPixmap pixmap;
    QString name;

    videoData itemdata;
};

class EditEditor : public QCanvasView {
    Q_OBJECT

public:
    EditEditor(QCanvas*, QWidget* parent=0, const char* name=0, WFlags f=0);
    void clear();
    void clearItem();
    void setItemtext(QString&);
    //QImage getCurrentClip();
    QImage globalimage;
    QString imagetext;
    //EditItem selecteditem;
     EditItem *globalitem;

    videoData selectedClip;

    //temporary workaround here as sgi dont check for infunc creation properly
    bool bordercreated;
    bool move;

    void refresh();

protected:
    void contentsMousePressEvent(QMouseEvent*);
    void contentsMouseReleaseEvent(QMouseEvent*);
    void contentsMouseMoveEvent(QMouseEvent*);

signals:
    void status(const QString&);

private:
    QCanvasItem* moving;
    QPoint moving_start;
    int height;

    QCanvasPolygonalItem  *border;
    QCanvasItem* borderframe;
    QCanvasText* itemtext;
};



class EditDesktop : public QWidget {
    Q_OBJECT

public:
    EditDesktop(QWidget* parent=0, const char* name=0, WFlags f=0);

    QImage getClip();
    void setClipname(QString&);
    QString getClipname();
    int clipnumber;
    videoData getVideoData();

   int videolayer;
	int layer_height;
	int layer_pos;

public slots:
    //void addImage2( const QImage&, videoData thedata );
    void addImage2( videoData thedata );
    void addImage( const QImage& );
    void addRectangle(int numline);
    void addEditLayer(int numline, bool status, int startframe, int endframe);

private slots:

    void clear();
    void init();
    void addRectangle();

    void drawScale(int width);
    void addLine();
    void addLines(int width, int numline);

    void addText();
    void addLogo();
    void addPolygon();


private:
    EditEditor *editor;
    QCanvas* canvas;
    int dbf_id;
    QImage* img;
};



class GLEdit :  public  GLWorld {
  Q_OBJECT

public:

GLEdit( QWidget* parent, const char* name,
        QHBox* controls, QHBox* controller, QHBox* leftcontroller, QHBox* rightcontroller,
        int* globalclipnumber )
        :  GLWorld(parent,name,controller,globalclipnumber ) {

    buildInterface(controls);
    buildInterfaceMenus(leftcontroller,rightcontroller);
    initializeObjects();


};

GLEdit::~GLEdit() {  }

    EditDesktop*           theeditdesktop;

    void buildInterface( QHBox* qtinterface);
    void buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller );

    void initializeObjects();

public slots:


    void addLayer(void);

    void clicked( QListViewItem* item );

    void changeParticle();
    void changeFontDraw();
    void changeObjectDraw();
    void keyCliplayer();
    void keyClipkey();

    void updateSliders();
    void sceneSettings(void);

    void grabDesktop(void) { emit GrabClip(); };

signals:
    void    GrabClip(void);    //linked to main jahcontrol to talk to desktop

protected:

    void objectMenu(int menu);



private:


    ////////////////////////////////////////////////////////////////////////////
    //interface variables
    QFrame*      LayersFrame;

	QWidgetStack*  EffectsMain;
    QFrame*        SceneFrame;
    QFrame*        EffectsFrame;
    bool           EffectsStatus;

    QPushButton*   ResetButton;             QPushButton*   SaveButton;    QPushButton*   LoadButton;
	  QCheckBox*     SmoothSelect;            QCheckBox*     BlurSelect;    QCheckBox*     TextSelect;

    QFrame*        ControlFrame_2;
    QWidgetStack*  fxoptionals;
    QHBox*         fxoptionals_clipone;     QHBox*         fxoptionals_cliptwo;
    QHBox*         fxoptionals_object;      QHBox*         fxoptionals_world;
    QHBox*         fxoptionals_text;

    QFrame*        ControlFrame1;
    QLabel*        RotateText_1;
    QLabel*        TranslateText_1;
    QLabel*        ScaleText_1;

    QSlider*       RotX_1;        QLCDNumber*  RotXlcd_1;
    QSlider*       RotY_1;        QLCDNumber*  RotYlcd_1;
    QSlider*       RotZ_1;        QLCDNumber*  RotZlcd_1;

    QSlider*       TranX_1;       QLCDNumber*  TranXlcd_1;
    QSlider*       TranY_1;       QLCDNumber*  TranYlcd_1;
    QSlider*       TranZ_1;       QLCDNumber*  TranZlcd_1;

    QSlider*       ScaleX_1;      QLCDNumber*  ScaleXlcd_1;
    QSlider*       ScaleY_1;      QLCDNumber*  ScaleYlcd_1;
    QSlider*       ScaleZ_1;      QLCDNumber*  ScaleZlcd_1;

    //world control variables
    QFrame*        ControlFrame4;
    QLabel*        RotateText_4;
    QLabel*        TranslateText_4;
    QLabel*        ScaleText_4;
    QSlider*       RotX_4;        QLCDNumber*  RotXlcd_4;
    QSlider*       RotY_4;        QLCDNumber*  RotYlcd_4;
    QSlider*       RotZ_4;        QLCDNumber*  RotZlcd_4;

    QSlider*       TranX_4;       QLCDNumber*  TranXlcd_4;
    QSlider*       TranY_4;       QLCDNumber*  TranYlcd_4;
    QSlider*       TranZ_4;       QLCDNumber*  TranZlcd_4;

	  QSlider*       ScaleX_4;      QLCDNumber*  ScaleXlcd_4;

	  QCheckBox*     GridSelect;

	//widget selection sidebar variables
    QFrame*        ControlFrame;       QWidgetStack*  fxselect;
    QHBox*         fxselect_clip;      QHBox*         fxselect_text;
    QHBox*         fxselect_world;     QHBox*         fxselect_key;
    QHBox*         fxselect_particle;  QHBox*         fxselect_object;


	//layer options
    QFrame*        ControlFrame5;
    QPushButton*   loadimage_1;        QPushButton*   grabdesktop;
    QToolButton*   lighticon;          QLabel*        light1text;      QCheckBox*     light1;
    QLabel*        Transparency_1;     QSlider*       Trans_1;         QLCDNumber*   Translcd_1;

	//key options
    QFrame*        ControlFrame8;
    QPushButton*   loadimage_2;        QPushButton*  keyimage_2;       QPushButton*  keyimage_3;
    QLabel*        Transparency_2;     QSlider*      Trans_2;          QLCDNumber*   Translcd_2;

	//font options
    QFrame*        ControlFrame6;
    QLabel*        ScaleText_5;        QSlider*      ScaleX_5;         QLCDNumber*  ScaleXlcd_5;
    QLineEdit*     textEd;
    QPushButton*   textFont;           QPushButton*  textColor;        QPushButton*  textDraw;

  //particles
    QFrame*        ControlFrame7;      QPushButton*  loadimage_3;      QPushButton*  particlestyle;

  //objects
    QFrame*        ControlFrame9;      QPushButton*  loadobject_1;     QPushButton*  objectstyle;

};



#endif // EDIT_H















