/*******************************************************************************
**
** The source file for the Jahshaka editing interface timeline module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/


#include "edit.h"


///////////////////////////////////////////////////////////////////


//EditItem::EditItem( QImage full, QImage img, QCanvas *canvas, QString& clipname )
EditItem::EditItem( videoData full, QImage img, QCanvas *canvas, QString& clipname )
        : QCanvasRectangle( canvas ), image(img) {
    setSize( image.width(), image.height() );

#ifndef _WS_QWS_
    pixmap.convertFromImage(image, OrderedAlphaDither);
#endif
    fullres = full.Image;
    name=clipname;

    itemdata = full;
}


void EditItem::drawShape( QPainter &p ) {
    // On Qt/Embedded, we can paint a QImage as fast as a QPixmap,
    // but on other platforms, we need to use a QPixmap.
#ifdef _WS_QWS_
    p.drawImage( int(x()), int(y()), image, 0, 0, -1, -1, OrderedAlphaDither );
#else
    p.drawPixmap( int(x()), int(y()), pixmap );
#endif
}

bool EditItem::hit( const QPoint &p ) const {
    int ix = p.x()-int(x());
    int iy = p.y()-int(y());
    if ( !image.valid( ix , iy ) )
        return FALSE;
    QRgb pixel = image.pixel( ix, iy );
    return qAlpha( pixel ) != 0;
}

EditEditor::EditEditor( QCanvas* c, QWidget* parent,
 const char* name, WFlags f) :
 QCanvasView(c,parent,name,f) {

    height=1;   //what happens if this goes over the max value for a int? crash!!!
    //EditItem globalitem;

    bordercreated = FALSE;
    move = FALSE;
}


void EditEditor::contentsMousePressEvent(QMouseEvent* e) {


    QCanvasItemList l=canvas()->collisions(e->pos());
	qDebug("mouse pressed");

    for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {

		if ( (*it)->rtti() == imageRTTI2 ) {
			EditItem *item= (EditItem*)(*it);

			if ( !item->hit( e->pos() ) )
			 continue;
		}

	moving = *it;
	moving_start = e->pos();
	return;

    }
    moving = 0;
    /*
    // creating border anywhere else ie in constructor crashes app...
    // we want to use a better method anyways so aint gonna waste no more time on this
    // ideas are just to draw selection boundar
    // moving->setSelected(1);  // QRect myframe = moving->boundingRect();

    QCanvasItemList l=canvas()->collisions(e->pos());

    for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {

        if ( (*it)->rtti() == imageRTTI2 ) {
            EditItem *item= (EditItem*)(*it);
            if ( !item->hit( e->pos() ) ) {
                continue;
            }
            move = 1;

            globalitem = (EditItem*)(*it);
            globalimage = item->getImage();
            imagetext = item->getText();

            selectedClip = item->getClip();
            selectedClip.printinfo();

        }

    if (!bordercreated) {

        border = new QCanvasRectangle( 0,0, 180,120,canvas());
        border->setBrush( QColor(50,0,200) );
        border->setPen( QPen(QColor(0,50,200), 6) );
        border->setZ(1);

        itemtext = new QCanvasText( canvas());
        itemtext->setColor( QColor(200,200,200) );
        itemtext->setText(imagetext);
        itemtext->setZ(1);
        bordercreated = TRUE;
    }

    itemtext->setText(imagetext);

        //moving is set to canvas item referenced by *it
        moving = *it;

        moving_start = e->pos();


        //now we position border and bring selected item and
        //border to the front of others on the canvas
        border->setX( moving->x() );
        border->setY( moving->y() );
        height++; border->setZ(height);

        itemtext->setX( moving->x() );
        itemtext->setY( moving->y() );

        height++; moving->setZ(height);
        height++; itemtext->setZ(height);

        border->show();
        itemtext->show();
        canvas()->update();

        return;
    }
    moving = 0;
      */
}

//QImage EditEditor::getCurrentClip() {
//    QImage temp;
 //   temp=moving.getImage();
 //   return temp;
//}


void EditEditor::clear() {
    QCanvasItemList list = canvas()->allItems();
    QCanvasItemList::Iterator it = list.begin();

    for (; it != list.end(); ++it) {
        if ( *it ) {
            delete *it;
        }
    }

    canvas()->update();
    height=1;    //reset this variable when we clear the desktop
}


void EditEditor::clearItem() {

    if (  moving ) {

        delete moving;   //delete selectedClip;
        //delete border;
        //delete itemtext;
        border->hide();   itemtext->hide();
        canvas()->update();
    }
}

void EditEditor::refresh() {
  
  canvas()->update();

}

void EditEditor::setItemtext(QString& newname) {

    debug("setting clipname");  debug(newname);
    globalitem->setText(newname);
    canvas()->update();
}

void EditEditor::contentsMouseReleaseEvent(QMouseEvent* e) {
  //need to release move here?
//  move = 0;
    //moving = 0;
  //  canvas()->update();

}

void EditEditor::contentsMouseMoveEvent(QMouseEvent* e) {


  	qDebug("We are moving");

    if ( moving ) {

		//	moving->moveBy(e->pos().x() - moving_start.x(),  e->pos().y() - moving_start.y());

		moving->moveBy(e->pos().x() - moving_start.x(),      0 );  //limits to move on x only

		//here we move video layers up or down by one video line

		int ymov = (e->pos().y()); //get current position

		if ( (ymov / 27)==0 ) {  //25 = layer height

			moving->moveBy(0,  e->pos().y() - moving_start.y());	   //limits move to y only
		//	moving->moveBy(0,  e->pos().y());	   //limits move to y only

		}

	moving_start = e->pos();
	canvas()->update();
    }

    /*
    if ( move ) {   // was moving

        //move clip and the background border
        moving->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());
        border->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());
        //text moveby
        itemtext->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());

        moving_start = e->pos();
        canvas()->update();
    }
    */
}


EditDesktop::EditDesktop(QWidget* parent, const char* name, WFlags f) :
QWidget(parent,name,f) {
    clipnumber=1;
    canvas = new QCanvas(980,506);
    QColor back;
    back.setHsv(35,35,35);
    canvas->setBackgroundColor(back);
    canvas->resize( 980, 506 );

    editor = new EditEditor(canvas,this);
    editor->resize( 980, 506 );
    editor->setHScrollBarMode( EditEditor::AlwaysOff );
    editor->setVScrollBarMode( EditEditor::AlwaysOff );

    init();
}

void EditDesktop::init()
{
    clear();

  	videolayer=1;   layer_height=25; layer_pos=0;

    static int r=24;   srand(++r);

    int i;

	drawScale(canvas->width() );

	// create our video grid
	int numlines = (canvas->width() / 85);

	//these should not be moveable
	for ( i=1; i<numlines+2; i++) {
	addLines(canvas->width(),i );
    }

    addLine();
}


void EditDesktop::clear() {
    editor->clear();
    //we need to erase all of the files here
}

void EditDesktop::addImage( const QImage& newimage) {
}

//void Main::addImage2( const QImage& newimage, videoData thedata ) {
void EditDesktop::addImage2( videoData thedata ) {


    //we need to scale the proxy size here instead of using the default 0.25% reduction
    float wratio = (180/(float)thedata.Image.width() );
    float hratio = (120/(float)thedata.Image.height() );

//    QString data; debug("wratio:"+data.setNum(wratio)+ " width:"+data.setNum( thedata.Image.width() *wratio ) +" height:" + data.setNum (thedata.Image.height()*hratio ) );

    QString theclipnumber;
      theclipnumber.setNum(thedata.clipnumber);

    QString clipper1; clipper1= thedata.clipname;
    clipper1.append(theclipnumber);

    img = new QImage[2];
    img[1] = thedata.Image.smoothScale( int(thedata.Image.width()*wratio), int(thedata.Image.height()*hratio) );

    // we need to update this so that we dont have to send the thedata.Image just thedata...
//    QCanvasPolygonalItem* i = new EditItem(thedata.Image, img[1],canvas, clipper1);
    QCanvasPolygonalItem* i = new EditItem(thedata, img[1],canvas, clipper1);
      i->setPen( QPen(QColor(200,0,0), 6) );
      i->setZ(1);
      i->show();

    delete [] img;      // is this necessary as img is not a pointer???
    editor->update();
}

void EditDesktop::addRectangle() {
    QCanvasPolygonalItem *i = new QCanvasRectangle( 0,0, 180,120,canvas);
    i->setBrush( QColor(50,0,200) );
    i->setPen( QPen(QColor(0,50,200), 6) );
    i->setZ(1);
    i->show();
}

QImage EditDesktop::getClip() {
    // change this to send a pointer instead of a qimage using globalobject!!!
    return editor->globalimage;
}

videoData EditDesktop::getVideoData() {

    return editor->selectedClip;

}

void EditDesktop::setClipname(QString& newname)
{
     editor->setItemtext(newname);
}

QString EditDesktop::getClipname() {
    // change this to send a pointer instead of a qimage using globalobject!!!
    return editor->imagetext;
}


/*
void Main::newView()
{
    // Open a new view... have it delete when closed.
    Main *m = new Main(canvas, 0, 0, WDestructiveClose);
    qApp->setMainWidget(m);
    m->show();
    qApp->setMainWidget(0);
}
*/

void EditDesktop::drawScale(int width)
{
	int scale = width / layer_height;

	QCanvasLine* i = new QCanvasLine(canvas);
    i->setPoints(0, 5, width, 5 );

    i->setPen( QPen(QColor(0,0,0), 3) );
    i->setZ(3);
    i->show();


	for ( int l=1; l<width; l+=scale ) {

		QCanvasLine* li = new QCanvasLine(canvas);
		li->setPoints(l, 5, l, 10 );

		li->setPen( QPen(QColor(0,0,0), 3) );
		li->setZ(3);
		li->show();

	}
}


//playbar vertical line
void EditDesktop::addLine()
{
    QCanvasLine* i = new QCanvasLine(canvas);
    i->setPoints(canvas->width()/2, 0,
                  canvas->width()/2, canvas->height() );

    i->setPen( QPen(QColor(100,0,0), 6) );
    i->setZ(3);
    i->show();
}



void EditDesktop::addLines(int width, int numline)
{
    QCanvasLine* i = new QCanvasLine(canvas);

	int vpos = numline * (layer_height+2);

    i->setPoints( 0, vpos,
                  canvas->width(), vpos );

    i->setPen( QPen(QColor(200,200,200), 2) );

    i->setZ(1);
    i->show();
}

void EditDesktop::addRectangle(int numline)
{

int vpos = numline * (layer_height+2);

	    QCanvasPolygonalItem *i = new QCanvasRectangle(
		        ((rand()%256)/2)     ,vpos+1,
			    100+((rand()%256)*2) ,layer_height,         canvas);

    int z = 128+((rand()%256)/4);
    i->setBrush( QColor(z,z,z) );
    i->setPen( QPen(QColor(0,0,200), 1) );
    i->setZ(0);
    i->show();

    editor->refresh();
}

void EditDesktop::addEditLayer(int numline, bool status, int startframe, int endframe)
{
    numline = numline - 1;
    
int vpos = numline * (layer_height+2);

	    QCanvasPolygonalItem *i = new QCanvasRectangle(
		        ((rand()%256)/2)     ,vpos+1,
			    100+((rand()%256)*2) ,layer_height,         canvas);

    int z = 128+((rand()%256)/4);
    i->setBrush( QColor(z,z,z) );
    i->setPen( QPen(QColor(0,0,200), 1) );
    i->setZ(0);
    i->show();

    editor->refresh();
}


void EditDesktop::addText()
{
    QCanvasText* i = new QCanvasText(canvas);
    i->setText("QCanvasText");
    i->move(rand()%canvas->width(),rand()%canvas->height());
    i->setZ(rand()%256);
    i->setFont(QFont("helvetica",24));
    i->show();
}

QString logo_fn;

void EditDesktop::addLogo()
{
 /*   if ( logo_fn.isEmpty() )
	return;
    static QImage *img;
    if ( !img ) {
	img = new QImage[4];
	img[0].load( logo_fn );
	img[1] = img[0].smoothScale( int(img[0].width()*0.75),
		int(img[0].height()*0.75) );
	img[2] = img[0].smoothScale( int(img[0].width()*0.5),
		int(img[0].height()*0.5) );
	img[3] = img[0].smoothScale( int(img[0].width()*0.25),
		int(img[0].height()*0.25) );
    }
    QCanvasPolygonalItem* i = new ImageItem(img[rand()%4],canvas);
    i->move(rand()%(canvas->width()-img->width()),
	    rand()%(canvas->height()-img->width()));
    i->setZ(rand()%256+256);
    i->show();                         */
}

void EditDesktop::addPolygon()
{
    QCanvasPolygon* i = new QCanvasPolygon(canvas);
    const int size = canvas->width()/2;
    QPointArray pa(6);
    pa[0] = QPoint(0,0);
    pa[1] = QPoint(size,size/5);
    pa[2] = QPoint(size*4/5,size);
    pa[3] = QPoint(size/6,size*5/4);
    pa[4] = QPoint(size*3/4,size*3/4);
    pa[5] = QPoint(size*3/4,size/4);
    i->setPoints(pa);
    i->setBrush( QColor(rand()%32*8,rand()%32*8,rand()%32*8) );
    i->move(rand()%canvas->width(),rand()%canvas->height());
    i->setZ(rand()%256);
    i->show();
}


















