/*******************************************************************************
**
** The source file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/


#include "effect.h"


void GLEffect::initializeObjects() {    if (JAHDEBUG) JahDebug("In the GLEffect initialize objects");

  //make the current opengl context current so the textures can be set up properly
  this->makeCurrent();

  //we moved this into GLWorld for linux's compiler
  // but it really should be here for efficiency
  effects = new effectLayer[NUMBER_LAYERS];

  /// first layer is the backgroundimage
  layers[0].blend = FALSE; layers[0].depth = FALSE;
  layers[0].filename = "Pixmaps/jahlayer"; layers[0].extension = ".bmp";
  createLayer(0,"World", LAYER, BACKDROP );

  addSelection(FALSE);   // adds backgroundimage as 'off' uses default numberLayers = 1
  layers[0].updateTexture();   //why doesnt this work here?

  //flag used to tell us if effects need to be rebuilt hardcoded for now
  rebuildStatus = true;

  //menu widgets to be implemented later on
  displaySplit = displayToggle = FALSE;

}


void GLEffect::buildEffect() {

	if (numberLayers>0) {

		//grab the source image
		QImage sourceimage = layers[0].textureData.objectImage.copy();

		//grab the source image into buffer ti apply effects to
		QImage renderimage = layers[0].textureData.objectImage.copy();

		//we reset the mesh every time if itrs in use - used for when we turn of geometry effects
		//need to fix this later on we'll start with effects being mesh-only
		if (layers[0].objtype == MESHLAYER) layers[0].initializeMeshLayer();

		//now we build the effects stacked on top of each other using renderimage
		for (int i=1; i<=numberLayers; i++) {
			if (layers[i].layerStatus ) // && effects(i).recalculate to be implemented later
				{
					callEffect(effects[i].objtype,i,renderimage);
				}
		}

		//draw the updated visual using the effects generated image
		//the memory leak is here!!!!!
		layers[0].rebuildTextureData(renderimage);

		//restore the source image
		layers[0].textureData.objectImage = sourceimage;

		//trying to clean up here is this necessary?
		//renderimage.detach();
		//savedimage.detach();
		//delete renderimage;
	}

	
}

void GLEffect::paintGL() {

    if ( animation ) {  updateanimation(currentFrame); currentFrame++;
         if ( currentFrame > maxFrame ) currentFrame=1;
    }

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    glPushMatrix();

    //turn off the backdrop for selection mode to work  until we figure out how to do selection Z testing
	//draw black video safe area if no layer present
    if (!select) {

        if (layers[0].layerStatus) {  	if (rebuildStatus) buildEffect();
										renderSpace->drawLayer(layers[0]); }

								else {  renderSpace->drawLayer(*titlesafe); }

    }

    glPopMatrix();

	//this would be titlesafe...	
	  if (textsafe->layerStatus) {  renderSpace->drawLayer(*textsafe);  }

    if ( animation ) {
        glFlush();  timer->start( redrawWait, TRUE );                // Wait this many msecs before redraw
    } else glFlush();


}

void GLEffect::updateSliders() {   if (JAHDEBUG) debug("GLEffect we are updating the sliders");

   if ( activeLayer>0 ) {     //its the layers

    light1->setChecked(layers[activeLayer].lighting);

    Trans_1->setValue( (int)(layers[activeLayer].node->Alpha)  );

    RotX_1->setValue( (int)layers[activeLayer].node->rx );
    RotY_1->setValue( (int)layers[activeLayer].node->ry );
    RotZ_1->setValue( (int)layers[activeLayer].node->rz );

    TranX_1->setValue( (int)layers[activeLayer].node->tx );
    TranY_1->setValue( (int)layers[activeLayer].node->ty );
    TranZ_1->setValue( (int)layers[activeLayer].node->tz );

    ScaleX_1->setValue( (int)(layers[activeLayer].node->sx *10) );
    ScaleY_1->setValue( (int)(layers[activeLayer].node->sy *10) );
    ScaleZ_1->setValue( (int)(layers[activeLayer].node->sz *10) );

    } else  { //its the world...

    //RotX_4->setValue( (int)layers[0].node->rx );
    //RotY_4->setValue( (int)layers[0].node->ry );
   // RotZ_4->setValue( (int)layers[0].node->rz );

   // TranX_4->setValue( (int)layers[0].node->tx );
   // TranY_4->setValue( (int)layers[0].node->ty );
   // TranZ_4->setValue( (int)layers[0].node->tz );

	  //QSlider*       ScaleX_4;

    }
}


void GLEffect::clicked( QListViewItem* items )
{

	if (!items) return;

	//if ( controlpanel->id (controlpanel->visibleWidget() )==0 )
	//	controlpanel->raiseWidget(1);

	int height = items->height();
	int pos    = items->itemPos();
	QString laynum =  items->text( 1 );

	int itemstatus;
	if (   ( ( QCheckListItem* )items )->isOn()==1   ) { itemstatus=1; }
	if (   ( ( QCheckListItem* )items )->isOn()==0   ) { itemstatus=0; }


	int pp = laynum.toInt();

	//set to background as default
	//if (pp==0)  { pp = 1;}

	if (JAHDEBUG) JahDebug("GLEffect stored layer number is", pp );

	//brings up the menu for the selected layer according to layer number
    objectMenu(pp);

    //we waste a interupt here, always checking the clicked item
    //would be better with a signal slot implementation

	if ( ( itemstatus==1 ) && (layers[pp].layerStatus==0)) { setLayerVisible(pp, TRUE);     }
		else
    if ( ( itemstatus==0 ) && (layers[pp].layerStatus==1) ){ setLayerVisible(pp, FALSE);  }
		else
	updateGL();

	updateSliders();


}

void GLEffect::objectMenu(int menu) {           //having trouble overriding this locally

JahDebug("objtype is:",layers[menu].objtype);

    // raise the main control box
    switch ( layers[menu].objtype ) {
      case BACKDROP     : { fxoptionals->raiseWidget(3); break; }  //was three
      case LAYER        : { fxoptionals->raiseWidget(0); break; }
      case MESHLAYER    : { fxoptionals->raiseWidget(0); break; }
      default           : { fxoptionals->raiseWidget(0); break; }
    }

    // raise the right menu box
    //if (layers[menu].objtype==BACKDROP)  { fxselect->raiseWidget(0); }     // was 2
    //      else if (layers[menu].keyStatus==TRUE)     {  fxselect->raiseWidget(3); }
    //      else {  fxselect->raiseWidget(0); }

    //select new object and update scene
    changeSelection(menu);
}


void GLEffect::sceneSettings(void)
{
    //name layer
    EffectsStatus = !EffectsStatus;
    if (EffectsStatus) { EffectsMain->raiseWidget(0); }
        else  { EffectsMain->raiseWidget(1); }

}

void GLEffect::addClip(videoData clip) {

debug("overriden in gleffect");

  layers[0].layername   = clip.clipname;
  layers[0].layernumber = clip.clipnumber;
  layers[0].filename    = clip.filename;
  layers[0].pathname    = clip.location;
  layers[0].extension   = clip.extension;

  layers[0].numframes   = clip.endframe;
  layers[0].startframe  = 1;
  layers[0].endframe    = clip.endframe-clip.startframe+1;
  layers[0].inframe	    = clip.startframe;
  layers[0].outframe    = clip.endframe;

  switch (clip.theCategory) {

        case IMAGE         : { layers[0].videotex = FALSE;  break; }
        case CLIP          : { layers[0].videotex = TRUE;
  			                       if (clip.cliptype==SEQUENCE) { layers[0].videoext = clip.videoext; }
  			                       break; }
  }

  layers[0].LoadLayerImage(IMAGE);
  layers[0].makeTexture();

  updateGL(); updateSliders();

}





















