/*******************************************************************************
**
** The source file for the Jahshaka animation interface module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "effect.h"

#include "Pixmaps/light.xpm"

void GLEffect::buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller) {

    //left menu here
    glworldLeft = new QFrame (leftcontroller, "gl left");
    //glworldLeft->setFixedSize( QSize( 30, 300 ) );

    tooltranslate = new QPushButton( ( QIconSet( QPixmap("Pixmaps/translate.xpm"), QPixmap("Pixmaps/translate.xpm") )) ,"", glworldLeft, "play" );
    tooltranslate->setGeometry( QRect( 0, 205, 30, 27 ) );
    tooltranslate->setFlat(true);

    toolrotate = new QPushButton( ( QIconSet( QPixmap("Pixmaps/rotate.xpm"), QPixmap("Pixmaps/rotate.xpm") )) ,"", glworldLeft, "rotate" );
    toolrotate->setGeometry( QRect( 0, 233, 30, 27 ) );
    toolrotate->setFlat(true);

    toolscale = new QPushButton( ( QIconSet( QPixmap("Pixmaps/scale.xpm"), QPixmap("Pixmaps/scale.xpm") )) ,"", glworldLeft, "scale" );
    toolscale->setGeometry( QRect( 0, 263, 30, 27 ) );
    toolscale->setFlat(true);

    //rightmenu here
    glworldRight =  new QFrame (rightcontroller, "gl right" );
    //glworldRight->setFixedSize( QSize( 30, 300 ) );

    toolzoomin = new QPushButton( ( QIconSet( QPixmap("Pixmaps/zoomin.xpm"), QPixmap("Pixmaps/zoomin.xpm") )) ,"", glworldRight, "zoomin" );
    toolzoomin->setGeometry( QRect( 0, 115, 30, 27 ) );
    toolzoomin->setFlat(true);

    zoom = new QSlider( glworldRight, "scrubslider" );
    zoom->setStyle( new QPlatinumStyle );
    zoom->setGeometry( QRect( 0, 150, 30, 200 ) );
    zoom->setOrientation( QSlider::Vertical );
    zoom->setMinValue( -10 );    zoom->setMaxValue( 10 );    zoom->setValue   ( 0 );

    toolzoomout = new QPushButton( ( QIconSet( QPixmap("Pixmaps/zoomout.xpm"), QPixmap("Pixmaps/zoomout.xpm") )) ,"", glworldRight, "zoomout" );
    toolzoomout->setGeometry( QRect( 0, 355, 30, 27 ) );
    toolzoomout->setFlat(true);

	  //connect tools to object and add object to display widget
    //note that zoom isnt hooked up yet...
    connect( tooltranslate,  SIGNAL(clicked()), this, SLOT(toolTranslate() )  );
    connect( toolscale,      SIGNAL(clicked()), this, SLOT(toolScale() )  );
    connect( toolrotate,     SIGNAL(clicked()), this, SLOT(toolRotate() )  );

    connect( zoom,  SIGNAL(sliderMoved(int)), this, SLOT(changeZoom(int) )  );


}

void GLEffect::buildInterface( QHBox* qtinterface ) {

    ///////////////////////////////////////////////////////////
    //The layers interface
    LayersFrame = new QFrame( qtinterface, "EffectsFrame" );
    LayersFrame->setGeometry( QRect( 0, 0, 60, 195 ) );
    LayersFrame->setMinimumSize( QSize( 60, 195 ) );  //was 960
    LayersFrame->setMaximumSize( QSize( 60, 195 ) );

    ///////////////////////////////////////////////////////////
    //The layer options
    AddButton = new QPushButton( LayersFrame, "AddButton" );
    addButton( AddButton,  0, 5, 55, 25,  "Add" );
    connect  ( AddButton,  SIGNAL(clicked()), this, SLOT( addLayer())  );

    NameButton = new QPushButton( LayersFrame, "NameButton" );
    addButton( NameButton, 0, 30, 55, 25, "Name" );
    connect  ( NameButton,  SIGNAL(clicked()), this, SLOT( nameLayer())  );

      namepopup = new FancyPopup( this, "Enter Layer Name",200,150 );  //send size and position as well
      connect ( namepopup,      SIGNAL( returnText(QString)), SLOT( setlayerName(QString)) );

    DelButton = new QPushButton( LayersFrame, "Delete Button" );
    addButton( DelButton, 0, 60, 55, 25 , "Del" );
    connect  ( DelButton,  SIGNAL(clicked()), this, SLOT( delLayer())  );

    //this is now printbutton
    SceneButton = new QPushButton( LayersFrame, "NameButton" );
    addButton( SceneButton, 0, 100, 55, 25, "Data" );
    connect  ( SceneButton,  SIGNAL(clicked()), this, SLOT( printScene())  );       // sceneSettings() now obsolete


    //////////////////////////////////////////////////////////////
    // set up the scene options, or prefernces
    EffectsMain = new QWidgetStack(qtinterface);
    SceneFrame   = new QFrame(EffectsMain);
    EffectsFrame = new QFrame(EffectsMain);

    EffectsMain->addWidget(SceneFrame,0);
    EffectsMain->addWidget(EffectsFrame,1);
    EffectsMain->setGeometry( QRect( 0, 1, 855, 195) );

    EffectsStatus = FALSE; EffectsMain->raiseWidget(1);

    /////////////////////////////////////////////////////////
    //this is the list-view
    lv = new QListView( EffectsFrame );

    connect( lv, SIGNAL( clicked( QListViewItem* ) ), this, SLOT( clicked( QListViewItem* ) ) );

    lv->setGeometry( QRect( 2, 2, 146, 180 ) );
    lv->addColumn( "EFFECTS         ", -1 );
    lv->setSorting( -1,1 );   // disables the autosort

    /////////////////////////////////////////////////////////
    //Set up tabbed interface

    tabframe = new QTabWidget( EffectsFrame, "axiscontrols" );
    tabframe->setGeometry( QRect( 148, 1, 700, 180 ) );
    tabframe->setTabShape( QTabWidget::Triangular );
    tabframe->setAutoMask( FALSE );
    tabframe->setStyle( new QPlatinumStyle );

    scenecontrols = new QWidget( EffectsFrame, "scenesettings" );
    tabframe->insertTab( scenecontrols, tr( "  SCENE  " ) );

    objectcontrols = new QWidget( EffectsFrame, "objectcontrols" );
    tabframe->insertTab( objectcontrols, tr( "  EFFECTS  " ) );

    axiscontrols = new QWidget( EffectsFrame, "axiscontrols" );
    tabframe->insertTab( axiscontrols, tr( "  CONTROLS  " ) );

    //texturecontrols = new QWidget( EffectsFrame, "axiscontrols" );
    //tabframe->insertTab( texturecontrols, tr( "  TEXTURE  " ) );

    //effectscontrols = new QWidget( EffectsFrame, "axiscontrols" );
    //tabframe->insertTab( effectscontrols, tr( "  EFFECTS  " ) );


    /////////////////////////////////////////////////////////////////////
    // the layout boxes for different types of layers
    // these should be changed and regroups to represent tabs above

    // the axis control tab bar
    fxoptionals = new QWidgetStack(axiscontrols);

     fxoptionals_clipone = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_clipone,0);
     fxoptionals_cliptwo = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_cliptwo,1);
     fxoptionals_object  = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_object, 2);
     fxoptionals_world   = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_world,  3);
     fxoptionals_text    = new QHBox(fxoptionals);     fxoptionals->addWidget(fxoptionals_text,   4);

    fxoptionals->setGeometry( QRect( 0, 0, 700, 180) );


    ///////////////////////////////////////////////////////////////////
    // the object control tab bar

    fxselect = new QWidgetStack(objectcontrols);

     fxselect_clip     = new QHBox(fxselect);          fxselect->addWidget(fxselect_clip,    0);
     fxselect_text     = new QHBox(fxselect);          fxselect->addWidget(fxselect_text,    1);
     fxselect_world    = new QHBox(fxselect);          fxselect->addWidget(fxselect_world,   2);
     fxselect_key      = new QHBox(fxselect);          fxselect->addWidget(fxselect_key,     3);
     fxselect_particle = new QHBox(fxselect);          fxselect->addWidget(fxselect_particle,4);
     fxselect_object   = new QHBox(fxselect);          fxselect->addWidget(fxselect_object,  5);
	 fxselect_maxobject= new QHBox(fxselect);          fxselect->addWidget(fxselect_maxobject,  6);

    fxselect->setGeometry( QRect( 0, 0, 700, 180) );   // was 635

    ///////////////////////////////////////////////////////////////////
    // the scene control tab bar

    //fxobject->setGeometry( QRect( 0, 0, 700, 120) );   // was 635
    LoadButton = new QPushButton( scenecontrols, "LoadButton" );
    addButton( LoadButton, 10, 10, 86, 25 , "Load" );
    connect  ( LoadButton,  SIGNAL(clicked()), this, SLOT( SceneLoad())  );

    SaveButton = new QPushButton( scenecontrols, "SaveButton" );
    addButton( SaveButton, 10, 45, 86, 25, "Save" );
    connect  ( SaveButton,  SIGNAL(clicked()), this, SLOT( SceneSave())  );

    ResetButton = new QPushButton( scenecontrols, "ResetButton" );
    addButton( ResetButton, 10, 80, 86, 25, "Reset All" );
    connect  ( ResetButton,  SIGNAL(clicked()), this, SLOT( ResetAll())  );

    ClearButton = new QPushButton( scenecontrols, "ResetButton" );
    addButton( ClearButton, 110, 10, 86, 25, "Clear Scene" );
    connect( ClearButton,  SIGNAL(clicked()), this, SLOT( ClearAll())  );

    sframetext = new QLabel( scenecontrols, "sframetext" );
    sframetext->setGeometry( QRect( 400, 4, 80, 27 ) );
    sframetext->setText( tr( "STARTFRAME" ) );
    sframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont sframetext_font(  sframetext->font() );
    sframetext_font.setPointSize( 9 );
    sframetext->setFont( sframetext_font );

    lcdstartframe = new InputLCD( scenecontrols, "startframe",9 );
	  addSliderLcd(lcdstartframe, 500, 5, 90, 21, Astartframe);
	  lcdstartframe->setMinMaxInt( Astartframe, Aendframe-1 );
    lcdstartframe->setValue(Astartframe);
	  connect( lcdstartframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderStartframe(int)) );

    eframetext = new QLabel( scenecontrols, "eframetext" );
    eframetext->setGeometry( QRect( 400, 34, 80, 27 ) );
    eframetext->setText( tr( "ENDFRAME"  ) );
    eframetext->setAlignment( int( QLabel::AlignCenter ) );
    QFont eframetext_font(  eframetext->font() );
    eframetext_font.setPointSize( 9 );
    eframetext->setFont( eframetext_font );

//addSliderLcd(animframe, 5, 0, 90, 31, 100);
    lcdendframe = new InputLCD( scenecontrols, "endframe",9 );
	  addSliderLcd(lcdendframe, 500, 34, 90, 21, Aendframe);
	  lcdendframe->setMinMaxInt( Astartframe+1, MaxFrames );
    lcdendframe->setValue(Aendframe);
	  connect( lcdendframe, SIGNAL(valueChanged(int)),   SLOT(updatesliderEndframe(int)) );

/*    //these are checkboxes...
    SmoothSelect = new QCheckBox( scenecontrols, "SmoothSelect" );
    addCheckButton( SmoothSelect, 300, 10, 110, 21, "aliasing" );
    connect( SmoothSelect,  SIGNAL(clicked()), this, SLOT(setSmooth())  );

    BlurSelect = new QCheckBox( scenecontrols, "BlurSelect" );
    addCheckButton( BlurSelect, 300, 45, 110, 21, "motion blur" );
    connect( BlurSelect,  SIGNAL(clicked()), this, SLOT(setBlur())  );

    TextSelect = new QCheckBox( scenecontrols, "TitleSafe" );
    addCheckButton( TextSelect, 300, 80, 110, 21, "title safe" );
    connect( TextSelect,  SIGNAL(clicked()), this, SLOT(setTextSafe())  );
  */


    QPalette p( QColor( 0, 0, 0 ) );
    p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::green );

    ///////////////////////////////////////////////////////////////////////
    //set up object control set and sliders

    ControlFrame1 = new QFrame( fxoptionals_clipone, "ControlFrame1" );
    ControlFrame1->setGeometry( QRect( 0, 0, 760, 120 ) );    // was 510

    // set up rotate sliders
    RotateText_1 = new QLabel( ControlFrame1, "RotateText_1" );
    addLabel( RotateText_1, 10, 10, 110, 21, "OBJROTATE", 9);

    RotX_1 = new QSlider( ControlFrame1 );   RotXlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, RotX_1, 0, 30, 10, 21,  100, RotXlcd_1,  "X", 9);
    connect( RotX_1, SIGNAL(valueChanged(int)), this, SLOT(setXRotation(int)) );
    connect( RotX_1, SIGNAL(valueChanged(int)), RotXlcd_1, SLOT(display(int)) );
	  connect(RotXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXRotation(int)) );

    RotY_1 = new QSlider( ControlFrame1 );   RotYlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, RotY_1, 0, 60, 10, 21,  100, RotYlcd_1,  "Y", 9);
    connect( RotY_1, SIGNAL(valueChanged(int)), this, SLOT(setYRotation(int)) );
    connect( RotY_1, SIGNAL(valueChanged(int)), RotYlcd_1, SLOT(display(int)) );
	  connect(RotYlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setYRotation(int)) );

    RotZ_1 = new QSlider( ControlFrame1 );   RotZlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, RotZ_1, 0, 90, 10, 21,  100, RotZlcd_1,  "Z", 9);
    connect( RotZ_1, SIGNAL(valueChanged(int)), this, SLOT(setZRotation(int)) );
    connect( RotZ_1, SIGNAL(valueChanged(int)), RotZlcd_1, SLOT(display(int)) );
	  connect(RotZlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setZRotation(int)) );


    // set up translate sliders
    TranslateText_1 = new QLabel( ControlFrame1, "TranslateText_1" );
    addLabel( TranslateText_1, 180, 10, 110, 21, "TRANSLATE", 9);
    TranX_1 = new QSlider( ControlFrame1 );   TranXlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, TranX_1, 180, 30, 10, 21,  100, TranXlcd_1,  "X", 9);

    connect( TranX_1,   SIGNAL(valueChanged(int)), this, SLOT(setXTrans(int)) );
    connect( TranX_1,   SIGNAL(valueChanged(int)), TranXlcd_1, SLOT(display(int)) );
	  connect(TranXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXTrans(int)) );

    TranY_1 = new QSlider( ControlFrame1 );   TranYlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, TranY_1, 180, 60, 10, 21,  100, TranYlcd_1,  "Y", 9);
    connect( TranY_1, SIGNAL(valueChanged(int)), this, SLOT(setYTrans(int)) );
    connect( TranY_1, SIGNAL(valueChanged(int)), TranYlcd_1, SLOT(display(int)) );
	  connect(TranYlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setYTrans(int)) );

    TranZ_1 = new QSlider( ControlFrame1 );   TranZlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, TranZ_1, 180, 90, 10, 21,  100, TranZlcd_1,  "Z", 9);
    connect( TranZ_1,   SIGNAL(valueChanged(int)), this, SLOT(setZTrans(int)) );
    connect( TranZ_1,   SIGNAL(valueChanged(int)), TranZlcd_1, SLOT(display(int)) );
	  connect(TranZlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setZTrans(int)) );


    // set up scale sliders
    ScaleText_1 = new QLabel( ControlFrame1, "ScaleText_1" );
    addLabel( ScaleText_1, 350, 10, 110, 21, "SCALE", 9);

    ScaleX_1 = new QSlider( ControlFrame1 );   ScaleXlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, ScaleX_1, 350, 30, 10, 21,  100, ScaleXlcd_1,  "X", 9);
    connect( ScaleX_1, SIGNAL(valueChanged(int)), this, SLOT(setXScale(int)) );
    connect( ScaleX_1, SIGNAL(valueChanged(int)), ScaleXlcd_1, SLOT(display(int)) );
	  connect(ScaleXlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setXScale(int)) );

    ScaleY_1 = new QSlider( ControlFrame1 );   ScaleYlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, ScaleY_1, 350, 60, 10, 21,  100, ScaleYlcd_1,  "Y", 9);
    connect( ScaleY_1, SIGNAL(valueChanged(int)), this, SLOT(setYScale(int)) );
    connect( ScaleY_1, SIGNAL(valueChanged(int)), ScaleYlcd_1, SLOT(display(int)) );
	  connect(ScaleYlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setYScale(int)) );

    ScaleZ_1 = new QSlider( ControlFrame1 );   ScaleZlcd_1 = new InputLCD( ControlFrame1 );
    addSliderLcdLabel( ControlFrame1, ScaleZ_1, 350, 90, 10, 21,  100, ScaleZlcd_1,  "Z", 9);
    connect( ScaleZ_1, SIGNAL(valueChanged(int)), this, SLOT(setZScale(int)) );
    connect( ScaleZ_1, SIGNAL(valueChanged(int)), ScaleZlcd_1, SLOT(display(int)) );
	  connect(ScaleZlcd_1, SIGNAL(valueChanged(int)), this, SLOT(setZScale(int)) );


    /////////////////////////////////////////////////////////////////////////////
    //world control settings
    ControlFrame4 = new QFrame( fxoptionals_world, "ControlFrame4" );
    ControlFrame4->setGeometry( QRect( 0, 0, 660, 120 ) );

    // set up rotate sliders
    RotateText_4 = new QLabel( ControlFrame4, "RotateText_4" );
    addLabel( RotateText_4, 10, 10, 110, 21, "ROTATE", 9);

    RotX_4 = new QSlider( ControlFrame4 );   RotXlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, RotX_4, 10, 30, 10, 21,  100, RotXlcd_4,  "X", 9);
    connect( RotX_4, SIGNAL(valueChanged(int)), this, SLOT(setWorldXR(int)) );
    connect( RotX_4, SIGNAL(valueChanged(int)), RotXlcd_4, SLOT(display(int)) );
	  connect(RotXlcd_4, SIGNAL(valueChanged(int)), this, SLOT(setWorldXR(int)) );

    RotY_4 = new QSlider( ControlFrame4 );   RotYlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, RotY_4, 10, 60, 10, 21,  100, RotYlcd_4,  "Y", 9);
    connect( RotY_4, SIGNAL(valueChanged(int)), this, SLOT(setWorldYR(int)) );
    connect( RotY_4, SIGNAL(valueChanged(int)), RotYlcd_4, SLOT(display(int)) );
	  connect(RotYlcd_4, SIGNAL(valueChanged(int)), this, SLOT(setWorldYR(int)) );

    RotZ_4 = new QSlider( ControlFrame4 );   RotZlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, RotZ_4, 10, 90, 10, 21,  100, RotZlcd_4,  "Z", 9);
    connect( RotZ_4, SIGNAL(valueChanged(int)), this, SLOT(setWorldZR(int)) );
	  connect(RotZ_4, SIGNAL(valueChanged(int)), RotZlcd_4, SLOT(display(int)) );
    connect( RotZlcd_4, SIGNAL(valueChanged(int)), RotZlcd_4, SLOT(setWorldZR(int)) );


    // set up translate sliders
    TranslateText_4 = new QLabel( ControlFrame4, "TranslateText_4" );
    addLabel( TranslateText_4, 200, 10, 110, 21, "TRANSLATE", 9);

    TranX_4 = new QSlider( ControlFrame4 );   TranXlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, TranX_4, 200, 30, 10, 21,  100, TranXlcd_4,  "X", 9);
    connect( TranX_4, SIGNAL(valueChanged(int)), this, SLOT(setWorldXT(int)) );
    connect( TranX_4, SIGNAL(valueChanged(int)), TranXlcd_4, SLOT(display(int)) );
    connect( TranXlcd_4, SIGNAL(valueChanged(int)), RotZlcd_4, SLOT(setWorldXT(int)) );

    TranY_4 = new QSlider( ControlFrame4 );   TranYlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, TranY_4, 200, 60, 10, 21,  100, TranYlcd_4,  "Y", 9);
    connect( TranY_4, SIGNAL(valueChanged(int)), this, SLOT(setWorldYT(int)) );
    connect( TranY_4, SIGNAL(valueChanged(int)), TranYlcd_4, SLOT(display(int)) );
    connect( TranYlcd_4, SIGNAL(valueChanged(int)), RotZlcd_4, SLOT(setWorldYT(int)) );

    TranZ_4 = new QSlider( ControlFrame4 );   TranZlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, TranZ_4, 200, 90, 10, 21,  100, TranZlcd_4,  "Z", 9);
    connect( TranZ_4, SIGNAL(valueChanged(int)), this, SLOT(setWorldZT(int)) );
    connect( TranZ_4, SIGNAL(valueChanged(int)), TranZlcd_4, SLOT(display(int)) );
    connect( TranZlcd_4, SIGNAL(valueChanged(int)), RotZlcd_4, SLOT(setWorldZT(int)) );

    // set up zoom slider
    ScaleText_4 = new QLabel( ControlFrame4, "ScaleText_4" );
    addLabel( ScaleText_4, 370, 10, 110, 21, "ZOOM", 9);

    ScaleX_4 = new QSlider( ControlFrame4 );   ScaleXlcd_4 = new InputLCD( ControlFrame4 );
    addSliderLcdLabel( ControlFrame4, ScaleX_4, 370, 30, 10, 21,  100, ScaleXlcd_4,  "Z", 9);
    connect( ScaleX_4, SIGNAL(valueChanged(int)), ScaleXlcd_4, SLOT(display(int)) );


    // the world grid activation button
    GridSelect = new QCheckBox( ControlFrame4 );
    addCheckButton( GridSelect, 370, 70, 110, 21, "grid" );
    connect( GridSelect,  SIGNAL(clicked()), this, SLOT(setGrid())  );


    ///////////////////////////////////////////////////////////////
    // object layer options
    ControlFrame5 = new QFrame( fxselect_clip, "ControlFrame5" );
    ControlFrame5->setGeometry( QRect( 0, 0, 660, 120 ) );

    loadimage_1 = new QPushButton( ControlFrame5, "loadimage_1" );
    addButton( loadimage_1, 0, 10, 110, 25, "Load Image" );
    connect( loadimage_1,  SIGNAL(clicked()), this, SLOT(loadClip())  );

    grabdesktop = new QPushButton( ControlFrame5, "grabdesktop" );
    addButton( grabdesktop, 140, 10, 110, 25, "Grab Desktop" );
    connect( grabdesktop,  SIGNAL(clicked()), this, SLOT(grabDesktop())  );

    //add a subroutine for creating icons
  	lighticon	= new QToolButton(ControlFrame5,"lighticon");
    lighticon->setGeometry( QRect( 600, 10, 18, 26 ) );
  	lighticon->setFixedSize(18, 26);  	//lighticon->setUsesBigPixmap(true);
  	lighticon->setAutoRaise(true);

  	QIconSet	*icoSet;
    	icoSet = new QIconSet(QPixmap((const char**)light_xpm) );
    	icoSet->setPixmap(QPixmap((const char**)light_xpm), QIconSet::Large, QIconSet::Active);
    	icoSet->setPixmap(QPixmap((const char**)light_xpm), QIconSet::Large, QIconSet::Disabled);
    	lighticon->setIconSet(*icoSet);
    delete icoSet;

    light1 = new QCheckBox( ControlFrame5, "light1" );
    light1->setGeometry( QRect( 600, 36, 20, 21 ) );
    connect( light1,  SIGNAL(clicked()), this, SLOT(toggleLight())  );


    Transparency_1 = new QLabel( ControlFrame5, "ScaleText_2" );
    addLabel( Transparency_1, 400, 5, 110, 21, "TRANSPARENCY", 9);
    Trans_1 = new QSlider( ControlFrame5 );  Translcd_1 = new InputLCD( ControlFrame5 );
    addSliderLcdLabel( ControlFrame5, Trans_1,   400, 25, 10, 21,  1, 100,    Translcd_1,  "T", 9);
    connect( Trans_1,    SIGNAL(valueChanged(int)), Translcd_1, SLOT(display(int)) );
    connect( Trans_1,    SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );
    connect( Translcd_1, SIGNAL(valueChanged(int)), this, SLOT(setAlpha(int)) );


    //////////////////////////////////////////////////////////////////
    // key options

    ControlFrame8 = new QFrame( fxselect_key, "ControlFrame4" );
    ControlFrame8->setGeometry( QRect( 0, 0, 40, 120 ) );

    loadimage_2 = new QPushButton( ControlFrame8, "ClipOne_2" );
    addButton( loadimage_2, 0, 20, 111, 30, "Load Image" );
    connect( loadimage_2,  SIGNAL(clicked()), this, SLOT(loadClip())  );

    keyimage_2 = new QPushButton( ControlFrame8, "keyOne_2" );
    addButton( keyimage_2, 115, 20, 35, 30, "IM" );
    connect( keyimage_2,  SIGNAL(clicked()), this, SLOT(keyCliplayer())  );

    loadkey = new QPushButton( ControlFrame8, "ClipOne_2" );
    addButton( loadkey, 0, 55, 111, 30, "Load Key" );
    connect( loadkey,  SIGNAL(clicked()), this, SLOT(loadKey())  );

    keyimage_3 = new QPushButton( ControlFrame8, "keyOne_3" );
    addButton( keyimage_3, 115, 55, 35, 30, "KM" );
    connect( keyimage_3,  SIGNAL(clicked()), this, SLOT(keyClipkey())  );

    Transparency_2 = new QLabel( ControlFrame8, "ScaleText_2" );           //use label function here
    addLabel( Transparency_2, 0, 90, 110, 21, "Invert Key", 9);

    keyStatusBox = new QPushButton( ControlFrame8, "Select1" );
    addButton( keyStatusBox, 0, 90, 111, 30, "Invert Key" );
    connect( keyStatusBox,  SIGNAL(clicked()), this, SLOT(InvertKeyData())  );



    ////////////////////////////////////////////////////////
    // raise the control bar and raise backgound as default

    fxoptionals->raiseWidget(3);
    fxselect->raiseWidget(0);

}























