/*******************************************************************************
**
** The source file for the Jahshaka effects objects module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "effect.h"


void GLEffect::callEffect( FxCategory thefx,  int i, QImage& newimage ) {

	effectobj = i;

	if (JAHDEBUG)   JahDebug("Building the Effect # ",i);

    switch ( thefx ) {

      case COLORMATRIX        : {  doColorMatrix();         break; }
      case CONTRAST           : {  break; }
      case NEGATIVE           : {  doNegative(newimage);    break; }
      case BLUR               : {  break; }
      case SHARPEN            : {  break; }
      case GREY               : {  doGrey(newimage);        break; }
      case EDGE               : {  doEdge(newimage);        break; }
      case RADIALINES         : {  doRadialLines(newimage); break; }
      case POLAR              : {  doPolar(newimage);       break; }
      case SWAPRGB            : {  doSwapRGB(newimage);     break; }
      case MIRROR             : {  doMirror(newimage);      break; }

	  //geometry effects here using meshlayer
	  case SWIRLRT            : {  doSwirl();				break; }
      case FISHEYE            : {  doFisheye();             break; }

      default                 : {  break; }

    }
}

// this is for reference
/*
class effectLayer {
public:
    effectLayer(void);
    ~effectLayer(void);
    int         effectnumber;
    QString     effectname;
    FxCategory  objtype;
    bool        selected;
    bool        layerStatus;
    bool        isOn;
    GLfloat     x,y,z;
    GLfloat     r;
    bool        active;
};
*/

void GLEffect::doSwapRGB(QImage& theimage)	{
	theimage = theimage.swapRGB();
	if (JAHDEBUG) JahDebug("swap");
}

void GLEffect::doMirror(QImage& theimage)	{
    bool VERT = TRUE;  bool HORIZ = TRUE;
    theimage = theimage.mirror(VERT,HORIZ);
	if (JAHDEBUG) JahDebug("mirror");
}

void GLEffect::doNegative(QImage& theimage) {
	theimage.invertPixels();
	if (JAHDEBUG) debug("negative");
}


void GLEffect::doContrast()	{
//color contrast
      //layers[1].Red = TintR; layers[1].Green = TintG; layers[1].Blue = TintB;
}


void GLEffect::doBlur(void) {

GLfloat  horizontal[3][3] = {
    { 0, -1, 0 },
    { 0,  1, 0 },
    { 0,  0, 0 }
};

   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
   glClearColor(0.0, 0.0, 0.0, 0.0);

   if (JAHDEBUG) printf("Using the horizontal filter\n");

   glPushMatrix();
//     glConvolutionFilter2D(GL_CONVOLUTION_2D, GL_LUMINANCE,
//  			 3, 3, GL_LUMINANCE, GL_FLOAT, horizontal);
//     glEnable(GL_CONVOLUTION_2D);
//     glDisable(GL_CONVOLUTION_2D);
   glPopMatrix();

}



void GLEffect::doEdge(QImage& pix) {

    //QImage newimage;   newimage = pix.copy();
    //doGrey(pix);

    int r,g,b,grey;
	

	int yy = layers[0].textureData.height;
	int xx = layers[0].textureData.width;

    for ( int y=0; y<yy; y++ ) {
        for ( int x=0; x<xx; x++ )   {

           QRgb *p = (QRgb *)pix.scanLine(y) + x;
           r=qRed(*p); g=qGreen(*p); b=qBlue(*p); grey=( (r+g+b)/3 );
           *p = qRgb(grey,g,b);

        }
    }

    int   p[4];
    float delta_u = 0;
    float delta_v = 0;
    short t;

    for ( int x=0; x<(pix.width()-1); x++ ) {
        for ( int y=0; y<(pix.height()-1); y++ )   {

            //we need to speed up this part of the loop
            //uint *p[0] = (uint *)pix.scanLine(y) + x;

              p[0] = pix.pixel(x,y);
              p[1] = pix.pixel((x+1),y);
              p[2] = pix.pixel(x, (y+1));
              p[3] = pix.pixel((x+1),(y+1));

            delta_u = p[0] - p[3];  //was 3
            delta_v = p[1] - p[2];  //was 2

            t = (short) sqrt( (delta_u*delta_u) + (delta_v*delta_v) );

            QRgb *p = (QRgb *)pix.scanLine(y) + x;
            *p = qRgb(t,t,t);

        }
    }
   // canvas->setImageBuffer(pix);
}

void GLEffect::doGrey(QImage& pix) {

    int r,g,b,grey;
	QString conv,text;

	//note that this crashes when x or y are over the scope
	//for some reason this happens with the default image

	int yy = layers[0].textureData.height;
	int xx = layers[0].textureData.width;

	text="width is :";  debug(text + conv.setNum( xx )  );
	text="height is :"; debug(text + conv.setNum( yy )  );

    for ( int y=0; y<yy; y++ ) {

        for ( int x=0; x<xx; x++ )   {

        //if we can do this one SCANLINE at a time it would be faster
        //uchar *pe = pix.scanLine(y);
			
           QRgb *p = (QRgb *)pix.scanLine(y) + x; //debug("here1");
           r=qRed(*p); g=qGreen(*p); b=qBlue(*p); //debug("here2");
		   grey=( (r+g+b)/3 ); //debug("here3");
           *p = qRgb(grey,grey,grey);

        }
		
    }

}



void GLEffect::doSharpen(void) {

    //we need to toggle this when ever its turned off...
bool    matrixMode = TRUE;
    int width = 9;

//    GLfloat convolveMat[3][3] = { -ColorMR, -ColorMR,     -ColorMR,
//                                  -ColorMR, 1+8*ColorMR,  -ColorMR,
//                                  -ColorMB ,-ColorMR,     -ColorMR };


   // glConvolutionFilter2D(GL_CONVOLUTION_2D, GL_LUMINANCE, 3,3, GL_LUMINANCE,
  //                        GL_FLOAT, convolveMat[0]);

    glMatrixMode(GL_COLOR);
    glPushMatrix();

	/*

    GLfloat convolve_2d[300][200];
    GLfloat convolve_1d[500];
    glMatrixMode(GL_COLOR);
    glConvolutionFilter2D(GL_CONVOLUTION_2D, GL_LUMINANCE, 300,200, GL_LUMINANCE,
                          GL_FLOAT, convolve_2d[]);

    glConvolutionFilter1D(GL_CONVOLUTION_1D, GL_LUMINANCE, 500, GL_LUMINANCE,
                          GL_FLOAT, convolve_1d);

  //  glMatrixMode(GL_COLOR);
  //  glPushMatrix();
    glLoadMatrixf(convolveMat);
  */

}


void GLEffect::doRadialLines(QImage& pix ) {

	int obj=1;

    int w = layers[0].textureData.width;
    int h = layers[0].textureData.height;

    QImage newimage;   newimage = pix.copy();

    int xc = layers[activeLayer].node->tx + (w/2);
    int yc = layers[activeLayer].node->ty + (h/2);

    //int xc = effects[obj]->x;  //int yc = Ycenter;    int xc = Xcenter;     int xc = w/2;
    //int yc = effects[obj]->y;  //   int yc = h/2; //yc=200;  int yc = Ycenter;   int yc = h/2;

    double gamma = 2.0;
    float  p[2]; // = new double [2];
    int xp, yp;

 //   double R = effects[obj]->r; //sqrt(w*w/4+h*h/4);  //    double R = Radius;
    double R = w/2; //sqrt(w*w/4+h*h/4);  //    double R = Radius;

    double dx,dy,radius,u,a;
    double PI = 3.14159265358979323846;

    //radius affects x,y and dx,dy here need some work
    for (int x=0; x < w; x++) {
        for (int y=0; y < h; y++) {

            dx = x-xc;
            dy = y-yc;
           // radius =R;
            radius = sqrt(dx*dx + dy*dy)-(R+100);
                if (radius<=0) radius = 0;

            u = pow(radius,gamma)/R;
            a = (180/PI)*atan2(dy,dx);
            a = a + radius;
            a = a * PI/180;

            p[0] = radius * cos(a);
            p[1] = radius * sin(a);
            xp = (int) p[0]+xc;
            yp = (int) p[1]+yc;

            if ( (xp < w) && (yp < h) && (xp >=0) && (yp >= 0)) {
               uint *pnew = (uint *)newimage.scanLine(y) + x;
               uint *pold = (uint *)pix.scanLine(yp) + xp;
               *pnew = *pold;
           //     newimage.setPixel(x,y, ( pix.pixel(xp,yp) ) );
            }
        }
    }
    pix = newimage;
}

void GLEffect::doPolar(QImage& pix ) {

	int obj=1;

    int w = layers[0].textureData.width;
    int h = layers[0].textureData.height;

    QImage newimage;   newimage = pix.copy();

    int xc = layers[activeLayer].node->tx + (w/2);
    int yc = layers[activeLayer].node->ty + (h/2);

    //int xc = effects[obj]->x;  //int yc = Ycenter;    int xc = Xcenter;     int xc = w/2;
    //int yc = effects[obj]->y;  //   int yc = h/2; //yc=200;  int yc = Ycenter;   int yc = h/2;

    double gamma = 2.0;
    float  p[2]; // = new double [2];
    int xp, yp;

 //   double R = effects[obj]->r; //sqrt(w*w/4+h*h/4);  //    double R = Radius;
    double R = w/2; //sqrt(w*w/4+h*h/4);  //    double R = Radius;

    double dx,dy,radius,u,a;
    double PI = 3.14159265358979323846;

    //radius affects x,y and dx,dy here need some work
    for (int x=0; x < w; x++) {
        for (int y=0; y < h; y++) {

            dx = x-xc;
            dy = y-yc;
           // radius =R;
            radius = sqrt(dx*dx + dy*dy)-(R+100);
                if (radius<=0) radius = 0;

            u = pow(radius,gamma)/R;
            a = (180/PI)*atan2(dy,dx);
            a = a + radius;
            a = a * PI/180;

            p[0] = radius * cos(a);
            p[1] = radius * sin(a);
            xp = (int) p[0]+xc;
            yp = (int) p[1]+yc;

            if ( (xp < w) && (yp < h) && (xp >=0) && (yp >= 0)) {
               uint *pnew = (uint *)newimage.scanLine(y) + x;
               uint *pold = (uint *)pix.scanLine(yp) + xp;
               *pnew = *pold;
           //     newimage.setPixel(x,y, ( pix.pixel(xp,yp) ) );
            }
        }
    }
    pix = newimage;
	
}


/* This is the old image based fisheye effect
void GLEffect::doFisheye(QImage& pix) {

	
        //if we can do this one SCANLINE at a time it would be faster
        //uchar *pe = pix.scanLine(y);
			
           QRgb *p = (QRgb *)pix.scanLine(y) + x; //debug("here1");
           r=qRed(*p); g=qGreen(*p); b=qBlue(*p); //debug("here2");
		   grey=( (r+g+b)/3 ); //debug("here3");
           *p = qRgb(grey,grey,grey);
	

	int obj=1;

    int w = layers[1].textureData.width;
    int h = layers[1].textureData.height;

    QImage newimage;   newimage = pix.copy();

    //int xc = effects[obj]->x;  //int yc = Ycenter;
    //int yc = effects[obj]->y;  //   int yc = h/2; //yc=200;

	//location of effect - default is center
    int xc = layers[activeLayer].node->tx + (w/2);
    int yc = layers[activeLayer].node->ty + (h/2);

    double gamma = 2.2;
    float  p[2]; // = new double [2];
    int xp, yp;

  //  double R = (sqrt(w*w/4+h*h/4));
      double R = layers[activeLayer].node->rx + (w/2); //effects[obj]->r; //was 100;
 //   double R = Radius;

    double dx,dy,radius,u,a;

    for (int x=0; x < w; x++) {
        for (int y=0; y < h; y++) {

            dx = x-xc; dy = y-yc;
            radius = sqrt(dx*dx + dy*dy);

            u = pow(radius,gamma)/R;
            a = atan2(dy,dx);

            p[0] = u * cos(a); p[1] = u * sin(a);
            xp = (int) p[0]+xc; yp = (int) p[1]+yc;

            if ( (xp < w) && (yp < h) && (xp >=0) && (yp >= 0)) {

            //which routine is faster, scanline or setPixel?
            //  int yoffset; yoffset = 511-yp;      //511-yp
            //  newimage.setPixel(x,y, ( pix.pixel(xp,yp) ) );
            uint *pnew = (uint *)newimage.scanLine(y) + x;
            uint *pold = (uint *)pix.scanLine(yp) + xp;
            *pnew = *pold;

            //  uint *p = (uint *)image.scanLine(y) + x;  *p = qRgb(255,255,0);

            }
        }
    }
    pix = newimage.copy();    //return the image added the copy thing here
}
*/

/*
void GLEffect::doRadialLines(QImage& pix ) {

	int obj=1;

    int w = layers[1].textureData.width;
    int h = layers[1].textureData.height;

    QImage newimage;   newimage = pix.copy();



   // int w; w = (int)layers[activeLayer].textureData.width;;
   // int h; h =  (int)layers[activeLayer].textureData.height;

   // QImage newimage;   newimage = pix.copy();

    int xc = layers[activeLayer].node->tx + (w/2);
    int yc = layers[activeLayer].node->ty + (h/2);

    //int xc = effects[obj]->x;  if (xc>w) xc=w;//int yc = Ycenter;    int xc = Xcenter;     int xc = w/2;
    //int yc = effects[obj]->y;  if (yc>w) yc=h;//   int yc = h/2; //yc=200;  int yc = Ycenter;   int yc = h/2;

    double gamma = 2.0;
    float  p[2]; // = new double [2];
    int xp, yp;

      double R = layers[activeLayer].node->rx + (w/2); //effects[obj]->r; //was 100;

//    double R = effects[obj]->r; //sqrt(w*w/4+h*h/4);  //    double R = Radius;

    double dx,dy,radius,u,a;
    double PI = 3.14159265358979323846;

    //radius affects x,y and dx,dy here need some work
    for (int x=0; x < w; x++) {
        for (int y=0; y < h; y++) {

            dx = x-xc;
            dy = y-yc;
            radius =R; if (radius<=0) radius = 0;

            u = pow(radius,gamma)/R;
            a = (180/PI)*atan2(dy,dx);
            a = a + radius;
            a = a * PI/180;

            p[0] = radius * cos(a);
            p[1] = radius * sin(a);
            xp = (int) p[0]+xc;
            yp = (int) p[1]+yc;

            if ( (xp < w) && (yp < h) && (xp >=0) && (yp >= 0)) {
               uint *pnew = (uint *)newimage.scanLine(y) + x;
               uint *pold = (uint *)pix.scanLine(yp) + xp;
               *pnew = *pold;
           //     newimage.setPixel(x,y, ( pix.pixel(xp,yp) ) );
            }
        }
    }
    pix = newimage;
}
*/
//need to use opengl to do these
/*
void GLEffect::doContrast() //was double power
{

    theCategory = CONTRAST;

    double power = 1.1;
    QRgb mypix;
    int r,g,b;

    QImage pix = canvas->getImageBuffer();

    for ( int y=0; y<(pix.height()); y++ ) {
        // uchar *p = pix.scanLine(y);
        for ( int x=0; x<(pix.width()); x++ )   {
            mypix =pix.pixel(x,y);

            r=qRed(mypix);
            g=qGreen(mypix);
            b=qBlue(mypix);

            int r1 = (short)( 255 * pow(  (r/255.0) ,power ));
            int g1 = (short)( 255 * pow(  (g/255.0) ,power ));
            int b1 = (short)( 255 * pow(  (b/255.0) ,power ));

            mypix=qRgb(r1,g1,b1);
            pix.setPixel(x,y,mypix);
        }
    }

    canvas->setImageBuffer(pix);
}

*/
