#ifndef PAINT_H
#define PAINT_H

/*******************************************************************************
**
** The header file for the Jahshaka animation module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include <glworld.h>
#include <paintdefs.h>

#include "qcheckbox.h"
#include "qtabwidget.h"

class GLPaint :  public  GLWorld {
  Q_OBJECT

public:

GLPaint( QWidget* parent, const char* name,
         QHBox* controls, QHBox* controller, QHBox* leftcontroller, QHBox* rightcontroller,
         int* globalclipnumber  )
        :  GLWorld(parent,name,controller,globalclipnumber ), mousePressed( FALSE ) {

  //clipnumber = globalclipnumber;

  buildInterface(controls);
  buildInterfaceMenus(leftcontroller,rightcontroller);
  initializeObjects();

  };

  GLPaint::~GLPaint() {  }


  void buildInterface( QHBox* qtinterface);
  void buildInterfaceMenus( QHBox* leftcontroller, QHBox* rightcontroller );

  void initializeObjects();

  void refreshGL();

  // History functions.
  void setHistoryWidgets(QWidget *wPrevious, QWidget *wNext);
  void historyStore();
  void historyRetrieve(int relativeLoc);


  // Image Manipulation functions.
  void fade();
  void intensify();
  void invert();
  void convolute(const convolutionType type);
  void clear();



   // Return the given integer value limited within the range 0 to 255.
  int limit0_255(const int & val) {
    if      (val < 0  ) return 0;
    else if (val > 255) return 255;
    else return val;
  }

   // Activate the given tool.
  void activateTool(CanvasTool toolNum) { myActiveTool = toolNum; }
  void makeCheckImage(void);

   // Accessor functions.
  QColor penColor()        { return *myPenColor; }
  QColor fillColor()       { return *myFillColor; }
  QColor backgroundColor() { return *myBackgroundColor; }
  int    brushSize()       { return myBrushSize; }
  int    gradientDegree()  { return myGradientDegree; }

   // Modifier functions.
  void setPenColor (QColor newColor)       { *myPenColor        = newColor; }
  void setFillColor(QColor newColor)       { *myFillColor       = newColor; }
  void setBackgroundColor(QColor newColor) { *myBackgroundColor = newColor; }
  void setBrushSize(int newSize)           { myBrushSize       = newSize;  }
  void setGradientDegree(int newVal)       { myGradientDegree  = newVal; }

//the paint tools
void drawToolPen( int myBrushSize,color3 penColor, GLint minPointSize,
                  int x1, int y1, int x2, int y2  );

void drawToolLine( int myBrushSize,color3 penColorLight,color3 penColorDark, GLint minPointSize,
                   int x1, int y1, int x2, int y2  );

void drawToolRect( int myBrushSize,
                   color3 penColor, color3 penColorLight,color3 penColorDark, GLint minPointSize,
                   int x1, int y1, int x2, int y2  );

void drawToolRectFilled( int myBrushSize,  GLint minPointSize,
                         color3 penColor, color3 penColorLight,color3 penColorDark,
                         color3 fillColor, color3 fillColorLight,color3 fillColorDark,
                         int x1, int y1, int x2, int y2  );


void drawToolCircle( color3 penColor, GLfloat gradientDegree, int x1, int y1, int x2, int y2  );

void drawToolCircleFilled( color3 penColor, color3 fillColor, GLfloat gradientDegree, int x1, int y1, int x2, int y2  );

void drawToolTri ( color3 penColor, color3 penColorLight,color3 penColorDark,
                   int x1, int y1, int x2, int y2  );

void drawToolTriFilled( color3 penColor, color3 penColorLight,color3 penColorDark,
                        color3 fillColor, color3 fillColorLight,color3 fillColorDark,
                        int x1, int y1, int x2, int y2  );
 protected:

  void   drawWithActiveTool();
  void   resizeGL (int w, int h);
  void   initializeGL();
  void   paintGL();

   // Overloaded QT functions.
  virtual void mousePressEvent  ( QMouseEvent* event);
  virtual void mouseReleaseEvent( QMouseEvent* event);
  virtual void mouseMoveEvent   ( QMouseEvent* event);

  QWidget *wHistoryPrevious, *wHistoryNext;
  QImage myHistory[myHistorySize];                  // Used as a circular array.
  int    myHistoryStartLoc, myHistoryEndLoc, myHistoryLoc;

  QImage buffer, workingBuffer;
  QColor *myPenColor, *myFillColor, *myBackgroundColor;
  int    myBrushSize, myActiveTool, myGradientDegree, myFadeDegree;
  int    x1, y1, x2, y2;
  bool   mousePressed, openPic;

  QButtonGroup  *bgDrawingTools;

  /*
  QToolButton	*bClear, *bHistoryPrevious, *bHistoryNext,
                *bInvert, *bFade, *bIntensify,
                *bBlur, *bSharpen,
                *bEdgeDetectX, *bEdgeDetectY,
                *bSobel, *bLaplacian, *bLaplacian2, *bLapOfGauss,
                *bPen, *bLine, *bRectangle, *bRectangleFilled,
                *bCircle, *bCircleFilled, *bTriangle, *bTriangleFilled,
                *bPenColor, *bFillColor, *bBackgroundColor;
  QSlider       *sBrushSize, *sGradientDegree;
  QLabel        *lBrushSize, *lGradientDegree;
  QPopupMenu	*file;
  QMenuBar	*menubar;*/

  QString       myProgramDirectory; // Directory where this program resides.
  QString       myWorkingPath;      // Path in which to look for files.
  QString       myAuthorText;

 protected slots:

  void slotHistoryPrevious();
  void slotHistoryNext();

   // Image Manipulation Slots.
  void slotInvert();
  void slotFade();
  void slotIntensify();
  void slotBlur();
  void slotSharpen();
  void slotEdgeDetectX();
  void slotEdgeDetectY();
  void slotSobel();
  void slotLaplacian();
  void slotLaplacian2();
  void slotLapOfGauss();
  void slotClear();

   // Tool slots.
  void slotPen();
  void slotLine();
  void slotRectangle();
  void slotRectangleFilled();
  void slotCircle();
  void slotCircleFilled();
  void slotTriangle();
  void slotTriangleFilled();

   // Tool configuration slots.
  void slotPenColor();
  void slotFillColor();
  void slotBackgroundColor();
  void slotBrushSize(int value);
  void slotGradientDegree(int value);


public slots:
  void addLayer(void);

  void addClip(videoData clip);
  	
  void clicked( QListViewItem* item );

  void changeParticle();
  void changeFontDraw();
  void changeObjectDraw();
  void keyCliplayer();
  void keyClipkey();
  void updateSliders();
  	
  //void sceneSettings(void);

signals:
  void    GrabClip(void);    //linked to main jahcontrol to talk to desktop
  void    GrabClipKey(void);    //linked to main jahcontrol to talk to desktop

private:

  GLfloat mousex, mousey;

  ////////////////////////////////////////////////////////////////////////////
  //interface variables

  //for the left and right menus
  QFrame         *glworldLeft;	
  QPushButton    *tooltranslate, *toolscale, *toolrotate;

  QFrame         *glworldRight;
  QPushButton    *toolzoomin, *toolzoomout;
  QSlider        *zoom;

  QFrame*      LayersFrame;



  QPushButton*  AddButton;                QPushButton*  DelButton;
  QPushButton*  NameButton;               QPushButton*  SceneButton;
  FancyPopup* namepopup;

  QWidgetStack*  EffectsMain;
  QFrame*        SceneFrame;
  QFrame*        EffectsFrame;
  bool           EffectsStatus;

  QPushButton*   ResetButton;             QPushButton*   SaveButton;    QPushButton*   LoadButton;    QPushButton*   ClearButton;
  QCheckBox*     SmoothSelect;            QCheckBox*     BlurSelect;    QCheckBox*     TextSelect;
    	
  QPushButton*  ResetLayerButton;

  QTabWidget*        tabframe;
  QWidget*        axiscontrols;
  QWidget*        objectcontrols;
  QWidget*        texturecontrols;
  QWidget*        scenecontrols;
  QWidget*        brushcontrols;
  QWidget*        effectscontrols;

  //    QWidgetStack*  fxaxis;
  QHBox*         fxaxis_clipone;     QHBox*         fxaxis_cliptwo;
  QHBox*         fxaxis_object;      QHBox*         fxaxis_world;
  QHBox*         fxaxis_text;

  QFrame*        ControlFrame1;
  QFrame*        sceneframe;
  QLabel*        RotateText_1;
  QLabel*        TranslateText_1;
  QLabel*        ScaleText_1;

  QSlider*       RotX_1;        InputLCD*  RotXlcd_1;   QLabel* RotX_1H;
  QSlider*       RotY_1;        InputLCD*  RotYlcd_1;   QLabel* RotY_1H;
  QSlider*       RotZ_1;        InputLCD*  RotZlcd_1;   QLabel* RotZ_1H;

  QSlider*       TranX_1;       InputLCD*  TranXlcd_1;
  QSlider*       TranY_1;       InputLCD*  TranYlcd_1;
  QSlider*       TranZ_1;       InputLCD*  TranZlcd_1;

  QSlider*       ScaleX_1;      InputLCD*  ScaleXlcd_1;
  QSlider*       ScaleY_1;      InputLCD*  ScaleYlcd_1;
  QSlider*       ScaleZ_1;      InputLCD*  ScaleZlcd_1;

  //world control variables
  QFrame*        ControlFrame4;

  //widget selection sidebar variables
  QFrame*        ControlFrame;       //QWidgetStack*  fxobject;
  QHBox*         fxobject_clip;      QHBox*         fxobject_text;
  QHBox*         fxobject_world;     QHBox*         fxobject_key;
  QHBox*         fxobject_particle;  QHBox*         fxobject_object;
  QHBox*		   fxobject_maxobject;

  //texture options
  QFrame*        ControlFrame2;
  QCheckBox*     ClipSelect;

  //world option box
  QFrame*        ControlFrame3;

  //layer options
  QFrame*        ControlFrame5;
  QLabel*        ClipLabel;

  QPushButton*   loadimage_1;        QPushButton*   grabdesktop;
  QToolButton*   lighticon;          QLabel*        light1text;      QCheckBox*     light1;
  QLabel*        Transparency_1;     QSlider*       Trans_1;         InputLCD*   Translcd_1;

  //key options
  QFrame*        ControlFrame8;
  QPushButton*   loadimage_2;        QPushButton*  keyimage_2;       QPushButton*  loadkey; QPushButton*  keyimage_3;
  QCheckBox*     KeySelect;

  QSlider*      Trans_2;          QLCDNumber*   Translcd_2;
  QPushButton* keyStatusBox;

  //video labels
  QLabel*			VideoLabel;
  InputLCD*		FramesLcd;		InputLCD*		inFramesLcd;	InputLCD*		outFramesLcd;
  InputLCD*		KeyFramesLcd;	InputLCD*		inKeyFramesLcd;	InputLCD*		outKeyFramesLcd;
  QToolButton*    lockicon;

  //font options
  QFrame*        ControlFrame6;
  QLabel*        ScaleText_5;        QSlider*      ScaleX_5;         InputLCD*  ScaleXlcd_5;
  QLineEdit*     textEd;
  QPushButton*   textFont;           QPushButton*  textColor;        QPushButton*  textDraw;

  //particles
  QFrame*        ControlFrame7;      QPushButton*  loadimage_3;      QPushButton*  particlestyle;

  //objects
  QFrame*        ControlFrame9;      QPushButton*  loadobject_1;     QPushButton*  objectstyle;

  QFrame*        ControlFrame10;     QPushButton*  loadmaxobject_1;
  QFrame*        ControlFrame11;     QCheckBox*    ForegroundSelect;

  QPushButton*	ObjectLayer;
  QPushButton*	ObjectCube;
  QPushButton*	ObjectCylinder;
  QPushButton*	ObjectSphere;
  QPushButton*	ObjectBezier;
  QPushButton*	ObjectMesh;

  //paint brushes
  QPushButton*	paintbrush1;
  QPushButton*	paintbrush2;

  QPushButton*	paintclip1;
  QPushButton*	paintclip1grab;
  QPushButton*	paintclip2;
  QPushButton*	paintclip2grab;
  QPushButton*	paintcolor;

  QPushButton *ControlFrame1B1, *ControlFrame1B2, *ControlFrame1B3, *ControlFrame1B4,
  	         	*ControlFrame1B5, *ControlFrame1B6, *ControlFrame1B7, *ControlFrame1B8,
  	        	*ControlFrame1B9, *ControlFrame1B10;

  QToolButton     *ControlFrame1B20, *ControlFrame1B21, *ControlFrame1B22, *ControlFrame1B23,
  	            	*ControlFrame1B24, *ControlFrame1B25, *ControlFrame1B26, *ControlFrame1B27;

  QSlider			*ControlFrame1S1,    *ControlFrame1S2;
  InputLCD		*ControlFrame1S1Lcd, *ControlFrame1S2Lcd;
  QLabel			*ControlFrame1L1,    *ControlFrame1L2,    *ControlFrame1L3;

  QPushButton     *ControlFrame4B1,  *ControlFrame4B2,  *ControlFrame4B3;
  QToolButton     *ControlFrame4B4, *ControlFrame4B5,  *ControlFrame4B6,  *ControlFrame4B7,  *ControlFrame4B8;
  					
  QSlider			*ControlFrame4S1, *ControlFrame4S2;
  InputLCD		*ControlFrame4S1Lcd, *ControlFrame4S2Lcd;
  QLabel			*ControlFrame4L1, *ControlFrame4L2, *ControlFrame4L3;

  QPushButton     *ControlFrame5B1;
  QToolButton     *ControlFrame5B2,  *ControlFrame5B3,  *ControlFrame5B4;

  QToolButton     *ControlFrame7B1, *ControlFrame7B2, *ControlFrame7B3, *ControlFrame7B4,
  		            *ControlFrame7B5, *ControlFrame7B6, *ControlFrame7B7, *ControlFrame7B8,
  		            *ControlFrame7B9, *ControlFrame7B10;

  QPushButton *effectInvert, *effectFade,  *effectIntensify, *effectBlur,     *effectSharpen,
              *effectEdgeX,  *effectEdgeY, *effectSobel,     *effectLapacian, *effectLapacian2, *effectGauss;


};



#endif // ANIMATION_H





