/*******************************************************************************
**
** The source file for the Jahshaka jahcontrol module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

////////////////////////////////////////////////////////////////
// This is the main class that creates and launches the jahshaka interface
// and sets up and connects all the modules and interface objects

#include "jahcreate.h"
#include "qlabel.h"

//extern JahPathObject JahPaths;
//  jahapp = new JahControl(  this , mainleftmenu, mainworld, mainrightmenu, maincontroller );

JahControl::JahControl( QWidget *parent, const char *name,
                        QFrame *mainleftmenu, QFrame *mainworld, QFrame *mainrightmenu,
                        QFrame *maincontroller,
                        QFrame *moduleselection, QFrame *modulesettings, QFrame *feedbackbar )
{

  ///////////////////////////////////////////////////////////
  // We set up main application  gobals here

  globalclipnumber = 0;

  //get this from preferences
  getGlobalRenderQuality();

  //////////////////////////////////////////////////////////////
  //create the main frame object controllers
  //used to switch the framesets out according to
  //what module we are currently in

  //////////////////////////////////////////////////////////////
  //create the main controler for the world space object

  mainworldLayout = new QHBoxLayout( mainworld, 0, 0, "mainworldLayout");
  WorldLayout = new QWidgetStack(mainworld);
  mainworldLayout->addWidget( WorldLayout );

  //////////////////////////////////////////////////////////////
  //create the main left menu controler

  mainworldLeftLayout = new QHBoxLayout( mainleftmenu, 0, 0, "mainworldLeftLayout");
  MainLeftControler = new QWidgetStack(mainleftmenu);
  mainworldLeftLayout->addWidget( MainLeftControler );

  //////////////////////////////////////////////////////////////
  //create the main right menu controler

  mainworldRightLayout = new QHBoxLayout( mainrightmenu, 0, 0, "mainworldRightLayout");
  MainRightControler = new QWidgetStack(mainrightmenu);
  mainworldRightLayout->addWidget( MainRightControler );

  //////////////////////////////////////////////////////////////
  //create the middle frame menu/option controler

  maincontrollerLayout = new QHBoxLayout( maincontroller, 0, 0, "maincontrollerLayout");
  MainControler = new QWidgetStack(maincontroller);
  maincontrollerLayout->addWidget( MainControler );

  //////////////////////////////////////////////////////////////
  //Create Interface frame and switching controllers for modules

  optionals = new QWidgetStack(modulesettings);
  optionals->setGeometry( QRect( 5, 1, 860, 200) );


  //////////////////////////////////////////////////////////////
  //connect this to the splash screen through the  parent object

  connect( this,  SIGNAL(creatingModule(const QString &)), parent, SLOT(JahchangeModeText(const QString &))  );
  connect( this,  SIGNAL(finishedModule(int)), parent, SLOT(JahsetProgressValue(int))  );


  /////////////////////////////////////////////////////////////////
  //Create Desktop interface and buttons
  //we should be doing this in desktopinterface.cpp so it can create
  //its own interface like all the other modules...

  //Set up the desktop menu button
  emit creatingModule("Desktop");

  Desktop = new QPushButton( moduleselection, "Desktop" );
  Desktop->setGeometry( QRect( 10, 10, 111, 30 ) );
  Desktop->setText( tr( "Desktop"  ) );
  connect( Desktop, SIGNAL(clicked()), SLOT(changeModeDesktop()) );

  desktopdisplay = new QHBox (WorldLayout, "desktop");
  CreateDesktopModule(desktopdisplay);
  WorldLayout->addWidget(desktopdisplay,0);

  //setup desktop feedback bar here or pass it to main setup above
  CreateDesktopFeedbackBox(feedbackbar);

  //connec the desktop preferences to parent object
  connect( this,  SIGNAL(emitStyleColor(QPalette)), parent, SLOT(setStyleColor(QPalette))  );
  connect( thedesktop,  SIGNAL(resetGlobalClipNumber()),  this, SLOT(resetGlobalClipNumber())  );

  emit finishedModule(0+1);

  /////////////////////////////////////////////////////////////////
  //Now we Create and setup the desktop related modules

  /////////////////////////////////////////////////////////////////
  //we set up the player application here

  emit creatingModule("Player");

  //playerdisplay = new QHBox (WorldLayout, "player");
  playerdisplay = new QFrame (WorldLayout, "player");
  CreatePlayerModule(playerdisplay);
  WorldLayout->addWidget(playerdisplay,1);
  emit finishedModule(1+1);

  /////////////////////////////////////////////////////////////////
  //we set up the network module  here

  emit creatingModule("Network");

  networkdisplay = new QHBox (WorldLayout, "network");
  CreateNetworkModule(networkdisplay);
  WorldLayout->addWidget(networkdisplay,2);

  emit finishedModule(2+1);

  /////////////////////////////////////////////////////////////////
  //Now we can Create and setup the different jahshaka modules

  ////////////////////////////////////////////////////////////////
  //we set up the animation module here

  emit creatingModule("Animation");

  AnimationModule = new QPushButton( moduleselection, "Animation" );
  AnimationModule->setGeometry( QRect( 10, 50, 111, 30 ) );
  AnimationModule->setText( tr( "Animation"  ) );
  connect( AnimationModule, SIGNAL(clicked()), SLOT(changeModeAnimation()) );  //was Effects

  animationdisplay = new QHBox (WorldLayout, "animation");
  CreateAnimationModule(animationdisplay);
  WorldLayout->addWidget(animationdisplay,3);

  emit finishedModule(3+1);

  /////////////////////////////////////////////////////////////////
  //we set up the Effects module here

  emit creatingModule("Effects");

  EffectsModule = new QPushButton( moduleselection, "Effects" );   	
  EffectsModule->setGeometry( QRect( 10, 83, 111, 30 ) );
  EffectsModule->setText( tr( "Effects"  ) );
  connect( EffectsModule,  SIGNAL(clicked()), SLOT(changeModeEffects())  );
  	
  effectsdisplay = new QHBox (WorldLayout, "effects");
  CreateEffectsModule(effectsdisplay);                              	
  WorldLayout->addWidget(effectsdisplay,4);
  emit finishedModule(4+1);

  /////////////////////////////////////////////////////////////////
  //we set up the editing module here

  emit creatingModule("Editing");

  EditingModule = new QPushButton( moduleselection, "Editing" );
  EditingModule->setGeometry( QRect( 10, 116, 111, 30 ) );
  EditingModule->setText( tr( "Editing"  ) );
  connect( EditingModule,  SIGNAL(clicked()), SLOT(changeModeEditing())  );

  editingdisplay = new QHBox (WorldLayout, "edit");
  CreateEditingModule(editingdisplay);
  WorldLayout->addWidget(editingdisplay,5);

  emit finishedModule(5+1);

  /////////////////////////////////////////////////////////////////
  //we set up the paint module here

  emit creatingModule("Paint");

  PaintModule = new QPushButton( moduleselection, "paint" );
  PaintModule->setGeometry( QRect( 10, 149, 111, 30 ) );
  PaintModule->setText( tr( "Paint"  ) );
  connect( PaintModule,  SIGNAL(clicked()), SLOT(changeModePaint())  );

  paintdisplay = new QHBox (WorldLayout, "paint");
  CreatePaintModule(paintdisplay);
  WorldLayout->addWidget(paintdisplay,6);

  emit finishedModule(6+1);

  /////////////////////////////////////////////////////////////////
  //we set up the CG module here

  emit creatingModule("Text");

  TextModule = new QPushButton( moduleselection, "text" );
  TextModule->setGeometry( QRect( 10, 182, 111, 30 ) );
  TextModule->setText( tr( "Text CG"  ) );
  connect( TextModule,  SIGNAL(clicked()), SLOT(changeModeText())  );

  textdisplay = new QHBox (WorldLayout, "text");
  CreateTextModule(textdisplay);
  WorldLayout->addWidget(textdisplay,7);

  emit finishedModule(7+1);


  /////////////////////////////////////////////////////////////////
  //these modules are in the desktop menu

  /////////////////////////////////////////////////////////////////
  //we set up the colorcorrection module here

  emit creatingModule("Colorize");

  colordisplay = new QHBox (WorldLayout, "colorize");
  CreateColorModule(colordisplay);
  WorldLayout->addWidget(colordisplay,10);

  emit finishedModule(8+1);


  /////////////////////////////////////////////////////////////////
  //Raise and activate the default modules

  //set the main module to desktop
  module = DESKTOP;
  changeModeDesktop();

  //clear out paint... we could also do this the firsttimepaint is called
  paint->clear();

  //update all modules with renderquality now that they have been created
  //maybe we can change globalclipnumber with a global prefs struct
  setModulesRenderQuality();

}

void JahControl::resetGlobalClipNumber() {

  debug(">>resetting global clipnumber");
  globalclipnumber=0;


}


//this is to pass data down to the modules
void JahControl::spaceballmoved(double x, double y, double z, double a, double b , double c) {

  //switch on module for the return    are we using mode or module?
  switch(module) {

    case ANIMATION : { animation->spaceEvent  ( x,y,z,a,b,c ); break; }
    case PLAYER    : {  break; }
    case DESKTOP   : {  break; }
    case PAINT     : {  break; }
    case EFFECTS   : {  break; }
    case TEXTCG    : {  break; }
    case COLORIZE  : {  break; }
    case EDITING   : {  break; }

    default        : {  break; }

  }

}

JahControl::~JahControl() {

  ////////////////////////////////////////////////////////
  //Jahshaka Destructor
  //cleans up as we leave and releases memory...

  if (JAHDEBUG) debug(">>Closing out the core module");

  delete thedesktop;   if (JAHDEBUG) debug(">> Closed thedesktop");
  delete animation;    if (JAHDEBUG) debug(">> Closed animation");
  delete effects;      if (JAHDEBUG) debug(">> Closed effects");

  delete editing;      if (JAHDEBUG) debug(">> Closed editing");
  delete text;         if (JAHDEBUG) debug(">> Closed text");
  delete colorize;     if (JAHDEBUG) debug(">> Closed colorize");
  delete vidplayer;    if (JAHDEBUG) debug(">> Closed vidplayer");
  delete jahnetwork;   if (JAHDEBUG) debug(">> Closed jahnetwork");
  delete paint;        if (JAHDEBUG) debug(">> Closed paint");

}


////////////////////////////////////////////////////////
//Jahshaka Prefernces Routines
//need to break these out into a separate file...


void JahControl::setStoragePath(void) {

    int status = QMessageBox::critical( 0, "Setting the Storage Path",
                                           "You are about to change the Storage Path!\n\n"
                                           "do so with caution, and only if you have\n"
                                           "moved your media folder!\n\n"
                                           "You will need to clear your desktop first as well!\n\n"
                                           "You will also need to restart for changes to take effect.\n\n"
                                           "Do you still want to Proceed?\n",
                                           QMessageBox::Yes | QMessageBox::Default,
                                           QMessageBox::No,
                                           QMessageBox::Cancel | QMessageBox::Escape );


    switch (status) {

     case QMessageBox::Yes    : { debug("Hit Yes");

                                  QString thepath = StorageBox->text();

                                  if (JAHDEBUG) debug(">>setting the media path to"+thepath);

                                  QSettings settings;
                                  settings.setPath( "jahshaka.com", "jahshaka" );
                                  settings.writeEntry( "/recent files/1", thepath );
                                  JahBasePath = thepath;

                                  break; }

     case QMessageBox::No     : { debug("Hit No"); break; }

     case QMessageBox::Cancel : { debug("Hit Cancel"); break; }

     default :                  { break; }

   }

}

void JahControl::setGlobalRenderQuality(void) {

    //read the slider setting
    int quality = renderqualityslider->value();

    if ( quality<0) { jahrenderquality=2; }
    else if ( quality>2) { jahrenderquality=2; }
    else { jahrenderquality = quality; }


    QString renqual; renqual.setNum(jahrenderquality);

    //update settings
    QSettings settings;
    settings.setPath( "jahshaka.com", "jahshaka" );
    settings.writeEntry( "/recent files/2", renqual );

    //update all the modules now
    setModulesRenderQuality();


}

//change the name here to set stored settings for prefs
void JahControl::setModulesRenderQuality(void) {

  //update all the modules with render quality settings
  animation->setRenderQuality(jahrenderquality);
  effects->setRenderQuality(jahrenderquality);
  editing->setRenderQuality(jahrenderquality);
  text->setRenderQuality(jahrenderquality);
  colorize->setRenderQuality(jahrenderquality);
  paint->setRenderQuality(jahrenderquality);

}

//change the name here to get stored settings for prefs
void JahControl::getGlobalRenderQuality(void) {

    bool itworked;
    int quality;
    QString qualitypref="2"; //default value incase of error

    //read the quality preferences
    QSettings settings;
    settings.setPath( "jahshaka.com", "jahshaka" );

    qualitypref   = settings.readEntry( "/recent files/2", qualitypref, &itworked );  //uses JahBasePath if its not alredy set
    if (!itworked) { quality=2; } else { quality = qualitypref.toInt(); }
    jahrenderquality = quality;

    //now we check for remember prefs
    bool remPrefs = false;
    remPrefs   = settings.readBoolEntry( "/recent files/3", remPrefs, &itworked );  //uses JahBasePath if its not alredy set
    if (!itworked) { rememberprefs=false; } else { rememberprefs = remPrefs; }

}


void JahControl::setGlobalPositionSize(void) {

 rememberprefs = PositionPrefs->isChecked();

}










