/*******************************************************************************
**
** The header file for the Jahshaka Jahcontrol module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef JAHCONTROL_H
#define JAHCONTROL_H

#include <stdlib.h>
#include <qsettings.h>
#include "qlineedit.h"

#include <sqlite.h>

#include "interfaceheaders.h"

#include <effect.h>
#include "paint.h"
#include "desktop.h"
#include "character.h"
#include "MainWindow.h"      // the player
#include "dialogs.h"
#include "glworld.h"
#include "anime.h"
#include "colorize.h"
#include "edit.h"
#include "networking.h"

#include "qcursor.h"

extern int xresolution;
extern int yresolution;
extern bool JAHDEBUG;

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QFrame;
class QPushButton;
class QTextView;


enum Module { DESKTOP, PLAYER, NETWORK, ANIMATION, EFFECTS, EDITING, TEXTCG, PAINT, COLORIZE };



class JahControl : public QObject {
    Q_OBJECT

public:

  JahControl( QWidget *parent=0, const char *name=0,
              QFrame  *mainleftmenu=0, QFrame *mainworld=0, QFrame *mainrightmenu=0,
              QFrame  *maincontroller=0,
              QFrame  *moduleselection=0, QFrame *modulesettings=0,QFrame *feedbackbar=0 );

  ~JahControl();

  ////////////////////////////////////////
  // main module classes
  JahDesktop*     thedesktop;
  JahNet*         jahnetwork;
  MainWindow*     vidplayer;

  GLAnime*        animation;
  GLEffect*       effects;
  GLEdit*         editing;
  GLChar*         text;
  GLPaint*        paint;

  GLColorize*     colorize;


  // the global clipnumber is used by all modules
  int globalclipnumber;

  //the render quality stored in prefs
  int jahrenderquality;

  //event driven routines here
  void keyPressEvent( QKeyEvent *k ) { debug("key event jahcreate"); }
  void spaceballmoved(double x, double y, double z, double a, double b , double c);

  //style prefs
  bool getPrefs(void) { return rememberprefs; }


signals:
  void creatingModule(const QString &);
  void finishedModule(int);
  void emitStyleColor(QPalette);

public slots:

  void switchMode(void);
  void switchDesktop(void);
  void GetDesktop();
  void ReturnDesktop();

  void addImageToDesktop( videoData );
  void GrabClipEffects();
  void GrabClipEdit();
  void GrabClipAnim();
  void GrabClipAnimKey();
  void GrabClipCG();
  void GrabClipColorize();
  void GrabClipPaint();
  void GrabClipNetwork();
  void printnameClip(QString item);
  void resetGlobalClipNumber();

private slots:

  //change mode for desktop and subsections
  void changeModeDesktop();

  void changeModePlayer();
  void displayClipInPlayer(videoData thedata ); //added for siggraph demo

  void changeModeNetwork();
  void changeModePref();
  void changeModeColorize();

  //change mode for modules
  void changeModeEffects();
  void changeModeAnimation();
  void changeModeEditing();
  void changeModePaint();
  void changeModeText();

  //jahroutine routines
  void checkClearPlayer();
  void checkDesktopButton();
  void stopAnimation();
  void nameClip();

  //change interface look and fell
  void stylePlatinum();
  void styleWindows();
  void styleSgi();
  void styleMotif();
  void styleCde();

  void styleGrey();
  void styleBlue();
  void styleGreen();
  void styleWhite();

  //store preferences
  void setStoragePath();

  void setGlobalRenderQuality(void);
  void getGlobalRenderQuality(void);
  void setModulesRenderQuality(void);

  void setGlobalPositionSize(void);


protected:

  void CreateDesktopModule(QHBox* frame);
  void CreateDesktopMiddleFrame(QHBox*);
  void CreateDesktopFeedbackBox(QFrame*);
  void CreateDesktopOptions(QHBox*);
  void CreateDesktopPreferences(QHBox*);

  void CreateAnimationModule(QHBox* frame);
  void CreateEffectsModule(QHBox* frame);
  void CreateEditingModule(QHBox* frame);
  void CreatePaintModule(QHBox* frame);
  void CreateTextModule(QHBox* frame);

  void CreateColorModule(QHBox* frame);
  //void CreatePlayerModule(QHBox* frame);
  void CreatePlayerModule(QFrame* frame);
  void CreateNetworkModule(QHBox* frame);



private:

  //////////////////////////////////////////
  // All variable Initializations here
  bool desktopButtonStatus;
  QFont          appFont;
  Module         module;

  //style prefs
  bool rememberprefs;

  // Main Interface layout variables
  QHBoxLayout* mainworldLayout;       QWidgetStack*  WorldLayout;

  QHBoxLayout * mainworldLeftLayout;  QWidgetStack * MainLeftControler;
  QHBoxLayout * mainworldRightLayout; QWidgetStack * MainRightControler;

  //main window layout variables
  QHBoxLayout*   maincontrollerLayout;
  QWidgetStack*  MainControler;


  //the main desktop module
  //desktop controllers
  QHBox*         MainDesktopControler;
  QHBox*         MainDesktopLeftControler, *MainDesktopRightControler;
  QHBox*         desktopdisplay;
  QHBox*         optionals_desktop;
  QPushButton*   Desktop;

  //mandatory modules in the desktop
  //network controllers
  QHBox*         networkdisplay;
  QHBox*         optionals_network;
	QHBox*         MainNetworkControler;
  QPushButton*   Network;

  //player controllers
	//QHBox*         playerdisplay;
	QFrame*         playerdisplay;
  QHBox*         optionals_player;
  QHBox*         MainPlayerControler;

  //main modules located in the left option panel
  //animation controllers
	QHBox*         MainAnimationControler;
  QHBox*         MainAnimationLeftControler, *MainAnimationRightControler;
  QHBox*         animationdisplay;
  QHBox*         optionals_animation;
  QPushButton*   AnimationModule;

  //effects controllers
	QHBox*         MainEffectsControler;
  QHBox*         MainEffectsLeftControler, *MainEffectsRightControler;
  QHBox*         effectsdisplay;
  QHBox*         optionals_effects;
  QPushButton*   EffectsModule;

  //editing controllers
	QHBox*         MainEditingControler;
  QHBox*         MainEditingLeftControler, *MainEditingRightControler;
  QHBox*         editingdisplay;
  QHBox*         optionals_editing;
  QPushButton*   EditingModule;

  //The Text Module
	QHBox*         MainTextControler;
  QHBox*         MainTextLeftControler, *MainTextRightControler;
  QHBox*         textdisplay;
  QHBox*         optionals_text;
	QPushButton*   TextModule;

  //The Paint Module
	QHBox*         MainPaintControler;
  QHBox*         MainPaintLeftControler, *MainPaintRightControler;
  QHBox*         paintdisplay;
  QHBox*         optionals_paint;
	QPushButton*   PaintModule;

  //sub modules located on the desktop
  //The Paint Module
	QHBox*         MainColorControler;
  QHBox*         MainColorLeftControler, *MainColorRightControler;
  QHBox*         colordisplay;
  QHBox*         optionals_color;
  QPushButton*   Colors;


  // for the desktop controlbar interface
  QWidget* page;
  QFrame*      modeselectframe;
  QPushButton* modeselectbutton;
  QFrame*      buttonframe;
  QPushButton* DesktopName;
  QPushButton* DesktopLoad;
  QPushButton* DesktopPlay;
  QPushButton* DeleteButton;
  QFrame*      desktopselectframe;
  QPushButton* desktopselectbutton;
  QHBoxLayout* pageLayout;

  //for the middleframe
  QFrame*        MiddleFrame;
  FancyPopup*    popup2;
  QPushButton*   clipname;
  QPushButton*   clipplay;
  QPushButton*   clipdelete;
  QPushButton*   clipload;
  QPushButton*   SwitchDesk;



//these need to be sorted and remove the inactive ones

  QFrame*         Display;
  QFrame*        FrameMain;

  QHBox * desktopcontrolerbar;

  QTextView*     TextView1_2;
  QPushButton*   GetButton;
  QPushButton*   SwitchButton;
  QPushButton*   ReturnButton;

  QFrame*        FrameSubmenu;
  QWidgetStack*  optionals;
  QVBox*         optionals_prefs;


  // QButtonGroup*  bsub;
  QHBox*         desktopframe;
  QFrame*        desktopframe1;
  QFrame*        desktopframe2;
  QFrame*        desktopframe3;
  QFrame*        desktopframe4;

  QPushButton*   SubLoad;
  QPushButton*   SubSave;
  QPushButton*   SubPref;
  QPushButton*   SubImport;
  QPushButton*   SubExport;
  QPushButton*   SubComp;

  QPushButton*   SubColor;
  //QPushButton*   SubPaint;
  QPushButton*   SubVideo;
  QPushButton*   SubLoadImg;
  QPushButton*   SubSaveImg;
  QPushButton*   SubQuit;

  QHBox* preferencesframe;
  QFrame* prefframe1;          QFrame* prefframe2;    QFrame* prefframe3;

  QPushButton* Pref1;          QPushButton* Pref2;
  QPushButton* Pref3;          QPushButton* Pref4;

  QPushButton* PrefCol1;

  QPushButton* PrefCol2;
  QSlider *renderqualityslider;

  QPushButton* PrefCol3;
  QPushButton* PrefCol4;

  QPushButton* PrefExit;

  //prefernces
  //QTextView * StorageBox;
  QLineEdit * StorageBox;
  QPushButton * StorageButton;
  QCheckBox *  PositionPrefs;



};


#endif




























