/*******************************************************************************
**
** The source file for the Jahshaka Jahstyle module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "jahshaka.h"

void Jah::setupInterface( QWidget* parent, const char* name )
{

  this->setMinimumSize( QSize( 1026, 768 ) );
  this->setMaximumSize( QSize( 1270, 970 ) );
  this->setSizeIncrement( QSize( 1, 1 ) );

  //should be based on screen resolution/gcard texturemem
  //and saved as a preference along with media dir and colorscheme
  this->setBaseSize( QSize( 1024, 786 ) );
  // this->setSizePolicy(  QSizePolicy::Maximum );
  //this->setSizePolicy(  QSizePolicy::Maximum, QSizePolicy::Maximum );
  this->setPaletteBackgroundColor( QColor( 35, 35, 35 ) );

  jahshakaLayout = new QGridLayout( this, 1, 1, 0, 0, "jahshakaLayout");

  mainleftmenu = new QFrame( this, "mainleftmenu" );
  mainleftmenu->setMinimumSize( QSize( 30, 0 ) );
  mainleftmenu->setMaximumSize( QSize( 30, 1000 ) );
  mainleftmenu->setPaletteBackgroundColor( QColor( 100, 100, 100 ) );
  mainleftmenu->setFrameShape( QFrame::NoFrame );
  mainleftmenu->setFrameShadow( QFrame::Plain );

  jahshakaLayout->addWidget( mainleftmenu, 0, 0 );

  mainworld = new QFrame( this, "mainworld" );
  mainworld->setMinimumSize( QSize( 964, 486 ) );
  mainworld->setMaximumSize( QSize( 1964, 1486 ) );
  mainworld->setPaletteBackgroundColor( QColor( 35, 35, 35 ) );
  mainworld->setCursor( QCursor( 2 ) );
  mainworld->setFrameShape( QFrame::NoFrame );
  mainworld->setFrameShadow( QFrame::Plain );

  jahshakaLayout->addWidget( mainworld, 0, 1 );

  mainrightmenu = new QFrame( this, "mainrightmenu" );
  mainrightmenu->setMinimumSize( QSize( 30, 0 ) );
  mainrightmenu->setMaximumSize( QSize( 30, 1000 ) );
  mainrightmenu->setPaletteBackgroundColor( QColor( 100, 100, 100 ) );
  mainrightmenu->setFrameShape( QFrame::NoFrame );
  mainrightmenu->setFrameShadow( QFrame::Plain );

  jahshakaLayout->addWidget( mainrightmenu, 0, 2 );

  maincontroller = new QFrame( this, "maincontroller" );
  maincontroller->setMinimumSize( QSize( 0, 35 ) );
  maincontroller->setMaximumSize( QSize( 32767, 35 ) );
  maincontroller->setPaletteBackgroundColor( QColor( 100, 100, 100 ) );
  maincontroller->setFrameShape( QFrame::StyledPanel );
  maincontroller->setFrameShadow( QFrame::Raised );

  jahshakaLayout->addMultiCellWidget( maincontroller, 1, 1, 0, 2 );

  moduleinterface = new QFrame( this, "moduleinterface" );
  moduleinterface->setMinimumSize( QSize( 0, 235 ) );  ////////was 222 now is 209
  moduleinterface->setMaximumSize( QSize( 32767, 209 ) );
  moduleinterface->setFrameShape( QFrame::Box );
  moduleinterface->setFrameShadow( QFrame::Plain );
  moduleinterfaceLayout = new QHBoxLayout( moduleinterface, 0, 0, "moduleinterfaceLayout");

  moduleselection = new QFrame( moduleinterface, "moduleselection" );
  moduleselection->setMinimumSize( QSize( 125, 0 ) );
  moduleselection->setMaximumSize( QSize( 125, 32767 ) );
  moduleselection->setFrameShape( QFrame::NoFrame );
  moduleselection->setFrameShadow( QFrame::Raised );
  moduleinterfaceLayout->addWidget( moduleselection );

  modules = new QFrame( moduleinterface, "modules" );
  modules->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, modules->sizePolicy().hasHeightForWidth() ) );
  modules->setBackgroundOrigin( QFrame::WidgetOrigin );
  modules->setFrameShape( QFrame::NoFrame );
  modules->setFrameShadow( QFrame::Raised );
  modulesLayout = new QGridLayout( modules, 1, 1, 0, 0, "modulesLayout");

  feedbackbar = new QFrame( modules, "feedbackbar" );
  feedbackbar->setMinimumSize( QSize( 0, 40 ) );
  feedbackbar->setMaximumSize( QSize( 32767, 40 ) );
  feedbackbar->setFrameShape( QFrame::NoFrame );
  feedbackbar->setFrameShadow( QFrame::Plain );
  feedbackbar->setLineWidth( 0 );

  modulesLayout->addMultiCellWidget( feedbackbar, 0, 0, 0, 1 );

  modulesettings = new QFrame( modules, "modulesettings" );
  modulesettings->setMinimumSize( QSize( 870, 0 ) );
  modulesettings->setFrameShape( QFrame::StyledPanel );
  modulesettings->setFrameShadow( QFrame::Raised );
  modulesettings->setMargin( 0 );
  //FrameSubmenu->setGeometry( QRect( 130, 40, 875, 145 ) );  //centered within 1280 horiz size


  modulesLayout->addWidget( modulesettings, 1, 0 );
  QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
  modulesLayout->addItem( spacer, 1, 1 );
  moduleinterfaceLayout->addWidget( modules );

  jahshakaLayout->addMultiCellWidget( moduleinterface, 2, 2, 0, 2 );

  ///////////////////////////////////////////////////////////////
  //set the caption
  this->setCaption( tr( "JahShaka", "Realtime Editing and Effects" ) );

  //resize the app here based on prefs either small or large
  //this->resize( QSize(1270, 970).expandedTo(minimumSizeHint()) );

  int jw,jh;

  if ( QApplication::desktop()->width()  >= 1280 ) jw = 1270; else jw = 1026;
  if ( QApplication::desktop()->height() >= 970 ) jh = 970;   else jh = 768;

  this->resize( QSize(jw, jh).expandedTo(minimumSizeHint()) );


}

void Jah::setStyleColor(QPalette pal) {

  this->setPalette( pal, TRUE );
  moduleselection->setPalette( pal, TRUE );
  modulesettings->setPalette( pal, TRUE );
  feedbackbar->setPalette( pal, TRUE );

  //Need to get the palette color and lighten for the rest of the interface...
  //also for the buttons

  QColor lighten = pal.color( QPalette::Active, QColorGroup::Background    );


  int red = lighten.red()+20;
  int green = lighten.green()+20;
  int blue = lighten.blue()+20;

  QPalette op( QColor( red, green, blue ) );

  maincontroller->setPalette( op, TRUE );
  mainleftmenu->setPalette( op, TRUE );
  mainrightmenu->setPalette( op, TRUE );

}


void Jah::setupMyCustomColors(QWidget * parent) {

  qApp->setStyle( new QSGIStyle );      //qApp->setFont( appFont, TRUE );
  QPalette p( QColor( 100, 100, 100 ) );
  qApp->setPalette( p, TRUE );

  QFont::setDefaultFont(QFont("Helvetica", 10, QFont::Bold, false));

}







