/*******************************************************************************
**
** The source file for the Jahshaka desktop module file
** The Jahshaka Project
** Copyright (C) 2000-2003 Visual Media, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

////////////////////////////////////////////////////////////////
// This is the file that sets up all the individual modules
// and connects them to the desktop

#include "jahcreate.h"

////////////////////////////////////////////////////////////////
//Code to set up the Animation module


void JahControl::CreateDesktopModule(QHBox* frame) {

  MainDesktopControler    = new QHBox(MainControler);
  MainControler->addWidget(MainDesktopControler,0); //desktop

  MainDesktopLeftControler  = new QHBox(MainLeftControler);
  MainLeftControler->addWidget(MainDesktopLeftControler,0); //animation

  MainDesktopRightControler  = new QHBox(MainRightControler);
  MainRightControler->addWidget(MainDesktopRightControler,0); //animation

  //moudle user controles
  optionals_desktop = new QVBox(optionals);
  optionals->addWidget(optionals_desktop,0);

  thedesktop = new JahDesktop( frame, "desktop", &globalclipnumber );

  ///////////////////////////////////////////////////////////////
  //now we set up the desktop interface
  //all other modules do this themselves ie effects,animation
  //but since desktop controls all of them its easier to do it here

  // setup the desktop middle frame first ie load, delete, play, etc
  CreateDesktopMiddleFrame(MainDesktopControler);

  //Set up Desktop Interface Options and buttons here
  desktopframe  = new QHBox(optionals_desktop);
  CreateDesktopOptions(desktopframe);

  // Create Desktop Interface Preferences Here
  optionals_prefs   = new QVBox(optionals);
  optionals->addWidget(optionals_prefs,8);   //hmmm why is this 8?  maybe make it 2?

  preferencesframe  = new QHBox(optionals_prefs);
  CreateDesktopPreferences(preferencesframe);

}

//void JahControl::CreatePlayerModule(QHBox* frame) {
void JahControl::CreatePlayerModule(QFrame* frame) {

  MainPlayerControler  = new QHBox(MainControler);
  MainControler->addWidget(MainPlayerControler,1);

  optionals_player   = new QHBox(optionals);
  optionals->addWidget(optionals_player,1);

  vidplayer = new MainWindow( frame, "playerlayout", optionals_player, MainPlayerControler);    // was Scribble is GLPaint
  //vidplayer->setFixedSize( QSize( 15, 486 ) );     //dont know why this works yet

}

void JahControl::CreateNetworkModule(QHBox* frame) {

	//this module only uses the bottom half of the desktop for now...

  MainNetworkControler  = new QHBox(MainControler);
  MainControler->addWidget(MainNetworkControler,2); //network

  optionals_network = new QHBox(optionals);
  optionals->addWidget(optionals_network,2);

  jahnetwork = new JahNet( frame, "networklayout", optionals_network,
                           MainNetworkControler, &globalclipnumber);

  connect ( jahnetwork,    SIGNAL(GrabClip()), SLOT( GrabClipNetwork()) );
  connect ( jahnetwork,    SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );

}

void JahControl::CreateAnimationModule(QHBox* frame) {

  MainAnimationControler  = new QHBox(MainControler);
  MainControler->addWidget(MainAnimationControler,3); //animation

  MainAnimationLeftControler  = new QHBox(MainLeftControler);
  MainLeftControler->addWidget(MainAnimationLeftControler,3); //animation

  MainAnimationRightControler  = new QHBox(MainRightControler);
  MainRightControler->addWidget(MainAnimationRightControler,3); //animation

  //set up module control space
  optionals_animation = new QHBox(optionals);
  optionals->addWidget(optionals_animation,3);


  animation = new GLAnime( frame, "Animation" , optionals_animation,
                           MainAnimationControler,
                           MainAnimationLeftControler, MainAnimationRightControler,
                           &globalclipnumber  );

    //animation->setFixedSize( QSize( 1210, 700 ) );

	//connect module to desktop framework
  connect ( animation,   SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
  connect ( animation,   SIGNAL(GrabClip()),    SLOT( GrabClipAnim()) );
  connect ( animation,   SIGNAL(GrabClipKey()), SLOT( GrabClipAnimKey()) );

  //added for siggraph demo
  connect ( animation,   SIGNAL(updatePlayer(videoData)), SLOT( displayClipInPlayer(videoData)) );


}

////////////////////////////////////////////////////////////////
//Code to set up the Effects module

void JahControl::CreateEffectsModule(QHBox* frame) {

  MainEffectsControler  = new QHBox(MainControler);
  MainControler->addWidget(MainEffectsControler,4); //animation

  MainEffectsLeftControler  = new QHBox(MainLeftControler);
  MainLeftControler->addWidget(MainEffectsLeftControler,4); //animation

  MainEffectsRightControler  = new QHBox(MainRightControler);
  MainRightControler->addWidget(MainEffectsRightControler,4); //animation

  //set up module control space
  optionals_effects = new QHBox(optionals);
  optionals->addWidget(optionals_effects,4);

  effects = new GLEffect( frame, "Effects" , optionals_effects,
                           MainEffectsControler,
                           MainEffectsLeftControler, MainEffectsRightControler,
                           &globalclipnumber  );

	//connect module to desktop framework
    connect ( effects,     SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
    connect ( effects,     SIGNAL(GrabClip()), SLOT( GrabClipEffects()) );

  //added for siggraph demo
  connect ( effects,   SIGNAL(updatePlayer(videoData)), SLOT( displayClipInPlayer(videoData)) );

}


void JahControl::CreateEditingModule(QHBox* frame) {

  MainEditingControler  = new QHBox(MainControler);
  MainControler->addWidget(MainEditingControler,5); //animation

  MainEditingLeftControler  = new QHBox(MainLeftControler);
  MainLeftControler->addWidget(MainEditingLeftControler,5); //animation

  MainEditingRightControler  = new QHBox(MainRightControler);
  MainRightControler->addWidget(MainEditingRightControler,5); //animation

  //set up module control space
  optionals_editing = new QHBox(optionals);
  optionals->addWidget(optionals_editing,5);

    editing = new GLEdit( frame, "Editing" , optionals_editing,
                             MainEditingControler,
                             MainEditingLeftControler, MainEditingRightControler,
                             &globalclipnumber  );

	//connect module to desktop framework
    connect ( editing,     SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
    connect ( editing,     SIGNAL(GrabClip()), SLOT( GrabClipEdit()) );       //not yet implemented

  //added for siggraph demo
  connect ( editing,   SIGNAL(updatePlayer(videoData)), SLOT( displayClipInPlayer(videoData)) );

}

void JahControl::CreatePaintModule(QHBox* frame) {

  MainPaintControler  = new QHBox(MainControler);
  MainControler->addWidget(MainPaintControler,6); //animation

  MainPaintLeftControler  = new QHBox(MainLeftControler);
  MainLeftControler->addWidget(MainPaintLeftControler,6); //animation

  MainPaintRightControler  = new QHBox(MainRightControler);
  MainRightControler->addWidget(MainPaintRightControler,6); //animation

  //set up module control space
  optionals_paint = new QHBox(optionals);
  optionals->addWidget(optionals_paint,6);

    paint = new GLPaint( frame, "Paint" , optionals_paint,
                             MainPaintControler,
                             MainPaintLeftControler, MainPaintRightControler,
                             &globalclipnumber  );

	 paint->setFixedSize( QSize( 720, 486 ) );

	//connect module to desktop framework
    connect ( paint,  SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
    connect ( paint,  SIGNAL(GrabClip()), SLOT( GrabClipPaint()) );

  //added for siggraph demo
  connect ( paint,   SIGNAL(updatePlayer(videoData)), SLOT( displayClipInPlayer(videoData)) );

}

void JahControl::CreateTextModule(QHBox* frame) {

  MainTextControler  = new QHBox(MainControler);
  MainControler->addWidget(MainTextControler,7); //animation

  MainTextLeftControler  = new QHBox(MainLeftControler);
  MainLeftControler->addWidget(MainTextLeftControler,7); //animation

  MainTextRightControler  = new QHBox(MainRightControler);
  MainRightControler->addWidget(MainTextRightControler,7); //animation

  //set up module control space
  optionals_text = new QHBox(optionals);
  optionals->addWidget(optionals_text,7);

  text = new GLChar( frame, "Text" , optionals_text,
                     MainTextControler,
                     MainTextLeftControler, MainTextRightControler,
                     &globalclipnumber  );

	//connect module to desktop framework
  connect ( text,     SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
  connect ( text,     SIGNAL(GrabClip()), SLOT( GrabClipCG()) );

  //added for siggraph demo
  connect ( text,   SIGNAL(updatePlayer(videoData)), SLOT( displayClipInPlayer(videoData)) );

}


void JahControl::CreateColorModule(QHBox* frame) {

  MainColorControler  = new QHBox(MainControler);
  MainControler->addWidget(MainColorControler,10); //animation

  MainColorLeftControler  = new QHBox(MainLeftControler);
  MainLeftControler->addWidget(MainColorLeftControler,10); //animation

  MainColorRightControler  = new QHBox(MainRightControler);
  MainRightControler->addWidget(MainColorRightControler,10); //animation

  //set up module control space
  optionals_color = new QHBox(optionals);
  optionals->addWidget(optionals_color,10);

  colorize = new GLColorize( frame, "Color" , optionals_color,
                             MainColorControler,
                             MainColorLeftControler, MainColorRightControler,
                             &globalclipnumber  );

  //colorize->setFixedSize( QSize( 1210, 700 ) );

	//connect module to desktop framework
  connect ( colorize,    SIGNAL(updateDesktop(videoData)), SLOT( addImageToDesktop(videoData)) );
  connect ( colorize,    SIGNAL(GrabClip()), SLOT( GrabClipColorize()) );

  //added for siggraph demo
  connect ( colorize,   SIGNAL(updatePlayer(videoData)), SLOT( displayClipInPlayer(videoData)) );

}

