/*******************************************************************************
**
** The source file for the Jahshaka Jah module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/
#include "jahshaka.h"

#include "tga.h"

///////////////////////////////////////////////////////////////////
// This is the first main class, that calls the startup screen
// and build the main jahshaka desktop widget defined in jahcontrol.cpp
// which sets up the interface and builds all the other modules

Jah::Jah( QWidget *parent, const char *name )
        : QWidget( parent, name ) { if (JAHDEBUG) debug("Loading Jahshaka");


  //////////////////////////////////////////////////////////////
  // find out directory information and set relative paths
	// good to set this up from now so we can be flexable here later

  //this is now used to load in preferences on startup
	if (!setBasePath()) { exitError(); }

  //////////////////////////////////////////////////////////////
  // add targa image support
  QImageIO::defineIOHandler ( "TGA","^TGA[0-9][0-9][a-z]",0, &kimgio_tga_read, &kimgio_tga_write );

  //////////////////////////////////////////////////////////////
  //this sets up the look and feel


  setupMyCustomColors(this);

  //////////////////////////////////////////////////////////////
  //this calls the startupscreen where we can add startup options
  //such as projects, username, etc

  thestartupscreen = new startup( "jahstartup" );
  thestartupscreen->setActiveWindow();

  connect( this,  SIGNAL(sendchangeModeText(const QString &)), thestartupscreen, SLOT(setProgressText( const QString & ))  );
  connect( this,  SIGNAL(sendsetProgressValue(int)), thestartupscreen, SLOT(setProgressValue( int ))  );


  //////////////////////////////////////////////////////////////
  //check the opengl extensions
  checkJahExtensions();

  //////////////////////////////////////////////////////////////
  //this calls our main Jahshaka constructor
  setupInterface(this,"interface");

  jahapp = new JahControl(  this , "jahshaka",
                            mainleftmenu, mainworld, mainrightmenu,
                            maincontroller, moduleselection, modulesettings, feedbackbar );


  //if we are using stored settings use them now
  if (jahapp->getPrefs()) { restorePreferences(); }
  else
  {
  //center application in screen on startup
  QRect qRect(QApplication::desktop()->screenGeometry());
  int iXpos=qRect.width()/2-this->width()/2;
  int iYpos=qRect.height()/2-this->height()/2;
  this->move(iXpos,iYpos);
  }

  //close out the splash screen now that all modules are loaded
  emit sendsetProgressValue(100);
  emit sendchangeModeText("Finished");

  //pause a sec and then close
#ifndef WIN32
  sleep(1);
#endif

  thestartupscreen->close();



}

Jah::~Jah() {

  ////////////////////////////////////////////////////////
  //Jahshaka Destructor
  //cleans up as we leave and releases memory...

  debug(">>Closing out the main module");

  //we store all preference data before exiting
  storeGlobalPositionSize();

  delete jahapp;

}


bool Jah::setBasePath(void)
{
  //////////////////////////////////////////////////////////
  //we always check for the base path first
  //makes sure jahshaka is installed correctly
  //and the copy protection was not cracked... heh :)
  QDir dir, temp;	
	temp=dir.current();

	QString thepath =  temp.path();
	QString thedir  =  temp.dirName();    //problem with this variable

	QString finalpath; bool foundpath=true;

	//make sure we are in the right directory
	//since some compilers switch this around

	if (thedir=="jahshaka") { finalpath = thepath; }
	else if (thedir="jah")  { int jitlen = thepath.length();
                            finalpath = thepath.left(jitlen - 4) ;
                          } //not complete code  finalpath = thepath - "/jah";

  else                    { foundpath = false; }

  if (foundpath) {
	  finalpath += "/";
	  JahBasePath = finalpath;
	  if (JAHDEBUG) debug ("The JahBasePath is >>");
	  if (JAHDEBUG) debug (JahBasePath);
  }  else return false;

    QSettings settings;
    settings.setPath( "jahshaka.com", "jahshaka" );

bool firstime;

JahBasePath   = settings.readEntry( "/recent files/1", JahBasePath, &firstime );  //uses JahBasePath if its not alredy set

if (!firstime)
{
    debug(">>Creating preferences for the first time");
    foundpath = settings.writeEntry( "/recent files/1", JahBasePath );
    foundpath = settings.writeEntry( "/recent files/2", 2 );     //renderquality
    foundpath = settings.writeEntry( "/recent files/3", false );     //rememberprefs

    foundpath = settings.writeEntry( "/geometry/x", 0 );
    foundpath = settings.writeEntry( "/geometry/y", 0 );
    foundpath = settings.writeEntry( "/geometry/width", 0 );
    foundpath = settings.writeEntry( "/geometry/height", 0 );

}

  return foundpath;
}

void Jah::storeGlobalPositionSize(void) {

  debug(">>>storing the preferences");

  //this routine stores all the interface settings
  //desktop position,size,color scheme,style
  //update settings  we can write bool, int and qstring only

  QSettings settings;
  settings.setPath( "jahshaka.com", "jahshaka" );


  int width=this->width();
  int height=this->height();
  int Xpos=this->x();
  int Ypos=this->y();
  //QStyle thestyle=this->style ();

  settings.writeEntry( "/recent files/3", jahapp->getPrefs() );
  //settings.writeEntry( "/recent files/4", rememberprefs );     // this is the color string
  //settings.writeEntry( "/recent files/5", rememberprefs );     // this is the style string
  settings.writeEntry( "/geometry/x", Xpos );
  settings.writeEntry( "/geometry/y", Ypos );
  settings.writeEntry( "/geometry/width", width );
  settings.writeEntry( "/geometry/height", height );


}

void Jah::restorePreferences(void) {

  if (JAHDEBUG) debug(">>>restoring the preferences");

  QSettings settings;
  settings.setPath( "jahshaka.com", "jahshaka" );

  //QStyle thestyle=this->style ();
  //settings.writeEntry( "/recent files/4", rememberprefs );     // this is the color string
  //settings.writeEntry( "/recent files/5", rememberprefs );     // this is the style string


  //now we check for remember prefs
  QRect qRect(QApplication::desktop()->screenGeometry());

  bool itworked;

  int xPrefs = qRect.width()/2-this->width()/2;
  xPrefs   = settings.readNumEntry( "/geometry/x", xPrefs, &itworked );  //uses JahBasePath if its not alredy set
  if (!itworked) { xPrefs = qRect.width()/2-this->width()/2; }

  int yPrefs = qRect.height()/2-this->height()/2;
  yPrefs   = settings.readNumEntry( "/geometry/y", yPrefs, &itworked );  //uses JahBasePath if its not alredy set
  if (!itworked) { yPrefs = qRect.height()/2-this->height()/2; }

  int wPrefs = 1026;
  wPrefs   = settings.readNumEntry( "/geometry/width", wPrefs, &itworked );  //uses JahBasePath if its not alredy set
  if (!itworked) { wPrefs = 1026; }

  int hPrefs = 768;
  hPrefs   = settings.readNumEntry( "/geometry/height", hPrefs, &itworked );  //uses JahBasePath if its not alredy set
  if (!itworked) { hPrefs = 768; }


  this->resize( QSize(wPrefs, hPrefs).expandedTo(minimumSizeHint()) );
  this->move(xPrefs,yPrefs);

}

void Jah::checkJahExtensions(void) {

  //get the gl extensions need to clean this up and put it into a routine
  vendor     = new QString (*thestartupscreen->vendor);
  renderer   = new QString (*thestartupscreen->renderer);
  version    = new QString (*thestartupscreen->version);
  extensions = new QString (*thestartupscreen->extensions);

  if (JAHDEBUG) {
    debug("Vendor    :"+*vendor);
    debug("Renderer  :"+*renderer);
    debug("Version   :"+*version);
    //debug("Extensions:"+*extensions);
  }

  ///////////////////////////////////////////////
  // Check for the OpenGL imaging extensions
  if (extensions->contains("ARB_imaging")) debug("We have hardware imaging support");
  if (extensions->contains("GL_ARB_vertex_program")) debug("We have hardware arb_vertex support");

}

void Jah::exitError(void) {

		//we could not find the default environment here
    //make this a popup window with a timer?
		debug ("Cannot find the basic jahshaka paths");
		debug ("Exiting jahshaka");
		debug ("please email your configuration to");
		debug ("jahshaka@users.sourceforge.net");
	  debug("-------------------------------------");
		exit(0);


}


///////////////////////////////////////////////////////
//commands to trap spaceball events and
//send them on to the jahapp


void Jah::customEvent(QCustomEvent *event) {

  switch (event->type()) {

    case 1001 : { getSpaceballData(event);   break; }
    case 1002 : { getSpaceballButton(event); break; }

    default:
        { qWarning("Unknown custom event type: %d", event->type()); }
    }
}

void Jah::getSpaceballData(QCustomEvent *event)
{
#if defined(Q_WS_X11)
  MagellanFloatEvent *s = (MagellanFloatEvent *) event->data();

  jahapp->spaceballmoved(s->MagellanData[ MagellanX ],s->MagellanData[ MagellanY ],
                         s->MagellanData[ MagellanZ ],
                         s->MagellanData[ MagellanA ],s->MagellanData[ MagellanB ],
                         s->MagellanData[ MagellanC ]);
#endif
}

void Jah::getSpaceballButton(QCustomEvent *event)
{
  int *s = (int *) event->data();
  QString theval; theval = theval.setNum(*s);
  debug("Spaceball button:"+theval);
}


/*
void JahControl::keyPressEvent( QKeyEvent *k )
{
   // switch ( tolower(k->ascii()) ) {
   //     case 'r':  debug("pressed r");       break;
   //     case ' ':  debug("pressed space");   break;
   //     case 'q':  debug("pressed q");       break; }
   debug("pressed key");

   //this->grabKeyboard();  needs this to work...

   switch ( module ) {
        case  DESKTOP    :  { debug("pressed key in desktop");         break; }
        case  PLAYER     :  { debug("pressed key in player");          break; }
        case  PAINT      :  { debug("pressed in paint");               break; }
        case  EFFECTS    :  { debug("pressed in effects");             break; }
        case  ANIMATION  :  { debug("pressed in animation");           break; }
        case  TEXTCG     :  { debug("pressed in text");                break; }

        default          :  { debug("unknown module in JahRoutines:keyPressEvent"); break; }
    }
}
  */


