/*******************************************************************************
**
** The header file for the Jahshaka Jah module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef JAH_H
#define JAH_H

#include "jahcreate.h"
#include "jahstartup.h"
#include <qstring.h>
#include <qdir.h>
#include <qsettings.h>
#include <stdlib.h>
#include <stdio.h>

//for spaceball under linux
#if defined(Q_WS_X11)
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <xdrvlib.h>
#endif


//from the new ui code
//#include "jahshakaui.h"
//#include "qobject.h"

extern QString JahBasePath;

//  The Main Application Constructor
class Jah : public QWidget {

    Q_OBJECT

public:
  Jah( QWidget *parent=0, const char *name=0 );
  ~Jah();

  bool setBasePath(void);
  void checkJahExtensions(void);
  void exitError(void);

  //for the spaceball event
  void customEvent(QCustomEvent *);
  void getSpaceballData(QCustomEvent *event);
  void getSpaceballButton(QCustomEvent *event);
  void keyPressEvent( QKeyEvent *k ) { debug("key event jahsaka"); }
  int moderator;

  //theme setting routines
  void setupMyCustomColors(QWidget * parent);



protected:
  //initailizes the main interface
  void setupInterface( QWidget* parent, const char* name );

  //the main interface frame variables
  QGridLayout* jahshakaLayout;
  QHBoxLayout* moduleinterfaceLayout;
  QGridLayout* modulesLayout;

  QFrame* mainleftmenu;
  QFrame* mainworld;
  QFrame* mainrightmenu;
  QFrame* maincontroller;
  QFrame* moduleinterface;
  QFrame* moduleselection;
  QFrame* modules;
  QFrame* feedbackbar;
  QFrame* modulesettings;
  QWidget * centralWidget;

public slots:

  void JahchangeModeText(const QString & text) { emit sendchangeModeText(text); };
  void JahsetProgressValue(int progress) { emit sendsetProgressValue(progress*10);  };
  void setStyleColor(QPalette pal);

  void storeGlobalPositionSize(void);
  void restorePreferences(void);

signals:

  void sendchangeModeText(const QString & text);
  void sendsetProgressValue(int progress);

private:
  //the startup screen
  startup    *thestartupscreen;
  QTimer     *timer;

  //this is the core jahshaka class
  JahControl *jahapp;

  //graphics card extension
  QString * vendor, * renderer, * version, * extensions;

};


#endif // JAH_H






