/*******************************************************************************
**
** The source file for the Jahshaka desktop module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "desktop.h"


JahDesktop::JahDesktop(QWidget* parent, const char* name, int * globalclipnumber, WFlags f) :
QWidget(parent,name,f) {

    canvas = new QCanvas(this); //canvas = new QCanvas(parent);

    QColor back;
    back.setHsv(35,35,35);
    canvas->setBackgroundColor(back);
    canvas->resize( 1270, 700 );

    editor = new FigureEditor(canvas,parent); //editor = new FigureEditor(canvas,this);

    editor->setHScrollBarMode( FigureEditor::AlwaysOff );
    editor->setVScrollBarMode( FigureEditor::AlwaysOff );

    connect ( editor, SIGNAL(updateClipDatabase(videoData)), this, SLOT( updateClipDatabase(videoData)) );

    QString stringval,debugmessage;
    clipnumber = globalclipnumber;

    InitializeDatabase();
    RebuildDesktop();

	if (JAHDEBUG) {

		debugmessage="clipnumber is now set to:" + stringval.setNum(*clipnumber);
		debug(debugmessage);

    }
}

JahDesktop::~JahDesktop() {

  ////////////////////////////////////////////////////////
  //JahDesktop Destructor
  //cleans up as we leave and releases memory...
	//should check to make sure database is open

  if (JAHDEBUG) { debug(">>> closing out database"); }

	CloseDatabase();

  if (JAHDEBUG) { debug(">>> Database Closed"); }
}

void JahDesktop::init() { }

bool JahDesktop::clear() {

  ///////////////////////////////////////////////////////
  //caution using this routine! It wipes the datbase...


  //first we find out how many entries are in the database
  QString data;

  int counter = GetDatabaseClipNumber();
  debug("Number of records in database:"+data.setNum(counter));


  //now we iterate through each one and erase the files
  //if they are in the renders directory
  //so we dont erase stuff we are not supposed to

  //we can also clear the desktop and wipe everything using the database...
  //good for corrupt databases

  videoData clip;
  bool EraseFile;

  for (int loop=1; loop<=counter; loop++) {

    clip = getImageFromDatabase(loop); debug(">>erasing clip"+data.setNum(loop));
    EraseFile = false;

    QString jahrenderpath = JahBasePath + "media/renders/"; //renders is renderpath as defined in glworld.cpp
    if (clip.location == jahrenderpath) { EraseFile = true; }

	  //if we are clear to erase the clip we do so now
	  if (EraseFile) {

      switch(clip.theCategory) {

        case  IMAGE:  { debug("erasing image "+ clip.location + clip.filename + clip.extension);

                        QFile f(clip.location + clip.filename + clip.extension);
                        if ( f.remove() )  { if (JAHDEBUG) debug("erase sucessful"); }
                        else debug("couldnt erase file");

                        break; }

        case  CLIP :  { debug("erasing clip " + clip.filename);
					
            						int length, blanklength;
            						QString currentframe, erasefile;
            						bool lasterase = false;
            						bool cleanerase = true;

            						for (int i = clip.startframe; i<=clip.endframe; i++) {

            							// This is where we create the extension for the sequence
            							//first figure out length of current frame
            							currentframe.setNum(i);
            							length = currentframe.length();  //was theblankframe

            							//then remove it from the blank template for the clip
            							QString blank(clip.videoext);
            							blank = blank.remove(0, length);

            							//now we need to build the sequence template
            							blanklength = blank.length();
            							QString theblank;
            							for (int ii=0; ii<blanklength; ii++) { theblank +="0"; }

            							blank = theblank+currentframe;

            							//then we build the path to the frame and open the file
            							erasefile = clip.location + clip.filename + blank + clip.extension;
            							if (JAHDEBUG) debug("erasing file:"+erasefile);

            							QFile f(erasefile);

            							//now some error checking on the erase procedure
            							if ( f.remove() )	{ if (JAHDEBUG) debug("erase sucessful"); lasterase = true; }
            							else				{ if (JAHDEBUG) debug("couldnt erase file"); cleanerase = false; }

            						}

						            if (cleanerase == true && lasterase == true) {
                           if (JAHDEBUG) debug("erased sequence successfully"); }

			                break; }

		  } //end of switch here
    }//end of if

  } //finish looping thorugh files


    debug("finished erasing");
    //////////////////////////////////////////////////////////////////
    //now we wipe the database and set global clipnumber back to 0

    bool wipe = WipeDatabase();

    //after wiping all the clips this clears the desktop
    //need to iterate thru desktop objects and erase them that way
    //this crashes system later on
    editor->clear();

    //set globalclipnumber to 0
    *clipnumber = 0;

    return wipe;
}

void JahDesktop::clearAll() {

    //create a popup window saying are you really sure?
    //do a qdialog...


    if (JAHDEBUG) { debug("called clear all..."); }


    int status = QMessageBox::critical( 0, "Clearing The Desktop",
                                           "You are about to Clear the Desktop!\n\n"
                                           "This will Erase all clips and images\n"
                                           "in your projects Renders directory!\n\n"
                                           "Export files you want to keep first!\n\n"
                                           "Do you still want to Clear All?\n",
                                           QMessageBox::Yes | QMessageBox::Default,
                                           QMessageBox::No,
                                           QMessageBox::Cancel | QMessageBox::Escape );


    switch (status) {

     case QMessageBox::Yes    : { debug("Hit Yes"); clear(); break; }

     case QMessageBox::No     : { debug("Hit No"); break; }

     case QMessageBox::Cancel : { debug("Hit Cancel"); break; }

     default :                  { break; }

   }





}


void JahDesktop::clearItem() {

	//first we remove the clip from the desktop
    editor->clearItem(); //should return a true for debugging if successful before moving ahead

	//then we grab the clips information for the erase procedure
    videoData  erase =  editor->selectedClip;

	//then we remove the image from the database
	//currently its just de-activated as we will be doing purging later on and erasing at that time
	removeImagefromdatabase(erase.clipnumber);

	//for safety we dont erase any stock images only images in the renders directory
    //we need to add the option to remove from database if the location is outside of the database
	bool EraseFile = false;

    QString jahrenderpath = JahBasePath + "media/renders/"; //renders is renderpath as defined in glworld.cpp
    if (erase.location == jahrenderpath) { EraseFile = true; }


	//if we are clear to erase the clip we do so now
	if (EraseFile) {

      switch(erase.theCategory) {

        case  IMAGE:  { debug("erasing image "+ erase.location + erase.filename + erase.extension);

                        QFile f(erase.location + erase.filename + erase.extension);
                        if ( f.remove() )  { if (JAHDEBUG) debug("erase sucessful"); }

								else debug("couldnt erase file");

                        break;
                       }

        case  CLIP :  { debug("erasing clip " + erase.filename);
					
						int length, blanklength;
						QString currentframe, erasefile;

						//flag that makes sure
						bool lasterase = false;
						bool cleanerase = true;

						for (int i = erase.startframe; i<=erase.endframe; i++) {

							// This is where we create the extension for the sequence

							//first figure out length of current frame
							currentframe.setNum(i);
							length = currentframe.length();  //was theblankframe

							//then remove it from the blank template for the clip
							QString blank(erase.videoext);
							blank = blank.remove(0, length);

							//now we need to build the sequence template
							blanklength = blank.length();
							QString theblank;
							for (int ii=0; ii<blanklength; ii++) {
								theblank +="0";
							}

							blank = theblank+currentframe;

							//then we build the path to the frame and open the file
							erasefile = erase.location + erase.filename + blank + erase.extension;
							if (JAHDEBUG) debug("erasing file:"+erasefile);

							QFile f(erasefile);

							//now some error checking on the erase procedure
							if ( f.remove() )	{ if (JAHDEBUG) debug("erase sucessful"); lasterase = true; }
							else				{ if (JAHDEBUG) debug("couldnt erase file"); cleanerase = false; }
						}

						if (cleanerase == true && lasterase == true) { if (JAHDEBUG) debug("erased sequence successfully"); }
			
			}

		}

	}

}

//this is used when we rebuild the database as it doesnt append the clipnumber to the iomages
void JahDesktop::addImage( videoData thedata) {

    //we need to scale the proxy size here instead of using the default 0.25% reduction
    float wratio = (180/(float)thedata.Image.width() );
    float hratio = (120/(float)thedata.Image.height() );

//    QString data; debug("wratio:"+data.setNum(wratio)+ " width:"+data.setNum( thedata.Image.width() *wratio ) +" height:" + data.setNum (thedata.Image.height()*hratio ) );

    QString theclipnumber;
      theclipnumber.setNum(thedata.clipnumber);

    QString clipper1; clipper1= thedata.clipname;

    img = new QImage[2];
    img[1] = thedata.Image.smoothScale( int(thedata.Image.width()*wratio), int(thedata.Image.height()*hratio) );

    // we need to update this so that we dont have to send the thedata.Image just thedata...
//    QCanvasPolygonalItem* i = new ImageItem(thedata.Image, img[1],canvas, clipper1);
    QCanvasPolygonalItem* i = new ImageItem(thedata, img[1],canvas, clipper1);
      i->setPen( QPen(QColor(200,0,0), 6) );
      i->setZ(1);
      i->setX(thedata.xpos);
      i->setY(thedata.ypos);
      i->show();

    delete [] img;      // is this necessary as img is not a pointer???
    editor->update();
}

void JahDesktop::addImage2( videoData thedata ) {

    //we need to scale the proxy size here instead of using the default 0.25% reduction
    float wratio = (180/(float)thedata.Image.width() );
    float hratio = (120/(float)thedata.Image.height() );

    //QString data; debug("wratio:"+data.setNum(wratio)+ " width:"+data.setNum( thedata.Image.width() *wratio ) +" height:" + data.setNum (thedata.Image.height()*hratio ) );

    QString theclipnumber;
      theclipnumber.setNum(thedata.clipnumber);

    QString clipper1; clipper1= thedata.clipname;
    clipper1.append(theclipnumber);

    img = new QImage[2];
    img[1] = thedata.Image.smoothScale( int(thedata.Image.width()*wratio), int(thedata.Image.height()*hratio) );

    // we need to update this so that we dont have to send the thedata.Image just thedata...
//    QCanvasPolygonalItem* i = new ImageItem(thedata.Image, img[1],canvas, clipper1);
    QCanvasPolygonalItem* i = new ImageItem(thedata, img[1],canvas, clipper1);
      i->setPen( QPen(QColor(200,0,0), 6) );
      i->setZ(1);
      i->setX(thedata.xpos);
      i->setY(thedata.ypos);
      i->show();

    delete [] img;      // is this necessary as img is not a pointer???
    editor->update();
}



void JahDesktop::addRectangle() {
    QCanvasPolygonalItem *i = new QCanvasRectangle( 0,0, 180,120,canvas);
    i->setBrush( QColor(50,0,200) );
    i->setPen( QPen(QColor(0,50,200), 6) );
    i->setZ(1);
    i->show();
}

QImage JahDesktop::getClip() {
    // change this to send a pointer instead of a qimage using globalobject!!!
    return editor->globalimage;
}

videoData JahDesktop::getVideoData() {

    return editor->selectedClip;

}

void JahDesktop::setClipname(QString& newname)
{
     editor->setItemtext(newname);
	 editor->update();
}

void JahDesktop::doRefresh(void)
{   //editor->setChanged( editor->boundingRect() );
    canvas->setAllChanged();
    canvas->update();
    editor->update();
	
}

QString JahDesktop::getClipname() {
    // change this to send a pointer instead of a qimage using globalobject!!!
    return editor->imagetext;
}

















