/*******************************************************************************
**
** The header file for the Jahshaka desktop module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#ifndef DESKTOP_H
#define DESKTOP_H

//#include <interfaceheaders.h>
//#include <coreobject.h>
#include <glworld.h>

#include <sqlite.h>
#include <qmessagebox.h>

extern QString JahBasePath;
extern bool JAHDEBUG;

struct videoData;
struct sqlite;

static const int imageRTTI = 984376;
const bool no_writing = FALSE;

class ImageItem: public QCanvasRectangle {
public:

  ImageItem( videoData full, QImage img, QCanvas *canvas, QString& clipname );
  int rtti () const { return imageRTTI; }
  bool hit( const QPoint&) const;

  QImage getImage() { return fullres; }

  videoData getClip() { return itemdata; }

  QString getText() { return name; }
  void setText(QString& newname) { name=newname;}

protected:
  void drawShape( QPainter & );

private:
  QImage image;
  QImage fullres;
  QPixmap pixmap;
  QString name;

  videoData itemdata;
};

class FigureEditor : public QCanvasView {
  Q_OBJECT

public:
  FigureEditor(QCanvas*, QWidget* parent=0, const char* name=0, WFlags f=0);
  void clear();
  void clearItem();
  void setItemtext(QString&);
  QImage globalimage;
  QString imagetext;
  ImageItem *globalitem;

  videoData selectedClip;

  //temporary workaround here as sgi dont check for infunc creation properly
  bool bordercreated;
  bool move;

protected:
  void contentsMousePressEvent(QMouseEvent*);
  void contentsMouseReleaseEvent(QMouseEvent*);
  void contentsMouseMoveEvent(QMouseEvent*);

signals:
  void status(const QString&);
  void updateClipDatabase(videoData data);

private:
  QCanvasItem* moving;
  QPoint moving_start;
  int height;

  QCanvasPolygonalItem  *border;
  QCanvasItem* borderframe;
  QCanvasText* itemtext;
};



class JahDesktop : public QWidget {
  Q_OBJECT

public:
  JahDesktop(QWidget* parent=0, const char* name=0, int * globalclipnumber=0, WFlags f=0);

  ~JahDesktop();

  QImage getClip();
  void setClipname(QString&);
  void doRefresh(void);
  QString getClipname();
  int *clipnumber;
  videoData getVideoData();

public slots:

  void addImage2( videoData thedata );
  void addImage( videoData thedata );

  void addImage2database( videoData thedata );
  void renameImageindatabase( int imagenumber, QString newname);

signals:
  void resetGlobalClipNumber();

private slots:

  bool clear();
  void clearItem();
  void clearAll();
  void init();
  void addRectangle();
  void slotSave();
  void slotLoad();
  bool getExtension(QString & string);
  bool isanumber(QString & string);

  void InitializeDatabase();
  void CloseDatabase();
  bool WipeDatabase(void);

  int  OpenDesktopTable();
  char ** GetVideodataFromTable(int clipnum);

  void RebuildDesktop();
  int  GetDatabaseClipNumber();
  void removeImagefromdatabase( int imagenumber );
  void deleteImagefromdatabase( int imagenumber );
  videoData getImageFromDatabase(int clipnumber);
  void purgeDatabase( int imagenumber );

  void updateClipDatabase(videoData data);


private:
  FigureEditor *editor;
  QCanvas* canvas;
  int dbf_id;
  QImage* img;

  //database
  sqlite*         db;
  };


#endif







