/*******************************************************************************
**
** The source file for the Jahshaka desktop module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "desktop.h"

//ImageItem::ImageItem( QImage full, QImage img, QCanvas *canvas, QString& clipname )
ImageItem::ImageItem( videoData full, QImage img, QCanvas *canvas, QString& clipname )
        : QCanvasRectangle( canvas ), image(img) {
    setSize( image.width(), image.height() );

#ifndef _WS_QWS_
    pixmap.convertFromImage(image, OrderedAlphaDither);
#endif
    fullres = full.Image;
    name=clipname;

    itemdata = full;
}


void ImageItem::drawShape( QPainter &p ) {
    // On Qt/Embedded, we can paint a QImage as fast as a QPixmap,
    // but on other platforms, we need to use a QPixmap.
#ifdef _WS_QWS_
    p.drawImage( int(x()), int(y()), image, 0, 0, -1, -1, OrderedAlphaDither );
#else
    p.drawPixmap( int(x()), int(y()), pixmap );
#endif
}

bool ImageItem::hit( const QPoint &p ) const {
    int ix = p.x()-int(x());
    int iy = p.y()-int(y());
    if ( !image.valid( ix , iy ) )
        return FALSE;
    QRgb pixel = image.pixel( ix, iy );
    return qAlpha( pixel ) != 0;
}

FigureEditor::FigureEditor( QCanvas* c, QWidget* parent,
 const char* name, WFlags f) :
 QCanvasView(c,parent,name,f) {

    height=1;   //what happens if this goes over the max value for a int? crash!!!
    //ImageItem globalitem;

    bordercreated = FALSE;
    move = FALSE;
}


void FigureEditor::contentsMousePressEvent(QMouseEvent* e) {

    // creating border anywhere else ie in constructor crashes app...
    // we want to use a better method anyways so aint gonna waste no more time on this
    // ideas are just to draw selection boundary
    // moving->setSelected(1);  // QRect myframe = moving->boundingRect();

    QCanvasItemList l=canvas()->collisions(e->pos());
	videoData itemData;

    for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {

        if ( (*it)->rtti() == imageRTTI ) {
            ImageItem *item= (ImageItem*)(*it);
            if ( !item->hit( e->pos() ) ) {
                continue;
            }
            move = 1;

            globalitem = (ImageItem*)(*it);
            globalimage = item->getImage();
            imagetext = item->getText();

            selectedClip = item->getClip();
            selectedClip.xpos = item->x();
            selectedClip.ypos = item->y();

			itemData = item->getClip();
        }

		//if we know what kind of clip it is
		//we can draw the relevant border
		//and clip data
		//videoData itemData = item->getClip();
	QColor bordercolor;

    if (!bordercreated) {

        border = new QCanvasRectangle( 0,0, 180,120,canvas());

    	bordercolor.setRgb(200,0,50);
		if (itemData.theCategory == IMAGE) { bordercolor.setRgb(0,50,200); }
		if (itemData.theCategory == CLIP ) { bordercolor.setRgb(0,200,50); }

        border->setBrush( bordercolor );
        border->setPen( QPen( bordercolor, 6) );
        border->setZ(1);

        itemtext = new QCanvasText( canvas());
        itemtext->setColor( QColor(200,200,200) );
        itemtext->setText(imagetext);
        itemtext->setZ(1);
        bordercreated = TRUE;
    }

    itemtext->setText(imagetext);

		if (itemData.theCategory == IMAGE) { bordercolor.setRgb(0,50,200); }
		if (itemData.theCategory == CLIP ) { bordercolor.setRgb(0,200,50); }
        border->setBrush( bordercolor );
        border->setPen( QPen( bordercolor, 6) );

        //moving is set to canvas item referenced by *it
        moving = *it;

        moving_start = e->pos();


        //now we position border and bring selected item and
        //border to the front of others on the canvas
        border->setX( moving->x() );
        border->setY( moving->y() );
        height++; border->setZ(height);

        itemtext->setX( moving->x() );
        itemtext->setY( moving->y() );

        height++; moving->setZ(height);
        height++; itemtext->setZ(height);

        border->show();
        itemtext->show();
        canvas()->update();

        return;
    }
    moving = 0;

}

//QImage FigureEditor::getCurrentClip() {
//    QImage temp;
 //   temp=moving.getImage();
 //   return temp;
//}


void FigureEditor::clear() {
debug("clearing the list");

    //QCanvasItemList list = this->allItems();
    QCanvasItemList list = canvas()->allItems();
    QCanvasItemList::Iterator it = list.begin();

    for (; it != list.end(); ++it) {
        if ( *it ) {
            delete *it;
        }
    }

    canvas()->update();

    bordercreated = FALSE;
    move = FALSE;

    height=1;    //reset this variable when we clear the desktop
}


void FigureEditor::clearItem() {

    if (  moving ) {

        delete moving;   //delete selectedClip;
        //delete border;
        //delete itemtext;
        border->hide();   itemtext->hide();
        canvas()->update();
    }
}

void FigureEditor::setItemtext(QString& newname) {

    debug("setting clipname");  debug(newname);
    globalitem->setText(newname);
    canvas()->update();
}



void FigureEditor::contentsMouseReleaseEvent(QMouseEvent* e) {

if (moving) {

    bool update = false;

    if  ( selectedClip.xpos != moving->x() ) { selectedClip.xpos = moving->x(); update=true; }
    if  ( selectedClip.ypos != moving->y() ) { selectedClip.ypos = moving->y(); update=true; }

    //update the clips position in the database
    if (update) { emit updateClipDatabase(selectedClip); }

    //need to release move here?
    move = 0;
    //moving = 0;
    canvas()->update();
    }

}

void FigureEditor::contentsMouseMoveEvent(QMouseEvent* e) {
    if ( move ) {   // was moving

        //move clip and the background border
        moving->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());
        border->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());
        //text moveby
        itemtext->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());

        moving_start = e->pos();
        canvas()->update();
    }
}
















