/*******************************************************************************
**
** The source file for the Jahshaka desktop module
** The Jahshaka Project
** Copyright (C) 2000-2001 Bedagi, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/

#include "desktop.h"


static int callback(void *NotUsed, int argc, char **argv, char **azColName){
  int i; QString data, data1, data2;
  for(i=0; i<argc; i++) {
    data1 = azColName[i]; data2 = argv[i];
    data  = data1 + " = " +  data2;
    debug(data);
  }
  return 0;
}

void JahDesktop::InitializeDatabase(void) {

    ////////////////////////////////////////////////////////
    //Open the database

    char *zErrMsg = 0;

	QString databasename = "JahDesktopDB";
	QString jahdatabase  = "database/"+ databasename;

	if (JAHDEBUG) { debug("Initializing Jahshaka Database at"); debug(jahdatabase); }

	//execute the sql commnad to open the database
    db = sqlite_open(jahdatabase, 0, &zErrMsg);

    if( db==0 ){
      debug("Unable to open the database"); debug (zErrMsg);
      exit(1);
    } else { debug("Initialized JahDB Database");
			 debug("-------------------------------------");
	}

}

void JahDesktop::CloseDatabase(void) {

    if (JAHDEBUG) debug("Syncing desktop database tables");

    // update up the desktop database both name and clipnumber...
    char *zErrMsg = 0;
    int  rc;

    //////////////////////////////////////////////////////
    // update desktop table

	//get the global clipnumber
    QString conversion;
    int clipnum      = *clipnumber;
    QString numclips = conversion.setNum(clipnum);

    //build the sql command
    QString addtodbase = QString("update desktop set  numclips= %1").arg(numclips);
    if (JAHDEBUG) debug(addtodbase);

	//execute the sql command
    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);

    //close the database
    sqlite_close(db);   debug("Jahshaka database closed");

}


bool JahDesktop::WipeDatabase(void) {

  if (JAHDEBUG) debug("Wiping the desktop database tables");

  // update up the desktop database both name and clipnumber...
  char *zErrMsg = 0;
  int  rc;
  QString sql;

  //build the sql command
  sql = QString("update desktop set  numclips= 0");
  rc = sqlite_exec(db, sql, callback, 0, &zErrMsg);

  if( rc!=SQLITE_OK ) { return false; }

  //build the sql command
  sql = QString("delete from videodata");
  rc = sqlite_exec(db, sql, callback, 0, &zErrMsg);

  if( rc!=SQLITE_OK ) { return false; }

  return true;

}


void JahDesktop::updateClipDatabase(videoData data) {

	//update the database with new information from
	//videData on a clips position once it has been moved
	//need some error checking done here

    if (JAHDEBUG) debug("---------<<<<<<<<<<>>>>>>>>>>--------");
	//if (JAHDEBUG) data.printinfo();

    char *zErrMsg = 0;
    int  rc;

    int theclipnumber    = data.clipnumber;
    double posx          = data.xpos;
    double posy          = data.ypos;

    QString conversion;
    QString theclipnumber_st = conversion.setNum(theclipnumber);
    QString posx_st = conversion.setNum(posx);
    QString posy_st = conversion.setNum(posy);

    //build and execute the sql command to update xpos
    QString addtodbase;
    addtodbase = QString("update videodata set  xpos = %1 where MediaNumber = %2")
		                 .arg(posx_st)
						 .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);

	//build and execute the sql command to update ypos
    addtodbase = QString("update videodata set  ypos = %1 where MediaNumber = %2")
		                  .arg(posy_st)
			                .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);

}

int JahDesktop::OpenDesktopTable() {

    //command to read in images for desktop
    //and add them to desktop

	  if (JAHDEBUG) debug("Opening Desktop Table");

    int numclip;

    char *zErrMsg = 0;
    int   rc,max_rows,max_cols;
    char  **desktopdata;

    //build the sql command here
    char   *sql; sql = "select * from desktop;"; //where desktop number = 1 for multiple desktops


    rc  = sqlite_get_table(db, sql, &desktopdata, &max_rows, &max_cols, &zErrMsg);

    if( rc!=SQLITE_OK ){ debug("SQL Error bad database"); numclip = -1; }
	                else { debug("Opened desktop table");

                         //object data
                         QString desktopname    = desktopdata[2];  if (JAHDEBUG) debug(desktopname);
                         QString numberofclips  = desktopdata[3];  if (JAHDEBUG) debug(numberofclips);

                         numclip = numberofclips.toInt();
    }

    return numclip;

}

char ** JahDesktop::GetVideodataFromTable(int clipnum) {

    //command to read in images for desktop
    //and add them to desktop

	  if (JAHDEBUG) debug("Getting entry from Table");

      //convert clipnumber to char
			QString conversion;
			QString theclipnum   = conversion.setNum(clipnum);  //enum

      //build the sql command here as char*
			QString sqlcom = "select * from videodata where MediaNumber="+theclipnum;

			const char* fontchar = sqlcom;
			char*  sql = new char[sqlcom.length()+3];
			strcpy(sql,fontchar);


    char *zErrMsg = 0;
    int   rc, max_rows,max_cols;
    char  **thedesktopdata;

    debug("gettingit");
    rc  = sqlite_get_table(db, sql, &thedesktopdata, &max_rows, &max_cols, &zErrMsg);

    if( rc!=SQLITE_OK ){ debug("SQL Error bad database"); }
	                else { debug("Opened desktop table");   }

    return thedesktopdata;

}
//break this out into separate routines
//to make it easier to deal with...

//make this return bool for success to trap errors
void JahDesktop::RebuildDesktop() {

    //command to read in images for desktop
    //and add them to desktop
	  if (JAHDEBUG) debug("Building Jahshaka Desktop");

    int numclip = OpenDesktopTable();

	///////////////////////////////////////////////////////////////////////
	// if there are one or more clips registered then test the database and
	// load all the clips into the desktop!

	if (numclip>0)
	{

		// test database first
		if (JAHDEBUG) { debug("Testing the database");
    debug("-------------------------------------"); }

    //gets number of entries in database
		//int counter = 3;
		int counter = GetDatabaseClipNumber();

		//now we know clipnumber and number of entries
		if (counter==numclip) { if (JAHDEBUG) debug("database is ok"); }
		else                  { if (JAHDEBUG) debug("database got issues");

                            if (counter>numclip) { if (JAHDEBUG) debug("Extra clips! Salvaging missing clips");
                            numclip=counter; }
                            							

                            if (counter<numclip) { if (JAHDEBUG) debug("Missing data! Resetting desktop clipnumber");
                            numclip= counter; }
								            //rewrite desktop table data in case we crash again!
		}

		if (JAHDEBUG) debug("-------------------------------------");



		/////////////////////////////////////////////////
		//loop through number of clips in videodatatable and add each one to the desktop
		//we need to update image locations for all clips when a clip is added and when we are leaving the app
		//using the sql:update command

		if (JAHDEBUG) debug("Reading in Clips from Database");

		for (int x=1; x<=numclip; x++) {

	
		//results are in here
 		char  **videodata=GetVideodataFromTable(x);

		// we start at 12 because there are 11 column headers that come first

		QString category    = videodata[12];
		QString cliptype    = videodata[13];
		QString videoext    = videodata[14];
		QString clipname    = videodata[15];
		QString clipnum     = videodata[16];
		QString location    = videodata[17];
		QString filename    = videodata[18];
		QString extension   = videodata[19];
		QString startframe  = videodata[20];
		QString endframe    = videodata[21];
		QString xpos        = videodata[22];
		QString ypos        = videodata[23];

		if (JAHDEBUG) debug( category+cliptype+videoext+clipname+clipnum+location+filename+extension+startframe+endframe+xpos+ypos );

		videoData thedata;

		   int thecat = category.toInt();

		   if (thecat !=9) {

			   if (thecat==0) { thedata.theCategory = IMAGE; if (JAHDEBUG) { debug("is image"); } }
			   if (thecat==2) { thedata.theCategory = CLIP;  if (JAHDEBUG) { debug("is clip");  } }

				int thecliptype = cliptype.toInt();
				thedata.cliptype    = (VideoType)thecliptype;    //clipname is truncated filename

			   /*
			   switch(thecat) {   //convert to int?

				   case  0:  { thedata.theCategory = IMAGE; debug("is image"); }
				   case  3:  { thedata.theCategory = CLIP;  debug("is clip");  }
				 }
				 */
			   thedata.clipname    = clipname;    //clipname is truncated filename
			   thedata.videoext    = videoext;    //clipname is truncated filename
			   thedata.clipnumber  = clipnum.toInt();
			   thedata.location    = location;    //the path without the filename...
			   thedata.filename    = filename;     //the filename on disk without extension
			   thedata.extension   = extension; //we need to change this based on extension
			   thedata.startframe  = startframe.toInt();
			   thedata.endframe    = endframe.toInt();
			   thedata.keyStatus   = FALSE;
			   thedata.xpos        = xpos.toInt();
			   thedata.ypos        = ypos.toInt();

			   QString loadfilename;

			   if (thecat==0) {  loadfilename = location+filename+extension; }
			   if (thecat==2) {  loadfilename = location+filename+videoext+extension; }

			   debug("Going to load:");

			   if (JAHDEBUG) debug(loadfilename);
			   //QString loadfilename = location+filename+extension;

			   if ( !loadfilename.isEmpty() ) {

        QPixmap buffer;
        QImageIO iio; iio.setFileName( loadfilename );
        if (extension==".tga" || extension==".TGA") { iio.setFormat("TGA"); }
        if ( iio.read() )  buffer = iio.image();  // convert to pixmap




					//QPixmap buffer;
					//buffer.load( loadfilename );    //this needs to be exactly whats in filename
					thedata.Image = buffer;

					//a lousy way to check if the clip has been renamed
					if (clipname == filename)
					addImage2(thedata); else addImage(thedata);
				 }
			   canvas->update();

			   *clipnumber = numclip;

		   }

		} //the for loop for all clips here

	}

    if (JAHDEBUG) debug("Opened videodata table");

    //system crashes if correct number of clips in videodata is not specified
    //we need to update global clipnumber


}

///////////////////////////////////////////////////////////
// gets the number of each desktop clip from the database
// not the desktop so we can verify in case there was a crash and recover

int JahDesktop::GetDatabaseClipNumber() {

debug(">>>getting database clip number<<<");

	int thecounter=0;

	int theloop=1; //how about bool status here outside the loop

		QString conversion;
		QString theclipid;
    QString sqlcom;
    QString theclipnum;

		char  **videodata;
		int rc,max_rows,max_cols;
    char *zErrMsg = 0;
 		bool thestatus=true;

	while ( theloop ) { //fix this to keep looping until it breaks

		//build the sql command		
		theclipnum   = conversion.setNum(theloop);
		sqlcom = "select MediaName from videodata where MediaNumber="+theclipnum;
    debug("SQL:"+sqlcom);

		const char* fontchar =  sqlcom;
		char*  sqlcommand    =  new char[ sqlcom.length()+3 ];
		strcpy(sqlcommand,fontchar);

    //execute the aql command here
		rc  = sqlite_get_table(db, sqlcommand, &videodata, &max_rows, &max_cols, &zErrMsg);


    //need to clean this up
    if( rc!=SQLITE_OK ){ debug("SQL Error in database"); thestatus=false; }
	  theclipid = videodata[0]; debug(theclipid);
    if (theclipid=="MediaName") { debug("read it"); } else { debug("didnt read it"); thestatus=false;}


    if (thestatus) { thecounter = theloop; theloop++; } else { break; }

  }


	if (JAHDEBUG) { QString temp; debug("The number returned was"+temp.setNum(thecounter)); }
  delete videodata;
	return thecounter;
	

}


//need to clean this up and add new data types such as SEQUENCE and videoext, keyext, etc

void JahDesktop::addImage2database( videoData thedata )  {

   //thedata.printinfo();

    //we also need to add this data to the database JahDB

    //but we need to make sure its open first...
    debug("Adding image to database");

    int rc;  char *zErrMsg = 0;

    //create add string
    QString addtodbase; QString conversion;

    QString category   = conversion.setNum(thedata.theCategory);  //enum
    QString cliptype   = conversion.setNum(thedata.cliptype);  //enum
    QString clipname   = thedata.clipname;
    QString clipnumber = conversion.setNum(thedata.clipnumber);   //int
    QString location   = thedata.location;
    QString filename   = thedata.filename;
    QString extension  = thedata.extension;
    QString startframe = conversion.setNum(thedata.startframe);   //int       .setNum(value)
    QString endframe   = conversion.setNum(thedata.endframe);     //int
    QString xypos      = conversion.setNum(thedata.xpos) +","+conversion.setNum(thedata.ypos);
    QString videoext   = thedata.videoext;

    QString thetable="videodata";

    addtodbase = QString("insert into videodata values( '%1','%2', '%3','%4','%5','%6','%7','%8', ")
                 .arg(category)
                 .arg(cliptype)
                 .arg(videoext)
                 .arg(clipname)
                 .arg(clipnumber)
                 .arg(location)
                 .arg(filename)
                 .arg(extension);

	addtodbase += startframe + ", " + endframe + ", " + xypos + ");";

    debug(addtodbase);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);

    debug("image added to database");

}

void JahDesktop::removeImagefromdatabase( int imagenumber ) {

  //we need to remove the data from the database
  //for the case where clipnumber = imagenumber
  // which is field MediaNumer in table videodata

	//for now we will just de-activate clips
	//until we can figure this out properly
	//we do this by setting category to 9


    if (JAHDEBUG) debug("---------<<<<<<<<<<delete>>>>>>>>>>--------");

    char *zErrMsg = 0;
    int  rc;

    int theclipnumber    = imagenumber;


    QString conversion;
    QString theclipnumber_st = conversion.setNum(theclipnumber);

    //build and execute the sql command to update xpos
    QString addtodbase;
    addtodbase = QString("update videodata set  category = 9 where MediaNumber = %1")
						 .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);


}

void JahDesktop::deleteImagefromdatabase( int imagenumber ) {

    if (JAHDEBUG) debug("---------<<<<<<<<<<erase>>>>>>>>>>--------");

    char *zErrMsg = 0;
    int  rc;

    int theclipnumber    = imagenumber;


    QString conversion;
    QString theclipnumber_st = conversion.setNum(theclipnumber);

    //build and execute the sql command to update xpos
    QString addtodbase;
    addtodbase = QString("delete from videodata where MediaNumber = %1")
						 .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);


}

void JahDesktop::renameImageindatabase( int imagenumber, QString newname) {

  //we need to remove the data from the database
  //for the case where clipnumber = imagenumber
  // which is field MediaNumer in table videodata

	//for now we will just de-activate clips
	//until we can figure this out properly
	//we do this by setting category to 9


    if (JAHDEBUG) debug("---------<<<<<<<<<<rename>>>>>>>>>>--------");

    char *zErrMsg = 0;
    int  rc;

    int theclipnumber    = imagenumber;


    QString conversion;
    QString theclipnumber_st = conversion.setNum(theclipnumber);

    //build and execute the sql command to update xpos
    QString addtodbase;
    addtodbase = QString("update videodata set  MediaName = '%1' where MediaNumber = %2")
						 .arg(newname)
						 .arg(theclipnumber_st);

    rc = sqlite_exec(db, addtodbase, callback, 0, &zErrMsg);


}

///////////////////////////////////////////////////////////
// gets the number of each desktop clip from the database

videoData JahDesktop::getImageFromDatabase(int clipnumber) {

		//build the sql command
		QString conversion;
		QString theclipnum   = conversion.setNum(clipnumber);  //enum

		//workaround - if theres no name theres no clip
		QString sqlcom = "select * from videodata where MediaNumber="+theclipnum;

		//better way to do this ?
		const char* fontchar =  sqlcom;
		char*  sqlcommand    =  new char[ sqlcom.length() ];
		strcpy(sqlcommand,fontchar);

		char  **videodata;
		int rc,max_rows,max_cols;
    char *zErrMsg = 0;

		rc  = sqlite_get_table(db, sqlcommand, &videodata, &max_rows, &max_cols, &zErrMsg);

      //we should set the clip name to something weird to trap errors
		  if( rc!=SQLITE_OK ){ fprintf(stderr, "SQL error: %s\n", zErrMsg); }
						else { if (JAHDEBUG) debug("located clip in database");  }

		// we start at 12 because there are 11 column headers that come first

		QString category    = videodata[12];
		QString cliptype    = videodata[13];
		QString videoext    = videodata[14];
		QString clipname    = videodata[15];
		QString clipnum     = videodata[16];
		QString location    = videodata[17];
		QString filename    = videodata[18];
		QString extension   = videodata[19];
		QString startframe  = videodata[20];
		QString endframe    = videodata[21];
		QString xpos        = videodata[22];
		QString ypos        = videodata[23];

		//if (JAHDEBUG) debug( category+cliptype+videoext+clipname+clipnum+location+filename+extension+startframe+endframe+xpos+ypos );

		videoData thedata;

		   int thecat = category.toInt();

		   if (thecat !=9) {

			   if (thecat==0) { thedata.theCategory = IMAGE; if (JAHDEBUG) { debug("is image"); } }
			   if (thecat==2) { thedata.theCategory = CLIP;  if (JAHDEBUG) { debug("is clip");  } }

				int thecliptype = cliptype.toInt();
				thedata.cliptype    = (VideoType)thecliptype;    //clipname is truncated filename

			   thedata.clipname    = clipname;    //clipname is truncated filename
			   thedata.videoext    = videoext;    //clipname is truncated filename
			   thedata.clipnumber  = clipnum.toInt();
			   thedata.location    = location;    //the path without the filename...
			   thedata.filename    = filename;     //the filename on disk without extension
			   thedata.extension   = extension; //we need to change this based on extension
			   thedata.startframe  = startframe.toInt();
			   thedata.endframe    = endframe.toInt();
			   thedata.keyStatus   = FALSE;
			   thedata.xpos        = xpos.toInt();
			   thedata.ypos        = ypos.toInt();

         //thedata.printinfo();
     }

  return thedata;

}

/////////////////////////////////////////////////////
// called on exit in order to purge deactivated clips

//the cool part here is when erase doesnt actuall erase the clips... just hides them
//so you just purge the database when youre running out of storage
//instead of a index we can erase all hidden clips and renumber the database from start to finish
// ie set medianumber from 1 to xxx after removing all dead entries and deleting the files
//then we can set globalclipnumber back...

// old notes here

// but first we really need a index field since the clinumber is also part of
// the pathname for rendered clips
// but the general idea is to go through all clips
// erase the ones with a 9
// delete associated files
// and then reindex the clips

void JahDesktop::purgeDatabase( int imagenumber ) {
/*
	if (JAHDEBUG) debug("Purging Desktop Database");

    char *zErrMsg = 0;
    int   rc,max_rows,max_cols;
    char  **desktopdata;

    //build the sql command here
    char   *sql; sql = "select * from desktop;"; //where desktop number = 1 for multiple desktops

    rc  = sqlite_get_table(db, sql, &desktopdata, &max_rows, &max_cols, &zErrMsg);

    if( rc!=SQLITE_OK ){ fprintf(stderr, "SQL error: %s\n", zErrMsg); }
	                    else { debug("Opened desktop table");  }

    //object data
    QString desktopname    = desktopdata[2];  if (JAHDEBUG) debug(desktopname);
    QString numberofclips  = desktopdata[3];  if (JAHDEBUG) debug(numberofclips);

    int numclip = numberofclips.toInt();

    //now we know howmany clips are in the database
	QString conversion;
	QString theclipnum       = conversion.setNum(numclip);

	int counter = GetDatabaseClipNumber();
	QString theclipcounter   = conversion.setNum(counter);

	if (JAHDEBUG) debug("the clip number read is "+theclipnum);
	if (JAHDEBUG) debug("the clip counter read   "+theclipcounter);




	int counter;

	int loop=1; //how about bool status here outside the loop

	while ( loop ) { //fix this to keep looping until it breaks

		//build the sql command
		QString theclipnum   = conversion.setNum(loop);  //enum

		//workaround - if theres no name theres no clip
		QString sqlcom = "select MediaName from videodata where MediaNumber="+theclipnum;

		//better way to do this ?
		const char* fontchar =  sqlcom;
		char*  sqlcommand    =  new char[ sqlcom.length() ];
		strcpy(sqlcommand,fontchar);

		char  **videodata;
		int rc,max_rows,max_cols;
    char *zErrMsg = 0;


		rc  = sqlite_get_table(db, sqlcommand, &videodata, &max_rows, &max_cols, &zErrMsg);

		 QString clipnum;
		 bool status;

		 if( rc==SQLITE_OK )  { clipnum = videodata[0];
			 if (clipnum == "MediaName") { status=true; } else { status=false; }	
		 } else { status=false; }

		 if (status) { debug("true");  counter = loop; loop++; } else { break; }
	}

	return counter;
	
*/

}











