/*
 * main.c --
 *
 *      Main procedure
 *
 */

/*
 * Copyright (c) 1995 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

/*
 * Portions of this software Copyright (c) 1995 Brown University.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement
 * is hereby granted, provided that the above copyright notice and the
 * following two paragraphs appear in all copies of this software.
 *
 * IN NO EVENT SHALL BROWN UNIVERSITY BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF BROWN
 * UNIVERSITY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * BROWN UNIVERSITY SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND BROWN UNIVERSITY HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
 * SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include "mpegplay.h"

/*
   Changes to make the code reentrant:
     Got rid of setjmp, longjmp
     deglobalized: EOF_flag, FilmState, curVidStream, bitOffset, bitLength,
     bitBuffer, sys_layer, input, seekValue, window, X Windows globals (to
     xinfo), curBits, ditherType, matched_depth, totNumFrames, realTimeStart

   Additional changes:
     Ability to play >1 movie (w/out CONTROLS)
     Make sure we do a full frame for each movie
     DISABLE_DITHER #ifdef to avoid compiling dithering code
     Changes to deal with non-MPEG streams
     Now deals with NO_DITHER, PPM_DITHER and noDisplayFlag==1
     CONTROLS version now can deal with >1 movie
   -lsh@cs.brown.edu (Loring Holden)
 */



int loopFlag = 0;

#ifdef QUIET
int quietFlag = 1;
#else
int quietFlag = 0;
#endif

/* "Press return" flag, requires return for each new frame */
int requireKeypressFlag = 0;

/* Display image on screen? */
int noDisplayFlag = 0;

/* Seek Value.
   0 means do not seek.
   N (N>0) means seek to N after the header is parsed
   N (N<0) means the seek has beeen done to offset N
*/

/* Framerate, -1: specified in stream (default)
               0: as fast as possible
               N (N>0): N frames/sec
               */
int framerate;

/* Flag for high quality at the expense of speed */
int qualityFlag;

/* no further error messages */
static BOOLEAN exiting=FALSE;

/* Flags/values to control Arbitrary start/stop frames. */
int partialFlag = 0, startFrame = -1, endFrame = -1;

int gammaCorrectFlag;
double gammaCorrect;

/* Flag for chroma correction */
int chromaCorrectFlag;
double chromaCorrect;

/* global variable for interrupt handlers */
VidStream **curVidStream;

XInfo * xinfo;
int numInput;

/*
 #define Color16DitherImage ColorDitherImage
 #define Color32DitherImage ColorDitherImage
*/

/*
 *--------------------------------------------------------------
 *
 * int_handler --
 *
 *        Handles Cntl-C interupts..
 *      (two different ones for different OSes)
 * Results:    None.
 * Side effects:   None.
 *--------------------------------------------------------------
 */
#ifndef SIG_ONE_PARAM
void int_handler()
#else
void int_handler(int signum)
#endif
{
  int i,displayClosed=0;

  if (!quietFlag && !exiting) {
    fprintf(stderr, "Interrupted!\n");
  }
  exiting = TRUE;
  for (i = 0;  i < numInput;  i++) {
    if (curVidStream[i] != NULL)
      DestroyVidStream(curVidStream[i], &xinfo[i]);
    if ((xinfo[i].display != NULL) && !displayClosed) {
      XCloseDisplay(xinfo[i].display);
      displayClosed=1;
    }
  }
  exit(1);
}


/*
 *--------------------------------------------------------------
 *
 * bad_handler --
 *
 *        Handles Seg faults/bus errors...
 *      (two different ones for different OSes)
 * Results:    None.
 * Side effects:   None.
 *
 *--------------------------------------------------------------
 */

#ifndef SIG_ONE_PARAM
void bad_handler()
#else
void bad_handler(int signum)
#endif
{
  if (!exiting) {
    fprintf(stderr, "Bad MPEG?  Giving up.\ntry 'mpeg_stat -verify' to see if the stream is valid.\n");
  }
  //exit(0);
  return;
}


/*
 *--------------------------------------------------------------
 *
 * getposition --
 *
 *--------------------------------------------------------------
 */
void getposition(char *arg, int *xpos, int *ypos)
{
  char *pos;

  if ((pos = strtok(arg, "+-")) != NULL) {
    *xpos = atoi(pos);
    if ((pos = strtok(NULL, "+-")) != NULL) {
      *ypos = atoi(pos);
      return;
    }
  }
  if (!quietFlag) {
    fprintf(stderr, "Illegal position... Warning: argument ignored! (-position +x+y)\n");
  }
  return;
}


/*
 *--------------------------------------------------------------
 *
 * main --
 *
 *        Parses command line, starts decoding and displaying.
 *
 * Results:
 *        None.
 *
 * Side effects:
 *        None.
 *
 *--------------------------------------------------------------
 */

//need to fix this so it only plays 1 clip
//for better memory handling

void mainplay(int argcc,char **argvv)

{

#ifdef DCPREC
/* Declaration of global variable to hold DC precision */
int dcprec = 0;
#endif

/* Global file pointer to incoming data. */
FILE **input;
char **inputName;

/* Loop flag. */
loopFlag = 0;

/* Shared memory flag. */
int shmemFlag = 0;

framerate = -1;

/* Flag for high quality at the expense of speed */
#ifdef QUALITY
 qualityFlag = 1;
#else
 qualityFlag = 0;
#endif




/* Flags/values to control Arbitrary start/stop frames. */
partialFlag = 0; startFrame = -1; endFrame = -1;

/* Flag for gamma correction */
  gammaCorrectFlag = 0;
  gammaCorrect = 1.0;

/* Flag for chroma correction */
  chromaCorrectFlag = 0;
  chromaCorrect = 1.0;







/* Brown - put X specific variables in xinfo struct */
#define NUMMOVIES 1
XInfo xinfo[NUMMOVIES];
numInput=0;







  fprintf(stderr, ">>starting player...");

  //hardcoded parameter file
  char * argve="-dither color /root/jahstorage/media/encoded/AnimationClip2.mpg";
  char ** argv= *argve;//"-dither color /root/jahshaka/renders/AnimClip4_.mpg";
  //char * argv="/tmp/flipMPEG-root.param";
  int argc = 3;
  fprintf(stderr, ">>we made it here...");

  char *name;
  static VidStream **theStream;
  int mark;
  int i, mult, largy, y, lastStream, firstStream=-1, workToDo=TRUE;
  int doDisplay=0; /* Current movie is displaying on screen */
  long seekValue=0;/* holds value before it is put in vid_stream */
  int  owncmFlag=0;   /* holds value before it is put in xinfo  */
  BOOLEAN firstRead=FALSE;
  int ppm_width = -1,  ppm_height = -1, ppm_modulus = -1;

  mark = 1;
  argc--;

  input = (FILE **) malloc(NUMMOVIES*sizeof(FILE *));
  inputName = (char **) malloc(NUMMOVIES *sizeof(char *));
  theStream = (VidStream **) malloc(NUMMOVIES *sizeof(VidStream *));
  curVidStream = (VidStream **) malloc(NUMMOVIES *sizeof(VidStream *));

  for (i = 0; i < NUMMOVIES; i++) {
     input[i] = NULL;
     inputName[i] = "stdin";
     theStream[i] = NULL;
     curVidStream[i] = NULL;
     xinfo[i].hints.x = -1;
     xinfo[i].hints.y = -1;
     xinfo[i].ExistingWindow = 0;
  }
  name = (char *) "";


#ifndef DISABLE_DITHER
#ifndef DEFAULT_ORDERED_DITHER
  xinfo[0].ditherType = FULL_COLOR_DITHER;
#else
  xinfo[0].ditherType = ORDERED_DITHER;
#endif
#endif

  LUM_RANGE = 8;
  CR_RANGE = CB_RANGE = 4;
  noDisplayFlag = 0;

#ifdef SH_MEM
  shmemFlag = 1;
#endif



///////////////////////////////////////////////////
//we open the file here

  char * argvev=*argvv;

  xinfo[0].ditherType = FULL_COLOR_DITHER;

  input[numInput]=fopen(argvev, "r");


  if (input[numInput] == NULL) {
    fprintf(stderr, "Could not open file");
    return;
  }

///////////////////////////////////////////////////
// file has been opened


  fprintf(stderr, ">>we made it here... opened clip");

  inputName[numInput++] = argvev;


  lum_values = (int *) malloc(LUM_RANGE*sizeof(int));
  cr_values = (int *) malloc(CR_RANGE*sizeof(int));
  cb_values = (int *) malloc(CB_RANGE*sizeof(int));

  signal(SIGINT, int_handler);
#ifndef DEBUG
  signal(SIGSEGV, bad_handler);
  signal(SIGBUS,  bad_handler);
#endif
  if ((startFrame != -1) && (endFrame != -1) &&
      (endFrame < startFrame)) {
      return;
  }

  init_tables();
  for (i = 0;  i < numInput;  i++) {
    xinfo[i].owncmFlag = owncmFlag;
    xinfo[i].display = NULL;       /* xinfo.ximage is set to null later */
    if (xinfo[i].hints.x == -1) {
      xinfo[i].hints.x = 200;
      xinfo[i].hints.y = 300;
    }
    xinfo[i].hints.width = 150;
    xinfo[i].hints.height = 150;
    xinfo[i].visual = NULL;
    xinfo[i].name = inputName[i];
    xinfo[i].cmap = 0;
    xinfo[i].gc = 0;
  }

    InitColorDisplay(name, &xinfo[0]);
    InitColorDither(xinfo[0].depth>=24);


#ifdef SH_MEM
    if (shmemFlag && (xinfo[0].display != NULL)) {
      if (!XShmQueryExtension(xinfo[0].display)) {
        shmemFlag = 0;
        if (!quietFlag) {
          fprintf(stderr, "Shared memory not supported\n");
          fprintf(stderr, "Reverting to normal Xlib.\n");
        }
      }
    }
#endif

  InitCrop();

  y=300;
  largy=0;
  for (i=0;i<numInput;i++) {
    doDisplay=!noDisplayFlag;
#ifndef DISABLE_DITHER
    if ((xinfo[i].ditherType == NO_DITHER) ||
        (xinfo[i].ditherType == PPM_DITHER))
       doDisplay = FALSE;
#endif
    lastStream = i-1;
    while ((lastStream>=0) && (theStream[lastStream]==NULL)) {
       lastStream--;
    }
    if ((i != 0) && doDisplay) {
       if (lastStream > -1) {
         xinfo[i].hints.x =
            xinfo[lastStream].hints.x+10 + theStream[lastStream]->h_size;
         if (theStream[lastStream]->v_size>largy)
	   largy = theStream[lastStream]->v_size;
         if (xinfo[i].hints.x > DisplayWidth(xinfo[firstStream].display,
			       XDefaultScreen(xinfo[firstStream].display)) -80) {

		y += largy + 30;
		largy = 0;
		xinfo[i].hints.x = 0;
         }
         xinfo[i].hints.y = y;
         xinfo[i].visual = xinfo[firstStream].visual;
         xinfo[i].cmap = xinfo[firstStream].cmap;
         xinfo[i].gc = xinfo[firstStream].gc;
       }
       xinfo[i].display = xinfo[0].display;
       xinfo[i].depth = xinfo[0].depth;
       xinfo[i].ditherType = xinfo[0].ditherType;
       InitColorDisplay(name, &xinfo[i]);
    }
    curVidStream[i] = theStream[i] = NewVidStream((unsigned int) BUF_LENGTH);
    theStream[i]->ppm_width = ppm_width;
    theStream[i]->ppm_height = ppm_height;
    theStream[i]->ppm_modulus = ppm_modulus;
    theStream[i]->input = input[i];
    theStream[i]->seekValue = seekValue;
    theStream[i]->filename = inputName[i];
    theStream[i]->ditherType = xinfo[i].ditherType;
    theStream[i]->matched_depth = xinfo[i].depth;
    mark = quietFlag;
    quietFlag=1;
    if (mpegVidRsrc(0, theStream[i], 1, &xinfo[i])==NULL) {
       if (doDisplay) {
         XDestroyWindow(xinfo[i].display, xinfo[i].window);
       }	
       /* stream has already been destroyed */
       curVidStream[i] = theStream[i]=NULL;
       fprintf(stderr, "Skipping movie %d, \"%s\" - not an MPEG stream\n",
	  i, inputName[i]);
       fclose(input[i]);
       if (i+1 == numInput) numInput--;
    } else if (firstStream == -1) firstStream=i;
    quietFlag = mark;

#ifndef DISABLE_DITHER
    if (IS_2x2_DITHER(xinfo[i].ditherType)) {
      mult = 2;
    }
    else {
      mult = 1;
    }
#else
    mult = 1;
#endif

    if (doDisplay && (theStream[i]!=NULL)) {
      ResizeDisplay((unsigned int) theStream[i]->h_size* mult,
                    (unsigned int) theStream[i]->v_size* mult,
		  &xinfo[i]);
    }
  }

  if (numInput > 1) {
    loopFlag = TRUE;
    framerate = 0;
  }

  if (xinfo[0].display == NULL) {
    ControlShow = CTRLBAR_NONE;  /* no display => no controls */
  }

  if (ControlShow != CTRLBAR_NONE) {
    MakeControlBar(&xinfo[0]);
    ControlBar(theStream, xinfo, numInput);
  }
  for (i = 0; i < numInput; i++) {
     if (theStream[i] != NULL) theStream[i]->realTimeStart = ReadSysClock();
  }



  if (ControlShow == CTRLBAR_NONE) {
    while (TRUE) {
      for (i=0;i < numInput; i++) {
        while (theStream[i]->film_has_ended != TRUE) {
          mpegVidRsrc(0, theStream[i], 0, &xinfo[i]);
        }
        if (loopFlag) {
          rewind(theStream[i]->input);
          ResetVidStream(theStream[i]); /* Reinitialize vid_stream pointers */
          if (theStream[i]->seekValue < 0) {
            theStream[i]->seekValue = 0 - theStream[i]->seekValue;
          }
          mpegVidRsrc(0, theStream[i], 1, &xinfo[i]); /* Process start codes */
        } else if (doDisplay) break;
        else goto done;
      }
     }
   }
  else {
    ControlLoop(theStream, xinfo, numInput);
  }

done:
fprintf(stderr, ">>we made it here... cleaning up");
  mark=0;
  for (i=0;i < numInput; i++) {
    DestroyVidStream(theStream[i], &xinfo[i]);
    if ((xinfo[i].display != NULL) && !mark) {
      XCloseDisplay(xinfo[i].display);
      mark=1;
    }
  }

  fprintf(stderr, ">>we made it here... finished playing");
  //delete xinfo[NUMMOVIES];
//XCloseDisplay(xinfo[i].display);
  return;


}


/*
 *--------------------------------------------------------------
 *
 * usage --
 *
 *        Print mpeg_play usage
 *
 * Results:
 *        None.
 *
 * Side effects:
 *        exits with a return value -1
 *
 *--------------------------------------------------------------
 */
 /*
void
usage(s)
char *s;        // program name
{
    fprintf(stderr, "Usage:\n");
#ifndef NOCONTROLS
    fprintf(stderr, "mpeg_play [options] [filename]\n");
#else
    fprintf(stderr, "mpeg_play [[options] [filename]]  [[options] [filename]]  [..]\n");
#endif
    fprintf(stderr, "Options :\n");
   fprintf(stderr, "      [-display X_display]\t[-no_display]\n");
#ifndef DISABLE_DITHER
    fprintf(stderr, "      [-dither {ordered|ordered2|mbordered|fs4|fs2|fs2fast|hybrid|\n");
    fprintf(stderr, "                hybrid2|2x2|gray|gray256|color|color2|none|mono|threshold|ppm|\n");
    fprintf(stderr, "                gray2|gray256x2}]\n");
#endif
    fprintf(stderr, "      [-loop]\n");
    fprintf(stderr, "      [-start frame_num]\t[-end frame_num]\t[-seek file_offset]\n");
    fprintf(stderr, "      [-gamma gamma_correction_value]\t[-chroma chroma_correction_value]\n");
    fprintf(stderr, "      [-framerate num_frames_per_sec]  (0 means as fast as possible)\n");
    fprintf(stderr, "      [-position +x+y]\t[-quality {on|off}] (compiled default is ");
#ifdef QUALITY
    fprintf(stderr, "ON)\n");
#else
    fprintf(stderr, "OFF)\n");
#endif
#ifdef QUIET
    fprintf(stderr, "      [-noisy] (turns on all program output)\n");
#else
    fprintf(stderr, "      [-quiet] (turns off all program output)\n");
#endif
#ifndef NOCONTROLS
    fprintf(stderr, "      [-controls {on|off|none}] (default is on)\n");
#endif
    fprintf(stderr, "      [-?]\t[-help] for help (this message)\n");
    fprintf(stderr, "Rare options:\n");
    fprintf(stderr, "      [-nob]\t[-nop]\n");
#ifdef ANALYSIS
    fprintf(stderr, "      [-eachstat]\t[-owncm]\t[-shmem_off]\n");
#else
    fprintf(stderr, "      [-owncm]\t[-shmem_off]\n");
#endif
    fprintf(stderr, "      [-l_range num]\t[-cr_range num]\t[-cb_range num]\n");
//    fprintf(stderr, "      [-xid xid]\n");
#ifdef DCPREC
    fprintf(stderr, "      [-dc {8|9|10|11}] (defaults to 8)\n");
#endif
    fprintf(stderr, "      with -dither ppm:\n");
    fprintf(stderr, "          [-ppmwidth num]\t[-ppmheight num]\t[-ppmskip num]\n");
    exit (-1);
}
*/


/*
 *--------------------------------------------------------------
 *
 * DoDitherImage --
 *
 *      Called when image needs to be dithered. Selects correct
 *      dither routine based on info in xinfo[0].ditherType.
 *
 * Results:
 *        None.
 *
 * Side effects:
 *        None.
 *
 *--------------------------------------------------------------
 */

void
DoDitherImage(vid_stream)
VidStream *vid_stream;
{
 unsigned char *l=vid_stream->current->luminance,
               *Cr=vid_stream->current->Cr,
               *Cb=vid_stream->current->Cb,
               *disp=vid_stream->current->display;
 int h=(int) vid_stream->mb_height * 16;
 int w=(int) vid_stream->mb_width * 16;
 int ditherType=vid_stream->ditherType;
 int matched_depth=vid_stream->matched_depth;

#ifndef DISABLE_DITHER
  switch(ditherType) {
  case HYBRID_DITHER:
    HybridDitherImage(l, Cr, Cb, disp, h, w);
    break;

  case HYBRID2_DITHER:
    HybridErrorDitherImage(l, Cr, Cb, disp, h, w);
    break;

  case FS2FAST_DITHER:
    FS2FastDitherImage(l, Cr, Cb, disp, h, w);
    break;

  case FS2_DITHER:
    FS2DitherImage(l, Cr, Cb, disp, h, w);
    break;

  case FS4_DITHER:
    FS4DitherImage(l, Cr, Cb, disp, h, w);
    break;

  case Twox2_DITHER:
    Twox2DitherImage(l, Cr, Cb, disp, h, w);
    break;

  case FULL_COLOR2_DITHER:
    if (matched_depth >= 24)
      Twox2Color32DitherImage(l, Cr, Cb, disp, h, w);
    else
      Twox2Color16DitherImage(l, Cr, Cb, disp, h, w);
    break;
  case FULL_COLOR_DITHER:
    if (matched_depth >= 24)
#endif

      Color32DitherImage(l, Cr, Cb, disp, h, w);

#ifndef DISABLE_DITHER
    else
      Color16DitherImage(l, Cr, Cb, disp, h, w);
    break;

  case GRAY_DITHER:
  case GRAY256_DITHER:
    if (matched_depth == 8)
      GrayDitherImage(l, Cr, Cb, disp, h, w);
    else if (matched_depth == 16)
      Gray16DitherImage(l, Cr, Cb, disp, h, w);
    else if (matched_depth == 32 || matched_depth == 24)
      Gray32DitherImage(l, Cr, Cb, disp, h, w);
    break;

  case GRAY2_DITHER:
  case GRAY2562_DITHER:
    if (matched_depth == 8)
      Gray2DitherImage(l, Cr, Cb, disp, h, w);
    else if (matched_depth == 16)
      Gray216DitherImage(l, Cr, Cb, disp, h, w);
    else if (matched_depth == 32 || matched_depth == 24)
      Gray232DitherImage(l, Cr, Cb, disp, h, w);
    break;

  case NO_DITHER:
    break;

  case PPM_DITHER:
    Color32DitherImage(l, Cr, Cb, disp, h, w);
    break;

  case ORDERED_DITHER:
    OrderedDitherImage(l, Cr, Cb, disp, h, w);
    break;

  case MONO_DITHER:
    MonoDitherImage(l, Cr, Cb, disp, h, w);
    break;

  case MONO_THRESHOLD:
    MonoThresholdImage(l, Cr, Cb, disp, h, w);
    break;

  case ORDERED2_DITHER:
    Ordered2DitherImage(l, Cr, Cb, disp, h, w);
    break;

  case MBORDERED_DITHER:
    MBOrderedDitherImage(l, Cr, Cb, disp, h, w, vid_stream->ditherFlags);
    break;
  }
#endif
}
