/*******************************************************************************
**
** The header file for the Jahshaka network module header
** The Jahshaka Project
** Copyright (C) 2000-2003 Visual Media, Inc.
** Released under the GNU General Public License
**
*******************************************************************************/
#ifndef NETWORKING_H
#define NETWORKING_H

#include <interfaceheaders.h>
#include <httpd.h>
#include <qsocketdevice.h>
#include <videodata.h>
#include <qfile.h>
#include <qwidget.h>

#include <qhostaddress.h>

extern QString JahBasePath;

class JahNet :  public  QWidget {
  Q_OBJECT

public:

JahNet::JahNet( QWidget* parent, const char* name, QHBox* controls, QHBox* controller, int* globalclipnumber ) {


 buildInterface(controls);         //builds the network interfaces
 buildController(controller);      //  draw slider interface

 //start up the servers here for incoming connections on port 4242/4243  and connect the slots
 //server = new SimpleServer(4242,this);
 server      = new SimpleServer(4242,this);
 imageserver = new SimpleServer(4243,this);

 connect( server, SIGNAL(newConnect()), SLOT(newConnect()) );
 connect( server, SIGNAL(copyText(QString )), SLOT(echoText(QString  )) );

 //for reciprocating chat connections
 connect( server, SIGNAL(recipConnect(QHostAddress )), SLOT(recirprocateConnection(QHostAddress  )) );

 //server is created here so we have to hook it up to the parent
 //connect ( server,     SIGNAL(sendupdateDesktop(videoData)), this, SLOT( updatetheDesktop(videoData)) );
 //connect ( server,     SIGNAL(updateDesktop(videoData)), parent, SLOT( addImageToDesktop(videoData)) );

 //hook up the client here - one per conneection IMPORTANT we need to create a global to track connections
 connect(  connectbutton,   SIGNAL(clicked()), SLOT(createNewClient()) );
 connect(  sendbutton,      SIGNAL(clicked()), SLOT(sendToServer())    );
 connect(  texteditbox,     SIGNAL(returnPressed()), SLOT(sendToServer())    );

 connect(  sendimagebutton, SIGNAL(clicked()), SLOT(GrabDesktopClip()) );
 connect(  closebutton,     SIGNAL(clicked()), SLOT(closeConnection()) );

 initializeObjects();

 clipnumber = globalclipnumber;

};

  ~JahNet() { }


QWidget* parentobject;

    int*  clipnumber;

bool NetStatus;

    void buildInterface( QHBox* qtinterface);
    void buildController( QHBox* controller);
    void initializeObjects();

void ServerInfo();

public slots:

  void newConnect();
  void echoText(QString);

  void clientinterface();
  void serverinterface();

  void createNewClient();
  void createNewClient(QHostAddress);

  void  addClip(videoData,int);
  void  GrabDesktopClip();

  void recirprocateConnection(QHostAddress addy);

  void updatetheDesktop( videoData );

private slots:

  void closeConnection();
  void sendToServer();
  void sendMediaToServer(videoData,int);
  void socketReadyRead();
  void socketConnected();
  void socketConnectionClosed();
  void socketClosed();
  void socketError( int e );

signals:

  void GrabClip();
  void updateDesktop( videoData );

protected:

private:

  SimpleServer *server ;
  SimpleServer *imageserver ;

  QSocket *socket; QSocket *datasocket;
  QSocket *recipsocket;

  QTextView *infoText;
  QTextView *infoText2;

  QHBox *hb;
  QLineEdit *inputText;
  QPushButton *send;
	QPushButton *close;

  QVBox *netbox;
  QHBox*        ClientBox;
  QVBox*        ServerBox;

  QFrame* clientBox2;
  QFrame* clientBox1;

  QFrame* clientframe;
  QLabel* ClientLabel;
  QLabel* serverlabel;
  QLineEdit* serveredit;


  QLineEdit* texteditbox;
  QPushButton* sendbutton, * sendimagebutton, * connectbutton,* closebutton;
  QTextView* clienttextfeedback;


    ////////////////////////////////////////////////////////////////////////////
    //interface variables
    QFrame*      LayersFrame;

    QPushButton*  AddClient;                QPushButton*  AddServer;
    QPushButton*  AddGnutella;                QPushButton*  AddSfs;

	  QWidgetStack*  NetInterface;



    QPushButton*   ResetButton;             QPushButton*   SaveButton;    QPushButton*   LoadButton;
	  QCheckBox*     SmoothSelect;            QCheckBox*     BlurSelect;    QCheckBox*     TextSelect;

    QFrame*        ControlFrame_2;
    QWidgetStack*  fxoptionals;
    QHBox*         fxoptionals_clipone;     QHBox*         fxoptionals_cliptwo;
    QHBox*         fxoptionals_object;      QHBox*         fxoptionals_world;
    QHBox*         fxoptionals_text;

    QFrame*        ControlFrame1;
    QLabel*        RotateText_1;
    QLabel*        TranslateText_1;
    QLabel*        ScaleText_1;

};

#endif























