/* flip - pageflipper for TIFF animations
 * Copyright (C) 1999 Mark B. Allan (mba@reptilelabour.com)
 *
 * "flip" is free software; you can redistribute it and/or use
 * it and/or modify it under the terms of the "Artistic License"
 */

#include "MainWindow.h"

#include <stdio.h>

#include <qapplication.h>
#include <qkeycode.h>
#include <qtooltip.h>
#include <qiconset.h>
#include <qmessagebox.h>

#include "Config.h"
#include "InputLCD.h"
#include "FileDialog.h"
#include "TiffFlip.h"
#include "InputLCD.h"
#include "FileDialog.h"
#include "FlipPreferences.h"

#if defined(Q_WS_X11)
#include "RenderMPEGDialog.h"
#endif

#include "qstring.h"

#include "Pixmaps/f_clr_ff.xpm"
#include "Pixmaps/f_clr_ping.xpm"
#include "Pixmaps/f_clr_play.xpm"
#include "Pixmaps/f_clr_rew.xpm"
#include "Pixmaps/f_clr_stop.xpm"
#include "Pixmaps/f_gry_ff.xpm"
#include "Pixmaps/f_gry_ping.xpm"
#include "Pixmaps/f_gry_play.xpm"
#include "Pixmaps/f_gry_rew.xpm"
#include "Pixmaps/f_gry_stop.xpm"

extern int fontM;
extern int fontS;

double MainWindow::fpsVal[NumFPS];

//==============================================================================
MainWindow::MainWindow( QWidget *parent, const char* name, QHBox* control, QHBox* controler)
	: QWidget (parent, name)
{



  // we set up the main frame here
  controls = new QFrame( control, "EffectsFrame" );    //control
  controls->setGeometry( QRect( 0, 0, 825, 140 ) );


  Config *cfg = Config::getInstance();
  //setCaption(APP_NAME);
  	
  //-- we need to set this before creating FlipPreferences
  fpsVal[F60] = 60.0;
  fpsVal[F48] = 48.0;
  fpsVal[F30] = 30.0;
  fpsVal[F24] = 24.0;
  fpsVal[F20] = 20.0;
  fpsVal[F15] = 15.0;
  fpsVal[F12] = 12.0;
  fpsVal[F10] = 10.0;
	
	minFrameInput = maxFrameInput = 0;
	curFrame	=  0;
	pingpong	=  frameStep = 1;
	
	//-- create TiffFlip object

  //////////////////////////////////////////

  //ok this works if you set the size of the qframe first...
  //which can be implemented as we know the clip size before and after resizing...

/*
  //this allows us to open the player in a separate window
  //as a qdialog for now

  playerdialog = new QDialog( parent, "Player", false );
  playerdialog->setGeometry ( 0, 0, 720, 486 );
  playerdialog->setFixedSize(720,486);
  playerdialog->setPaletteBackgroundColor ( QColor( 0, 0, 0 ) );

  mainworldLayout = new QHBoxLayout( playerdialog, 11, 6, "mainworldLayout");
  player = new QFrame( playerdialog, "player" );
  mainworldLayout->addWidget( player );

  playerdialog->show();

*/
  mainworldLayout = new QHBoxLayout( parent, 11, 6, "mainworldLayout");
  player = new QFrame( parent, "player" );
  mainworldLayout->addWidget( player );


  //this creates the player
 	tiffFlip = new TiffFlip(player, 0);

  //coonect out frame resizing routine now
  connect(tiffFlip, SIGNAL(resizePlayer(mediaSize)), this, SLOT(ResizeDisplay(mediaSize)) );
	
	//-- create file dialog
	fileDialog = new FileDialog(controls);
	fileDialog->hide();
	
	//-- create preferences dialog
	preferences = new FlipPreferences(controls);
	preferences->hide();
	
	mpegID = 999;
	//-- menuBar
	QFont::setDefaultFont(QFont("Helvetica", 10, QFont::Bold, false));
	filePopup = new QPopupMenu(controls);
	//filePopup->insertItem ("&load",			this,	SLOT(LoadSequence()), CTRL+Key_O);
	filePopup->insertItem ("&clear",		this,	SLOT(Clear()), CTRL+Key_C);
 	
	utilPopup = new QPopupMenu;
	utilPopup->insertItem( "encode &mpeg...",  this, SLOT(renderMPEG()), CTRL+Key_M, mpegID );
	utilPopup->insertItem( "&play mpeg...",  this, SLOT(playMPEG()) );

#if defined(Q_WS_X11)
	utilPopup->setItemEnabled(mpegID, true);
#endif

	fpsPopup = new QPopupMenu;
	fpsID[F60]	= fpsPopup->insertItem( " 60   ", this, SLOT(Fps60()) );
	fpsID[F30]  = fpsPopup->insertItem( " 30   ", this, SLOT(Fps30()) );
	fpsID[F24]  = fpsPopup->insertItem( " 24   ", this, SLOT(Fps24()) );
	fpsID[F15]  = fpsPopup->insertItem( " 15   ", this, SLOT(Fps15()) );

	fpsPopup->setCheckable(true);
	FPS curFPS = cfg->flipFPS;
	fpsPopup->setItemChecked(fpsID[(int)curFPS], true);
	currentSpeed = fpsVal[curFPS];
	
	stepPopup = new QPopupMenu;
	step1_ID	= stepPopup->insertItem( "1    ", this, SLOT(Step1()) );
	step2_ID	= stepPopup->insertItem( "2    ", this, SLOT(Step2()) );
	step3_ID	= stepPopup->insertItem( "3    ", this, SLOT(Step3()) );
	step5_ID	= stepPopup->insertItem( "5    ", this, SLOT(Step5()) );
	stepPopup->setCheckable(true);
	stepPopup->setItemChecked(step1_ID, true);
	
	zoomPopup = new QPopupMenu;
	zoom1_ID	= zoomPopup->insertItem( "1x    ", this, SLOT(Zoom1()) );
	zoom2_ID	= zoomPopup->insertItem( "2x    ", this, SLOT(Zoom2()) );
	zoomPopup->setItemChecked(zoom1_ID, true);
	zoomPopup->setCheckable(true);
	zoomPopup->setItemChecked(zoom1_ID, true);
		
	menuBar = new QMenuBar (controls);

//mpeg utilities dont work on windowz yet
#if defined(Q_WS_X11)
	menuBar->insertItem ("&util", utilPopup);
#endif

	menuBar->insertItem ("&zoom", zoomPopup);
	menuBar->insertItem ("&step", stepPopup);
	menuBar->insertItem ("f&ps", fpsPopup);
	menuBar->insertSeparator();
	QFont::setDefaultFont(QFont("Helvetica", 12, QFont::Bold, false));

	//-- status line
	statusline	= new QLabel(controls);
	statusline->setText("no frames loaded.");
	statusline->setFrameStyle(QFrame::Sunken | QFrame::Box);
	statusline->setAlignment( AlignCenter );
	statusline->setFixedHeight(25);
	statusline->setFont ( QFont("Helvetica", 12, QFont::Bold) );
	
	frameNum	= new QLabel(controls);
	frameNum->setText("");
	frameNum->setFrameStyle(QFrame::Sunken | QFrame::Box);
	frameNum->setAlignment( AlignCenter );
	frameNum->setFixedSize(50, 25);
	
	actualFps	= new QLabel(controls);
	actualFps->setText(" ");
	actualFps->setFrameStyle(QFrame::Sunken | QFrame::Box);
	actualFps->setAlignment( AlignCenter );
	actualFps->setFixedSize(50, 25);
	
	//-- frame jog
	frameScroll = new QScrollBar (QScrollBar::Horizontal, controls);
	frameScroll->setRange(0, tiffFlip->GetNumFrames() );
	frameScroll->setFixedHeight(25);

	//-- Buttons

  // we set up the sub - frame here
  controlers = new QFrame( controler , "Frame6" );
  controlers->setGeometry( QRect( 0, 0, 820, 35 ) );

	//-- LCD Inputs
	minFrameInput = new InputLCD(controlers);
	minFrameInput->setFixedSize(50, 25);
	
	maxFrameInput = new InputLCD(controlers);
	maxFrameInput->setFixedSize(50, 25);


	rew	= new MToolButton(controlers);
	rew->setFixedSize(32, 32);
	rew->setUsesBigPixmap(true);
	rew->setAutoRaise(true);
	
	ping	= new MToolButton(controlers);
	ping->setToggleButton(true);
	ping->setFixedSize(32, 32);
	ping->setUsesBigPixmap(true);
	ping->setAutoRaise(true);
	
	stop	= new MToolButton(controlers);
	stop->setFixedSize(32, 32);
	stop->setUsesBigPixmap(true);
	stop->setAutoRaise(true);
	
	play	= new MToolButton(controlers);
	play->setToggleButton(true);
	play->setFixedSize(32, 32);
	play->setUsesBigPixmap(true);
	play->setAutoRaise(true);
	
	ff	= new MToolButton(controlers);
	ff->setFixedSize(32, 32);
	ff->setUsesBigPixmap(true);
	ff->setAutoRaise(true);
	
	//----------------------------------Buttons
	QIconSet	*icoSet;
	icoSet = new QIconSet(QPixmap((const char**)f_gry_rew_xpm) );
	icoSet->setPixmap(QPixmap((const char**)f_clr_rew_xpm), QIconSet::Large, QIconSet::Active);
	icoSet->setPixmap(QPixmap((const char**)f_gry_rew_xpm), QIconSet::Large, QIconSet::Disabled);
	rew->setIconSet(*icoSet);
	delete icoSet;
	icoSet = new QIconSet(QPixmap((const char**)f_gry_ping_xpm) );
	icoSet->setPixmap(QPixmap((const char**)f_clr_ping_xpm), QIconSet::Large, QIconSet::Active);
	icoSet->setPixmap(QPixmap((const char**)f_gry_ping_xpm), QIconSet::Large, QIconSet::Disabled);
	ping->setIconSet(*icoSet);
	delete icoSet;
	icoSet = new QIconSet(QPixmap((const char**)f_gry_stop_xpm) );
	icoSet->setPixmap(QPixmap((const char**)f_clr_stop_xpm), QIconSet::Large, QIconSet::Active);
	icoSet->setPixmap(QPixmap((const char**)f_gry_stop_xpm), QIconSet::Large, QIconSet::Disabled);
	stop->setIconSet(*icoSet);
	delete icoSet;
	icoSet = new QIconSet(QPixmap((const char**)f_gry_play_xpm) );
	icoSet->setPixmap(QPixmap((const char**)f_clr_play_xpm), QIconSet::Large, QIconSet::Active);
	icoSet->setPixmap(QPixmap((const char**)f_gry_play_xpm), QIconSet::Large, QIconSet::Disabled);
	play->setIconSet(*icoSet);
	delete icoSet;
	icoSet = new QIconSet(QPixmap((const char**)f_gry_ff_xpm) );
	icoSet->setPixmap(QPixmap((const char**)f_clr_ff_xpm), QIconSet::Large, QIconSet::Active);
	icoSet->setPixmap(QPixmap((const char**)f_gry_ff_xpm), QIconSet::Large, QIconSet::Disabled);
	ff->setIconSet(*icoSet);
	delete icoSet;
	
	//----------------------------------TIMER
	timer = new QTimer(this);
	
	//----------------------------------Connect
	QObject::connect(tiffFlip, SIGNAL(LoadingFrame(int)), this, SLOT(UpdateLoadStatus(int)) );
	QObject::connect(tiffFlip, SIGNAL(Done(int, bool)), this, SLOT(LoadSequenceCompleted(int, bool)) );
	
	QObject::connect(frameScroll, SIGNAL(valueChanged(int)), this, SLOT(SetFrameNumber(int)) );

	QObject::connect(play, SIGNAL(clicked()), this, SLOT( LoopForward())	);
	QObject::connect(ping , SIGNAL(clicked()), this, SLOT( PingPong())	);
	QObject::connect(stop , SIGNAL(clicked()), this, SLOT( Stop())		);
	QObject::connect(ff   , SIGNAL(clicked()), this, SLOT( FastForward())	);
	QObject::connect(rew  , SIGNAL(clicked()), this, SLOT( Rewind())		);
	
	QObject::connect(minFrameInput, SIGNAL(valueChanged(int)), this, SLOT(SetMinFrame(int)) );
	QObject::connect(maxFrameInput, SIGNAL(valueChanged(int)), this, SLOT(SetMaxFrame(int)) );
	
	QObject::connect(timer, SIGNAL(timeout()), this, SLOT(Action()) );
	
	//----------------------------------Layout
	topLayout	= new QVBoxLayout(controls, 5, 1);
	status		= new QHBoxLayout(2);
	buttons		= new QHBoxLayout(2);
	middle		= new QHBoxLayout(controlers, 5);
	midR		= new QVBoxLayout(0);
	midM		= new QVBoxLayout(0);
	midL		= new QVBoxLayout(0);
	
  topLayout->addSpacing(menuBar->heightForWidth(400));
  topLayout->addStretch(1);
  topLayout->addWidget(frameScroll, 0);
  //topLayout->addLayout(middle, 0);
  middle->addLayout(midR);
  midR->addWidget(minFrameInput);
  midR->addStretch(1);
  middle->addStretch(1);
  middle->addLayout(midM);
  midM->addSpacing(2);
  midM->addLayout(buttons, 0);
  buttons->addSpacing(5);
  buttons->addStretch();	
  buttons->addWidget(rew);	
  buttons->addWidget(ping);	
  buttons->addWidget(stop);
  buttons->addWidget(play);	
  buttons->addWidget(ff);
  buttons->addStretch();	
  buttons->addSpacing(5);
  middle->addStretch(1);
  middle->addLayout(midL);
  midL->addWidget(maxFrameInput);
  midL->addStretch(1);
  topLayout->addSpacing(2);
  topLayout->addLayout(status, 0);
  status->addWidget(frameNum);
  status->addWidget(statusline);
  status->addWidget(actualFps);

  resize(10,10);
}

//--------------------------------------------------------------------
MainWindow::~MainWindow()
{

/*
//if opening the player in its own window we need to close it out on exit

Stop();
playerdialog->close();
*/

}

//--------------------------------------------------------------------
void	MainWindow::DUMMYslot()
{
}

//--------------------------------------------------------------------
void	MainWindow::quit()
{
	Config::commit();
	qApp->quit();
}

//--------------------------------------------------------------------
void	MainWindow::PreferencesDialog()
{
	Config *cfg = Config::getInstance();
	preferences->exec();
	if (preferences->result() == QDialog::Accepted)
	{
		fileDialog->setNumHistory(cfg->fileHist);
		filePopup->setItemEnabled(mpegID, Config::hasMpeg());		
	}
}

//--------------------------------------------------------------------
void	MainWindow::Fps60()	{	SetFPS(F60);	}
void	MainWindow::Fps48() {	SetFPS(F48);	}
void	MainWindow::Fps30() {	SetFPS(F30);	}
void	MainWindow::Fps24() {	SetFPS(F24);	}
void	MainWindow::Fps20() {	SetFPS(F20);	}
void	MainWindow::Fps15() {	SetFPS(F15);	}
void	MainWindow::Fps12() {	SetFPS(F12);	}
void	MainWindow::Fps10()	{	SetFPS(F10);	}
//--------------------------------------------------------------------
void	MainWindow::SetFPS(FPS index)
{
	for(int i = 0; i < NumFPS; i++)
		fpsPopup->setItemChecked(fpsID[i], false);
		
	fpsPopup->setItemChecked(fpsID[index], true);
	currentSpeed = fpsVal[index];
	
	if (timer->isActive())
	{
		timer->changeInterval((int)(1000.0/currentSpeed));
	}
}
	
//--------------------------------------------------------------------
void	MainWindow::Step1()	{	SetStep(1);	}
void	MainWindow::Step2() {	SetStep(2);	}
void	MainWindow::Step3() {	SetStep(3);	}
void	MainWindow::Step4() {	SetStep(4);	}
void	MainWindow::Step5() {	SetStep(5);	}
void	MainWindow::Step6() {	SetStep(6);	}
void	MainWindow::SetStep(int index)
{
	stepPopup->setItemChecked(step1_ID, false);
	stepPopup->setItemChecked(step2_ID, false);
	stepPopup->setItemChecked(step3_ID, false);
	stepPopup->setItemChecked(step4_ID, false);
	stepPopup->setItemChecked(step5_ID, false);
	stepPopup->setItemChecked(step6_ID, false);
	switch(index)
	{
		case 1:	stepPopup->setItemChecked(step1_ID, true);
			frameStep = 1;
			break;
		case 2:	stepPopup->setItemChecked(step2_ID, true);
			frameStep = 2;
			break;
		case 3:	stepPopup->setItemChecked(step3_ID, true);
			frameStep = 3;
			break;
		case 4:	stepPopup->setItemChecked(step4_ID, true);
			frameStep = 4;
			break;
		case 5:	stepPopup->setItemChecked(step5_ID, true);
			frameStep = 5;
			break;
		case 6:	stepPopup->setItemChecked(step6_ID, true);
			frameStep = 6;
			break;
	}
}

//--------------------------------------------------------------------
void	MainWindow::Zoom1() {	SetZoom(1);	}
void	MainWindow::Zoom2() {	SetZoom(2);	}
void	MainWindow::SetZoom(int index)
{	
	zoomPopup->setItemChecked(zoom1_ID, false);
	zoomPopup->setItemChecked(zoom2_ID, false);
	switch(index)
	{
		case 1:
			zoomPopup->setItemChecked(zoom1_ID, true);
			tiffFlip->SetDisplayZoom(index);
			break;
		case 2:
			zoomPopup->setItemChecked(zoom2_ID, true);
			break;
	}
	tiffFlip->SetDisplayZoom(index);
}

void	MainWindow::ResizeDisplay(mediaSize thesize)
{
  //debug("resizing the image space");
  player->setMinimumSize( QSize( thesize.width, thesize.height ) );
  player->setMaximumSize( QSize( thesize.width, thesize.height ) );
}

//--------------------------------------------------------------------
void	MainWindow::SetFrameNumber(int i)
{
	// This slot tells tiffFlip to draw a certain frame
	// and updates the frame number label	
	curFrame = i;
	//frameNumString.setNum(curFrame-startFrame+1);
	frameNumString.setNum(curFrame-minFrame+1);
	frameNum->setText(frameNumString);
	tiffFlip->SetFrame(curFrame-startFrame);
}
	
//--------------------------------------------------------------------
void	MainWindow::Clear()
{
	Stop();
	frameScroll->setRange	(0, 0);
	minFrameInput->setActive(false);
	maxFrameInput->setActive(false);
	tiffFlip->Reset();
  debug("got here");
	statusline->setText("no frames loaded.");
	frameNum->setText("");
	actualFps->setText(" ");
}

//--------------------------------------------------------------------

void	MainWindow::ClearOut()
{
  Clear();
}

void	MainWindow::LoadSequence()
{
	Stop();
	if(tiffFlip->isLoading())
		tiffFlip->abortLoad();
	fileDialog->exec();
	
	if (fileDialog->result() == QDialog::Accepted)
	{
		frameScroll->setRange	(0, 0);
		minFrameInput->setActive(false);
		maxFrameInput->setActive(false);
		tiffFlip->Reset();
		statusline->setText("no frames loaded.");
		frameNum->setText("");
		actualFps->setText(" ");
		
		QObject::connect(stop, SIGNAL(clicked()), tiffFlip, SLOT(abortLoad()) );
		frameScroll->setRange	(0, 0);
		minFrameInput->display(" ");
		maxFrameInput->display(" ");

    QString temp;

    debug("The loader information sent is");
    debug (temp.operator=(fileDialog->getFileName()));
    debug("MaxFrames"); debug ( temp.setNum( fileDialog->getMaxFrames()) );
    debug("LoadSize "); debug ( temp.setNum( fileDialog->getLoadSize())  );


		startFrame = tiffFlip->LoadImages(fileDialog->getFileName(), fileDialog->getMaxFrames(), fileDialog->getLoadSize());
		//LoopForward();
	}
	
}

//actually we should use this routine for single frames as well
//for less confusion...
//and we can streamline the loader... in tiffflipp
//since all the necessary data is already in the desktopclip

void	MainWindow::LoadMySequence(videoData desktopclip)
{
	Stop();
	if(tiffFlip->isLoading())
		tiffFlip->abortLoad();

	//fileDialog->exec();
	
	//if (fileDialog->result() == QDialog::Accepted)
	//{
		frameScroll->setRange	(0, 0);
		minFrameInput->setActive(false);
		maxFrameInput->setActive(false);
		tiffFlip->Reset();
		statusline->setText("no frames loaded.");
		frameNum->setText("");
		actualFps->setText(" ");
		
		QObject::connect(stop, SIGNAL(clicked()), tiffFlip, SLOT(abortLoad()) );
		frameScroll->setRange	(0, 0);
		minFrameInput->display(" ");
		maxFrameInput->display(" ");


    QString temp;

    /////////////////////////////////////////////////////////////////
    /////////////////////////////////////////////////////////////////
    /*        USE ME*/


    debug ( desktopclip.getFileName() );
    debug("MaxFrames"); debug ( temp.setNum( desktopclip.getMaxFrames() ) );

    int MaxFrames = desktopclip.getMaxFrames();

#ifdef WIN32
//theres a issue on windows with loading more than 110 frames in the ram player
//related to this command
//pixlist[0]->setOptimization(QPixmap::NoOptim);
if (MaxFrames>110) { MaxFrames=110; }
#endif

		startFrame = tiffFlip->LoadImages( //fileDialog->getFileName(),
                                       desktopclip.getFileName(),
                                       MaxFrames,
                                       //fileDialog->getLoadSize()
                                       1
    );


    //how do we solve this problem?
	//	startFrame = tiffFlip->LoadImages( //fileDialog->getFileName(),
    //                                   desktopclip.filename,
    //                                   100,
    //                                   //fileDialog->getLoadSize()
                                      // 1
    //);
		
		
	
}

///////////////////////////////////////////////////
// we call this directly to load and display single frame images

void	MainWindow::GrabImage(const QImage& newimage)
{
	Stop();
	if(tiffFlip->isLoading())
		tiffFlip->abortLoad();


		frameScroll->setRange	(0, 0);
		minFrameInput->setActive(false);
		maxFrameInput->setActive(false);
		tiffFlip->Reset();
		statusline->setText("Still loaded.");
		frameNum->setText("1");
		actualFps->setText(" ");
		
		//QObject::connect(stop, SIGNAL(clicked()), tiffFlip, SLOT(abortLoad()) );
		frameScroll->setRange	(1, 1);
		minFrameInput->display("1");
		maxFrameInput->display("1");
		//tiffFlip->GetImages(newimage, 1, 1 );
    ClearOut();
    tiffFlip->GetImages(newimage, 1, 1 );
    startFrame = 1;
	  //m.reset();
}

//--------------------------------------------------------------------
void	MainWindow::UpdateLoadStatus(int i)
{
	static char	message[128];
	sprintf(message, "loading frame %d of %d", i+1, tiffFlip->GetMaxFrames() );
	statusline->setText(message);
}
	
//--------------------------------------------------------------------
void	MainWindow::LoadSequenceCompleted(int sf, bool aborted)
{
	startFrame = sf;
	minFrame = startFrame;
	maxFrame = tiffFlip->GetNumFrames()-1+startFrame;
	frameScroll->setRange	(minFrame, maxFrame);
	tiffFlip->setCurMinFrame(minFrame);
	tiffFlip->setCurMaxFrame(maxFrame);
	
//	minFrameInput->SetValue (0);
//	minFrameInput->SetMinMaxInt	(minFrame, maxFrame-1);
//	maxFrameInput->SetMinMaxInt	(minFrame+1, maxFrame);
//	maxFrameInput->SetValue (maxFrame);
//	
	minFrameInput->setValue (minFrame);
	minFrameInput->setMinMaxInt	(minFrame, maxFrame-1);
	maxFrameInput->setMinMaxInt	(minFrame+1, maxFrame);
	maxFrameInput->setValue (maxFrame);
	
	curFrame = 0;
	frameScroll->setValue(curFrame);
	SetFrameNumber(minFrame);
	
	char message[128];
	sprintf(message, "loaded %d frames at %dx%d.", tiffFlip->GetNumFrames(), tiffFlip->GetWidth(), tiffFlip->GetHeight() );
	statusline->setText(message);
	
	QObject::disconnect(stop, SIGNAL(clicked()), tiffFlip, SLOT(abortLoad()) );
	if(!aborted)
		LoopForward();
	
}

//--------------------------------------------------------------------
void	MainWindow::SetMinFrame(int val)
{
	maxFrameInput->setMinInt( (minFrame = val) + 1 );
	frameScroll->setRange(minFrame, maxFrame);
	tiffFlip->setCurMinFrame(minFrame);
	tiffFlip->setCurMaxFrame(maxFrame);
	frameScroll->setValue(minFrame);
}
void	MainWindow::SetMaxFrame(int val)
{
	minFrameInput->setMaxInt( (maxFrame = val) - 1 );
	frameScroll->setRange(minFrame, maxFrame);
	tiffFlip->setCurMinFrame(minFrame);
	tiffFlip->setCurMaxFrame(maxFrame);
	frameScroll->setValue(maxFrame);
}

//--------------------------------------------------------------------
void	MainWindow::LoopForward()
{
	if (timer->isActive())
		timer->stop();
	play->setOn(true);
	ping->setOn(false);
	
//	gettimeofday(&oldTime, 0);
	
	playMode = LOOP_FORWARD;
	timer->start((int)(1000.0/currentSpeed));
}
//--------------------------------------------------------------------
void	MainWindow::PingPong()
{
	if (timer->isActive())
		timer->stop();
	play->setOn(false);
	ping->setOn(true);
	
//	gettimeofday(&oldTime, 0);
	
	playMode = PING_PONG;
	pingpong = -pingpong;
	timer->start((int)(1000.0/currentSpeed));
}
//--------------------------------------------------------------------
void	MainWindow::Stop()
{
	if (timer->isActive())	
		timer->stop();	
	play->setOn(false);
	ping->setOn(false);
	actualFps->setText(" ");
	pingpong = 1;
}

void	MainWindow::StopPlayer()
{
  Stop();
}



//--------------------------------------------------------------------
void	MainWindow::FastForward()
{	frameScroll->setValue( curFrame = maxFrame );	}
//--------------------------------------------------------------------
void	MainWindow::Rewind()
{	frameScroll->setValue( curFrame = minFrame );	}

//--------------------------------------------------------------------
void	MainWindow::Action()
{
	static int		counter = 1;
	static int		elapsed;
	static int		accumulated;
	static double	framesPerSec;
	static char		labelText[64];
	
	
	switch(playMode)
	{
		case LOOP_FORWARD:
			ActionLoopForward();
			break;
		case PING_PONG:
			ActionPingPong();
			break;
		default:
			warning ("MainWindow::playMode is set incorrectly.\nplayMode = %d", playMode);
			timer->stop();
	}
	
//	gettimeofday(&curTime, 0);
	elapsed = (int)curTime.tv_usec - (int)oldTime.tv_usec;
	oldTime.tv_usec = curTime.tv_usec;
	if(elapsed < 0)
		elapsed += 1000000;
	
	accumulated += elapsed;
	if (counter >= 10)
	{
		framesPerSec = 1000*1.0/((double)accumulated*0.0001);
		counter = 0;
		accumulated = 0;
		sprintf(labelText, "%3.2f", framesPerSec);
		actualFps->setText(labelText);
	}
	counter++;
}
//- - - - - - - - - - - - - - - - - - - - - - -
void	MainWindow::ActionLoopForward()
{
	curFrame += frameStep;
	if (curFrame > maxFrame)
		curFrame = minFrame;
	frameScroll->setValue(curFrame);
	qApp->processEvents();
}

void	MainWindow::ActionPingPong()
{
	curFrame += pingpong;
	if (curFrame > maxFrame)
	{
		curFrame = maxFrame;
		pingpong = -frameStep;
	}
	else if (curFrame < minFrame)
	{
		curFrame = minFrame;
		pingpong = frameStep;
	}
	frameScroll->setValue(curFrame);
	qApp->processEvents();
}

//----------------------------------------------------------


//----------------------------------------------------------
void	MainWindow::renderMPEG()
{
#if defined(Q_WS_X11)
	Stop();
	
	if(tiffFlip->GetNumFrames() <= 0)
	{
		QMessageBox::warning(this, "Jahshaka", "No Frames Loaded");
	}
	else
	{
		RenderMPEGDialog mpegDialog(*tiffFlip, this);
		mpegDialog.exec();
	}
#endif
}

//x11 only routines here
void	MainWindow::playMPEG()
{
#if defined(Q_WS_X11)
	Stop();

  debug(">>>>launching thread<<<");

    QString loadfilename = QFileDialog::getOpenFileName( JahBasePath+"/media/encoded/",  "*.*"  );
    if ( !loadfilename.isEmpty() ) {

      mythread = new MpegPlayThread(this,loadfilename);
      mythread->start();

      while (mythread->running()) {  qApp->processEvents(); }
}
#endif
}



