/* flip - pageflipper for TIFF animations
 * Copyright (C) 1999 Mark B. Allan (mba@reptilelabour.com)
 *
 * "flip" is free software; you can redistribute it and/or use
 * it and/or modify it under the terms of the "Artistic License"
 */

#include "TiffFlip.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <qapplication.h>
#include <qwidget.h>
#include <qimage.h>
#include <qmessagebox.h>
#include <qwmatrix.h>

#include "define.h"

#ifndef _WIN32
#include <tiffio.h>
#include <unistd.h>
#endif

#define SWAP_THRESH 1.5f

static char	errorMessage[256];
#ifdef _WIN32
#define snprintf _snprintf
int getpid() {return 1;};
#endif

//------------------------------------------------
TiffFlip::TiffFlip(QWidget *parent, const char* name)
	:QWidget (parent, name)
{
	pixlist		= 0; 
	numFrames	= 0;
	maxFrames	= 0;
	frame		= 0;
	width		= 0;
	height		= 0;
	origWidth	= 0;
	origHeight	= 0;
	loadZoom	= 1;
	displayZoom	= 1;
	QObject::connect(&timer, SIGNAL(timeout()), SLOT(LoadController()) );
	flipPID		= (int)getpid();
	
	QFont	font("Helvetica", 10);
	
	memLabel 	= new QLabel(this);
	memLabel->setFixedSize(80, 15);
	memLabel->move(5, 5);
	memLabel->setFont(font);
	memLabel->setAlignment(AlignCenter);
	
	swapLabel	= new QLabel(this);
	swapLabel->setFixedSize(80, 15);
	swapLabel->move(5, 25);
	swapLabel->setFont(font);
	swapLabel->setAlignment(AlignCenter);
	
	swapThresh = SWAP_THRESH;
}
	

//------------------------------------------------
TiffFlip::~TiffFlip()
{
	if (pixlist)
	{
		for (int i = 0; i < numFrames; i++)
			delete pixlist[i];
		delete [] pixlist;
	}
}

//------------------------------------------------
void TiffFlip::showMemInfo(bool status)
{
	if(status)
	{
		memLabel->show();
		swapLabel->show();
	}
	else
	{
		memLabel->hide();
		swapLabel->hide();
	}
}

//------------------------------------------------
void TiffFlip::Reset()
{
	int	i;
	if ( timer.isActive() )
		timer.stop();
	hide();
	if (pixlist)
	{
		for (i = 0; i < numFrames; i++)
		{
			delete pixlist[i];
		}
		delete [] pixlist;
	}
	pixlist		= 0;
	numFrames	= 0;
	maxFrames	= 0;
	frame		= 0;
	width		= 0;
	height		= 0;
	origWidth	= 0;
	origHeight	= 0;
	showMemInfo(true);
}

//------------------------------------------------
void TiffFlip::SetDisplayZoom(int z)
{
	displayZoom = z;

  mediaSize thesize;
  thesize.width = width * displayZoom;
  thesize.height = height * displayZoom;
  emit resizePlayer(thesize);

	setFixedSize(width*displayZoom, height*displayZoom);
	m.reset();
	m.scale((float)displayZoom, (float)displayZoom);

}

//------------------------------------------------
int TiffFlip::LoadImages(const char *startfile, int max, int z)
{
	int		i;
	int		index;
	bool	ok;
	QString tmpString(startfile);
	FILE	*check;
	
	loadZoom		= z;
	maxFrames	= max;
	
	dotTif	= ".bmp"; index	= tmpString.findRev((const char*)dotTif);
	if (index < 0) {
		dotTif = ".BMP"; index	= tmpString.findRev((const char*)dotTif);
			if (index < 0) {
		dotTif = ".png"; index	= tmpString.findRev((const char*)dotTif);
	if (index < 0) {
		dotTif = ".PNG"; index	= tmpString.findRev((const char*)dotTif);

    if (index < 0) {
		dotTif = ".gif"; index	= tmpString.findRev((const char*)dotTif);
    if (index < 0) {
			dotTif = ".GIF"; index	= tmpString.findRev((const char*)dotTif);
			if (index < 0) {
      dotTif = ".jpg"; index	= tmpString.findRev((const char*)dotTif);
			if (index < 0) {
				dotTif = ".JPG"; index	= tmpString.findRev((const char*)dotTif);
			if (index < 0) {
				dotTif = ".tga"; index	= tmpString.findRev((const char*)dotTif);
			if (index < 0) {
				dotTif = ".TGA"; index	= tmpString.findRev((const char*)dotTif);
			}
	}  }  } }  } } } }


	if (index < 4)
	{
		sprintf (errorMessage, "Invalid filename:\n\'%s\'\n\nLooking for something like\n\'file0000.tif\', \'file0000.TIF\', etc.", startfile);
		QMessageBox::information(this, APP_NAME, errorMessage);
		return -1;
	}

	tmpString.truncate(index);
	
	baseString = (const char*)tmpString;
	index = tmpString.length();

//ok all we need to do for now is figure out how many zeros on our own...
//later on look at the texture loading code which is much better
//as it deals with a wider variety of options by building the sequences
debug("ras is now:"+tmpString);

QChar ch; slen = 0;
int sl=tmpString.length();  sl-=1;

while(1) {
    ch = tmpString.at( sl );
    if (ch.isNumber()) { slen++; sl--; } else break;
    if (slen>6) break;

}
QString rere;
//debug("index is now:"+rere.setNum(slen));

   index -=slen;
	//index -= 4;                      //this is the number of zeros in the image filename
	baseString.truncate(index);
	tmpString.remove(0, index);
//debug("ras is now:"+tmpString);
	
	QString	shortName((const char*)baseString);
	int	slash = shortName.findRev('/');
	if(slash > 0)
		shortName.remove(0, slash+1);
	setCaption(shortName);
	
	startFrame = tmpString.toInt(&ok);
	if (!ok)
	{
		sprintf(errorMessage, "Invalid filename:\n\'%s\'\n\nFilename must have a 4 digit sequence number.\n(i.e. filename0000.tif)", startfile);
		QMessageBox::information(this, APP_NAME, errorMessage);
		return -1;
	}

	numFrames	= 0;
	// Check that the files exist, and figure out max frames to load
	// There has to be a better way to do this besides opening and closing
	// all these files.
	for (i = startFrame; i < (int)startFrame+maxFrames; i++)
	{
    //we will fix this later mon this is real lazy but we need to rewrite the enire routine to use videoData
		if (slen==2) { tmpString.sprintf("%s%02d%s", (const char*)baseString, i, (const char*)dotTif); }
		if (slen==3) { tmpString.sprintf("%s%03d%s", (const char*)baseString, i, (const char*)dotTif); }
		if (slen==4) { tmpString.sprintf("%s%04d%s", (const char*)baseString, i, (const char*)dotTif); }
		if (slen==5) { tmpString.sprintf("%s%05d%s", (const char*)baseString, i, (const char*)dotTif); }
		if (slen==6) { tmpString.sprintf("%s%06d%s", (const char*)baseString, i, (const char*)dotTif); }

		//debug("checking:"+tmpString);

		check = fopen((const char*)tmpString, "r");
		if (!check)
		{
			if (i == 0)
			{
				warning ("File \'%s\' not found. Sequence aborted.", (const char*)tmpString);
				return -1;
			}
			maxFrames = i-startFrame;
			break;
		}
		else
			fclose(check);
	}
	
	if ( ! (pixlist = new QPixmap*[maxFrames+1]) )
	{	
		OutOfMemory();
		return -1;
	}
	
	swapThresh = SWAP_THRESH;
	doMemCheck = getMemUsage(&begMem, &begSwap);
	fprintf(stderr, "begin: mem %3.1f%%, swap: %3.1f%%\n", begMem, begSwap);
	showMemInfo(true);
	
	loadFrame = startFrame;
	timer.start(0);
	
	return startFrame;
}
/*

//when we clean this up we need to clean up
//void	TiffFlip::LoadSingleImage() routine underneath as well as they share globals...

//int TiffFlip::LoadDesktopImages(const char *startfile, int max, int z)
int TiffFlip::LoadDesktopImages(videoData theclip, int max, int z)
{

	int		i; int		index; bool	ok;
	QString tmpString;
	FILE	*check;
	
	loadZoom		= z;
	maxFrames	= max;
	
	dotTif	= theclip.extension;


	
	QString baseString = clip.location+clip.filename;
	index = theclip.videoext.length(); //this is the number of zeros in the image filename
	
	QString	shortName((const char*)baseString);
	
	setCaption(clip.clipname);
	
	startFrame = clip.startframe;

	numFrames	= 0;
	// Check that the files exist, and figure out max frames to load
	// There has to be a better way to do this besides opening and closing
	// all these files.
	for (i = startFrame; i < startFrame+maxFrames; i++)
	{
		tmpString.sprintf("%s%04d%s", (const char*)baseString, i, (const char*)dotTif);
		check = fopen((const char*)tmpString, "r");
		if (!check)
		{
			if (i == 0)
			{
				warning ("File \'%s\' not found. Sequence aborted.", (const char*)tmpString);
				return -1;
			}
			maxFrames = i-startFrame;
			break;
		}
		else
			fclose(check);
	}
	
	if ( ! (pixlist = new QPixmap*[maxFrames+1]) )
	{	
		OutOfMemory();
		return -1;
	}
	
	swapThresh = SWAP_THRESH;
	doMemCheck = getMemUsage(&begMem, &begSwap);
	fprintf(stderr, "begin: mem %3.1f%%, swap: %3.1f%%\n", begMem, begSwap);
	showMemInfo(true);
	
	loadFrame = startFrame;
	timer.start(0);
	
	return startFrame;
}
*/


void TiffFlip::GetImages( QImage SentImage, int max, int z)
{
	maxFrames	= 1;	numFrames	= 1; startFrame=0; loadFrame = 0;
	
	if ( ! (pixlist = new QPixmap*[1]) )
	  {	 OutOfMemory(); return; }
	
	swapThresh = SWAP_THRESH;
	
  QImage  image;    image=SentImage;

  origWidth = width = SentImage.width();   // = w
  origHeight = height = SentImage.height();  // = h


			if ( ! (pixlist[0] = new QPixmap) )
				{ OutOfMemory(); return; }
		
			


			pixlist[0]->setOptimization(QPixmap::MemoryOptim);
			//pixlist[0]->setOptimization(QPixmap::NoOptim);

      if ( ! pixlist[0]->convertFromImage(image, Qt::ColorOnly | Qt::NoAlpha ) )
			  { OutOfMemory(); return; }


  mediaSize thesize;
  thesize.width = width * displayZoom;
  thesize.height = height * displayZoom;
  emit resizePlayer(thesize);

  setFixedSize(width*displayZoom, height*displayZoom);

  drawPixmap();		

  show();

  loadFrame=1;
  showMemInfo(false);
  return;
}
//--------------------------------------------------
void	TiffFlip::LoadController()
{

	if (loadFrame >= startFrame+maxFrames)
	{
		LoadFinished();
	}
	else
	{
		int frame = loadFrame-startFrame;
		emit LoadingFrame(frame);
		LoadSingleImage();
		SetFrame(frame);
		if(frame%10 == 0)
			checkMem();
		loadFrame++;
	}
}
	
//--------------------------------------------------
void	TiffFlip::LoadFinished()
{
	showMemInfo(false);
	timer.stop();
	emit Done(startFrame, false);
}

//--------------------------------------------------
void	TiffFlip::abortLoad()
{
	fprintf(stderr, "load aborted.\n");
	showMemInfo(false);
	timer.stop();
	numFrames = maxFrames = (loadFrame-startFrame);
	emit Done(startFrame, true);
}

//--------------------------------------------------
void	TiffFlip::LoadSingleImage()
{
  QString	tmpString;
  QImage	image;
debug("loading:"+baseString);

		if (slen==2) { tmpString.sprintf("%s%02d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }
		if (slen==3) { tmpString.sprintf("%s%03d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }
		if (slen==4) { tmpString.sprintf("%s%04d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }
		if (slen==5) { tmpString.sprintf("%s%05d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }
		if (slen==6) { tmpString.sprintf("%s%06d%s", (const char*)baseString, loadFrame, (const char*)dotTif); }

  //tmpString.sprintf("%s%04d%s", (const char*)baseString, loadFrame, (const char*)dotTif);

  QString format = dotTif.right( (dotTif.length() -1 ) );  //remove the period

  QImage  myimage;
  QPixmap tif;
  bool gothere = 0;

    ////////////////////////////////////////////
    //add tga support here

      //  tif.load( (const char*)tmpString, format.upper() );

   QImageIO iio; iio.setFileName( (const char*)tmpString );
    if ( tmpString.endsWith( ".tga" ) || tmpString.endsWith( ".TGA" )) { iio.setFormat("TGA"); }//
	else { /*iio.setFormat( format.upper() );*/ }

    if ( iio.read() ) {	image = iio.image(); gothere=1; } else { debug("null image "+tmpString); }


  // tif.load( (const char*)tmpString );
  // image = tif;

   if( !(image.isNull() ) ) {  gothere=1; } else { debug("null image "+tmpString); }

		if (gothere)
		{

     // origWidth = width = tif.width();
	//		origHeight = height = tif.height();
      origWidth = width = image.width();
			origHeight = height = image.height();

			if ( ! (pixlist[loadFrame-startFrame] = new QPixmap) )
				{ OutOfMemory(); return; }
				
			pixlist[loadFrame-startFrame]->setOptimization(QPixmap::NoOptim);

      if ( ! pixlist[loadFrame-startFrame]->convertFromImage(image) )
			  { OutOfMemory(); return; }

			image.reset();
			numFrames++;
			frame = loadFrame-startFrame;
			drawPixmap();
			
			//--------------------------
			// if first time in loop, set the size of window
			// and show it
			//--------------------------
			if (loadFrame == startFrame)
			{

  mediaSize thesize;
  thesize.width = width * displayZoom;
  thesize.height = height * displayZoom;
  emit resizePlayer(thesize);

				setFixedSize(width*displayZoom, height*displayZoom);
				show();
			}
		}
		else
		{
			abortLoad();
			sprintf (errorMessage, "Error reading " + format +" file\n\"%s\"\n\nSequence truncated to %d frames.", (const char*)tmpString, maxFrames);
			QMessageBox::warning(this, APP_NAME, errorMessage);
		}
		
		//Fix this to take zoom values
}

//------------------------------------------------
void	TiffFlip::OutOfMemory()
{
	timer.stop();
	sprintf (errorMessage, "ERROR: Could not allocate enough memory.\n(memory exhausted at frame %d)\n\nLoad Aborted", loadFrame-startFrame);
	Reset();	// free memory
	QMessageBox::critical(this, APP_NAME, errorMessage);
	emit Done(-1, true);
}
//------------------------------------------------
void	TiffFlip::SetFrame(int in)
{
	frame = in;
	drawPixmap();
}

//------------------------------------------------
void	TiffFlip::drawPixmap()
{
	static	QPixmap		tempPix;
	
	if (numFrames > 0 && frame >= 0 && frame < numFrames)
	{
		if (displayZoom > 1)
		{
			tempPix = pixlist[frame]->xForm(m);
			bitBlt(this, 0, 0, &tempPix, 0, 0, -1, -1);
			//bitBlt(this, 0, 0, &(pixlist[frame]->xForm(m)), 0, 0, -1, -1);
		}
		else
			bitBlt(this, 0, 0, pixlist[frame], 0, 0, -1, -1);
	
		frame ++;
		if (frame >= numFrames)
			frame = 0;
	}
}
	
//------------------------------------------------
void	TiffFlip::paintEvent(QPaintEvent * )
{
	drawPixmap();
}

//------------------------------------------------
bool	TiffFlip::checkMem()
{
	bool retVal = getMemUsage(&curMem, &curSwap);
	if(retVal)
	{
		memString.sprintf("mem: %3.1f%%", curMem);
		swapString.sprintf("swap: %3.1f%%", curSwap);
		memLabel->setText(memString);
		swapLabel->setText(swapString);
	}
	
	if(curSwap > begSwap)
	{
		float diff = curSwap-begSwap;
		if(diff > swapThresh)
		{
			timer.stop();
			QString mssg;
			mssg.sprintf(	"WARNING:\nswap file useage has increased by %3.1f%%.\n", diff);
			int but = QMessageBox::warning(this, APP_NAME, mssg, QMessageBox::Abort, QMessageBox::Ignore);
			if(but == QMessageBox::Ignore)
			{
				swapThresh += SWAP_THRESH;
				timer.start(0);
			}
			else
			{
				abortLoad();
			}
		}
	}
	return retVal;
}

/**
 * get percentage of memory and swap used
 */
//------------------------------------------------
bool	TiffFlip::getMemUsage(float* mem, float* swap)
{
	int c = 0;
	FILE *meminfo;
	
	uint swap_total	= 16;
	uint swap_used	= 0;
	uint swap_free	= 16;
	
	uint mem_total	= 16;
	uint mem_used	= 0;
	uint mem_free	= 16;
	uint mem_share	= 0;
	uint mem_buff	= 0;
	uint mem_cache	= 0;
	
	meminfo = fopen("/proc/meminfo", "r");
	if(meminfo)
	{
		char buffer[512];
		while(fgets(buffer, 512, meminfo) != NULL)
		{
			if( strncmp(buffer, "Swap:", 5) == 0 )
			{
				sscanf(buffer, "Swap: %u %u %u", &swap_total, &swap_used, &swap_free);
				c++;
			}
			else if( strncmp(buffer, "Mem:", 4) == 0 )
			{
				sscanf(buffer, "Mem: %u %u %u %u %u %u",
					&mem_total, &mem_used, &mem_free,
					&mem_share, &mem_buff, &mem_cache);
				c++;
			}
		}
		fclose(meminfo);
	}
	float mb = 1024.0*1024.0;
	float mTotal = (float)mem_total/mb;
	float mUsed  = (float)(mem_used-(mem_share+mem_buff+mem_cache))/mb;
	*mem = 100.0f*(mUsed/mTotal);
	float sTotal = (float)swap_total/mb;
	float sUsed  = (float)swap_used/mb;
	*swap = 100.0f*(sUsed/sTotal);
	
	if(c == 2)
		return true;
	else
		return false;
}

/**
 * useless. Memory is allocated in X, not this process. Doh.
 */
//------------------------------------------------
void	TiffFlip::checkProcessMemory()
{
	FILE*	file = 0;
	char	filename[128];
	char	statline[1024];
	uint	vsize;
	snprintf(filename, 128, "/proc/%d/stat", flipPID);
	file = fopen(filename, "r");
	fgets(statline, 1024, file);
	fclose(file);
	statline[1023] = '\0';
	sscanf(statline,
		"%*d %*s %*c %*d " //		  pid,  	comm,	 state, 	  ppid
		"%*d %*d %*d %*d " //		 pgrp,   session,	   tty, 	 tpgid
		"%*u %*u %*u %*u " //		flags,    minflt,  cminflt, 	majflt
 		"%*u %*d %*d %*d " //	  cmajflt,     utime,	 stime, 	cutime
		"%*d %*d %*d %*u " //	   cstime,   counter, priority,    timeout
		"%*u %*d  %u %*u "  // itrealvalue,  stattime,	 vsize, 	   rss
		"%*u %*u %*u %*u " //		 rlim, startcode,  endcode, startstack
		"%*u %*u %*d %*d " //	  kstkesp,   kstkeip,	signal,    blocked
		"%*d %*d %*u",     //   sigignore,  sigcatch,    wchan
		&vsize);
}





